﻿var pas = {};

var rtl = {

  version: 10501,

  quiet: false,
  debug_load_units: false,
  debug_rtti: false,

  debug: function(){
    if (rtl.quiet || !console || !console.log) return;
    console.log(arguments);
  },

  error: function(s){
    rtl.debug('Error: ',s);
    throw s;
  },

  warn: function(s){
    rtl.debug('Warn: ',s);
  },

  checkVersion: function(v){
    if (rtl.version != v) throw "expected rtl version "+v+", but found "+rtl.version;
  },

  hiInt: Math.pow(2,53),

  hasString: function(s){
    return rtl.isString(s) && (s.length>0);
  },

  isArray: function(a) {
    return Array.isArray(a);
  },

  isFunction: function(f){
    return typeof(f)==="function";
  },

  isModule: function(m){
    return rtl.isObject(m) && rtl.hasString(m.$name) && (pas[m.$name]===m);
  },

  isImplementation: function(m){
    return rtl.isObject(m) && rtl.isModule(m.$module) && (m.$module.$impl===m);
  },

  isNumber: function(n){
    return typeof(n)==="number";
  },

  isObject: function(o){
    var s=typeof(o);
    return (typeof(o)==="object") && (o!=null);
  },

  isString: function(s){
    return typeof(s)==="string";
  },

  getNumber: function(n){
    return typeof(n)==="number"?n:NaN;
  },

  getChar: function(c){
    return ((typeof(c)==="string") && (c.length===1)) ? c : "";
  },

  getObject: function(o){
    return ((typeof(o)==="object") || (typeof(o)==='function')) ? o : null;
  },

  isTRecord: function(type){
    return (rtl.isObject(type) && type.hasOwnProperty('$new') && (typeof(type.$new)==='function'));
  },

  isPasClass: function(type){
    return (rtl.isObject(type) && type.hasOwnProperty('$classname') && rtl.isObject(type.$module));
  },

  isPasClassInstance: function(type){
    return (rtl.isObject(type) && rtl.isPasClass(type.$class));
  },

  hexStr: function(n,digits){
    return ("000000000000000"+n.toString(16).toUpperCase()).slice(-digits);
  },

  m_loading: 0,
  m_loading_intf: 1,
  m_intf_loaded: 2,
  m_loading_impl: 3, // loading all used unit
  m_initializing: 4, // running initialization
  m_initialized: 5,

  module: function(module_name, intfuseslist, intfcode, impluseslist, implcode){
    if (rtl.debug_load_units) rtl.debug('rtl.module name="'+module_name+'" intfuses='+intfuseslist+' impluses='+impluseslist+' hasimplcode='+rtl.isFunction(implcode));
    if (!rtl.hasString(module_name)) rtl.error('invalid module name "'+module_name+'"');
    if (!rtl.isArray(intfuseslist)) rtl.error('invalid interface useslist of "'+module_name+'"');
    if (!rtl.isFunction(intfcode)) rtl.error('invalid interface code of "'+module_name+'"');
    if (!(impluseslist==undefined) && !rtl.isArray(impluseslist)) rtl.error('invalid implementation useslist of "'+module_name+'"');
    if (!(implcode==undefined) && !rtl.isFunction(implcode)) rtl.error('invalid implementation code of "'+module_name+'"');

    if (pas[module_name])
      rtl.error('module "'+module_name+'" is already registered');

    var module = pas[module_name] = {
      $name: module_name,
      $intfuseslist: intfuseslist,
      $impluseslist: impluseslist,
      $state: rtl.m_loading,
      $intfcode: intfcode,
      $implcode: implcode,
      $impl: null,
      $rtti: Object.create(rtl.tSectionRTTI)
    };
    module.$rtti.$module = module;
    if (implcode) module.$impl = {
      $module: module,
      $rtti: module.$rtti
    };
  },

  exitcode: 0,

  run: function(module_name){
  
    function doRun(){
      if (!rtl.hasString(module_name)) module_name='program';
      if (rtl.debug_load_units) rtl.debug('rtl.run module="'+module_name+'"');
      rtl.initRTTI();
      var module = pas[module_name];
      if (!module) rtl.error('rtl.run module "'+module_name+'" missing');
      rtl.loadintf(module);
      rtl.loadimpl(module);
      if (module_name=='program'){
        if (rtl.debug_load_units) rtl.debug('running $main');
        var r = pas.program.$main();
        if (rtl.isNumber(r)) rtl.exitcode = r;
      }
    }
    
    if (rtl.showUncaughtExceptions) {
      try{
        doRun();
      } catch(re) {
        var errMsg = rtl.hasString(re.$classname) ? re.$classname : '';
	    errMsg +=  ((errMsg) ? ': ' : '') + (re.hasOwnProperty('fMessage') ? re.fMessage : re);
        alert('Uncaught Exception : '+errMsg);
        rtl.exitCode = 216;
      }
    } else {
      doRun();
    }
    return rtl.exitcode;
  },

  loadintf: function(module){
    if (module.$state>rtl.m_loading_intf) return; // already finished
    if (rtl.debug_load_units) rtl.debug('loadintf: "'+module.$name+'"');
    if (module.$state===rtl.m_loading_intf)
      rtl.error('unit cycle detected "'+module.$name+'"');
    module.$state=rtl.m_loading_intf;
    // load interfaces of interface useslist
    rtl.loaduseslist(module,module.$intfuseslist,rtl.loadintf);
    // run interface
    if (rtl.debug_load_units) rtl.debug('loadintf: run intf of "'+module.$name+'"');
    module.$intfcode(module.$intfuseslist);
    // success
    module.$state=rtl.m_intf_loaded;
    // Note: units only used in implementations are not yet loaded (not even their interfaces)
  },

  loaduseslist: function(module,useslist,f){
    if (useslist==undefined) return;
    for (var i in useslist){
      var unitname=useslist[i];
      if (rtl.debug_load_units) rtl.debug('loaduseslist of "'+module.$name+'" uses="'+unitname+'"');
      if (pas[unitname]==undefined)
        rtl.error('module "'+module.$name+'" misses "'+unitname+'"');
      f(pas[unitname]);
    }
  },

  loadimpl: function(module){
    if (module.$state>=rtl.m_loading_impl) return; // already processing
    if (module.$state<rtl.m_intf_loaded) rtl.error('loadimpl: interface not loaded of "'+module.$name+'"');
    if (rtl.debug_load_units) rtl.debug('loadimpl: load uses of "'+module.$name+'"');
    module.$state=rtl.m_loading_impl;
    // load interfaces of implementation useslist
    rtl.loaduseslist(module,module.$impluseslist,rtl.loadintf);
    // load implementation of interfaces useslist
    rtl.loaduseslist(module,module.$intfuseslist,rtl.loadimpl);
    // load implementation of implementation useslist
    rtl.loaduseslist(module,module.$impluseslist,rtl.loadimpl);
    // Note: At this point all interfaces used by this unit are loaded. If
    //   there are implementation uses cycles some used units might not yet be
    //   initialized. This is by design.
    // run implementation
    if (rtl.debug_load_units) rtl.debug('loadimpl: run impl of "'+module.$name+'"');
    if (rtl.isFunction(module.$implcode)) module.$implcode(module.$impluseslist);
    // run initialization
    if (rtl.debug_load_units) rtl.debug('loadimpl: run init of "'+module.$name+'"');
    module.$state=rtl.m_initializing;
    if (rtl.isFunction(module.$init)) module.$init();
    // unit initialized
    module.$state=rtl.m_initialized;
  },

  createCallback: function(scope, fn){
    var cb;
    if (typeof(fn)==='string'){
      cb = function(){
        return scope[fn].apply(scope,arguments);
      };
    } else {
      cb = function(){
        return fn.apply(scope,arguments);
      };
    };
    cb.scope = scope;
    cb.fn = fn;
    return cb;
  },

  cloneCallback: function(cb){
    return rtl.createCallback(cb.scope,cb.fn);
  },

  eqCallback: function(a,b){
    // can be a function or a function wrapper
    if (a==b){
      return true;
    } else {
      return (a!=null) && (b!=null) && (a.fn) && (a.scope===b.scope) && (a.fn==b.fn);
    }
  },

  initStruct: function(c,parent,name){
    if ((parent.$module) && (parent.$module.$impl===parent)) parent=parent.$module;
    c.$parent = parent;
    if (rtl.isModule(parent)){
      c.$module = parent;
      c.$name = name;
    } else {
      c.$module = parent.$module;
      c.$name = parent.$name+'.'+name;
    };
    return parent;
  },

  initClass: function(c,parent,name,initfn){
    parent[name] = c;
    c.$class = c; // Note: o.$class === Object.getPrototypeOf(o)
    c.$classname = name;
    parent = rtl.initStruct(c,parent,name);
    c.$fullname = parent.$name+'.'+name;
    // rtti
    if (rtl.debug_rtti) rtl.debug('initClass '+c.$fullname);
    var t = c.$module.$rtti.$Class(c.$name,{ "class": c });
    c.$rtti = t;
    if (rtl.isObject(c.$ancestor)) t.ancestor = c.$ancestor.$rtti;
    if (!t.ancestor) t.ancestor = null;
    // init members
    initfn.call(c);
  },

  createClass: function(parent,name,ancestor,initfn){
    // create a normal class,
    // ancestor must be null or a normal class,
    // the root ancestor can be an external class
    var c = null;
    if (ancestor != null){
      c = Object.create(ancestor);
      c.$ancestor = ancestor;
      // Note:
      // if root is an "object" then c.$ancestor === Object.getPrototypeOf(c)
      // if root is a "function" then c.$ancestor === c.__proto__, Object.getPrototypeOf(c) returns the root
    } else {
      c = { $ancestor: null };
      c.$create = function(fn,args){
        if (args == undefined) args = [];
        var o = Object.create(this);
        o.$init();
        try{
          if (typeof(fn)==="string"){
            o[fn].apply(o,args);
          } else {
            fn.apply(o,args);
          };
          o.AfterConstruction();
        } catch($e){
          // do not call BeforeDestruction
          if (o.Destroy) o.Destroy();
          o.$final();
          throw $e;
        }
        return o;
      };
      c.$destroy = function(fnname){
        this.BeforeDestruction();
        if (this[fnname]) this[fnname]();
        this.$final();
      };
    };
    rtl.initClass(c,parent,name,initfn);
  },

  createClassExt: function(parent,name,ancestor,newinstancefnname,initfn){
    // Create a class using an external ancestor.
    // If newinstancefnname is given, use that function to create the new object.
    // If exist call BeforeDestruction and AfterConstruction.
    var c = Object.create(ancestor);
    c.$create = function(fn,args){
      if (args == undefined) args = [];
      var o = null;
      if (newinstancefnname.length>0){
        o = this[newinstancefnname](fn,args);
      } else {
        o = Object.create(this);
      }
      if (o.$init) o.$init();
      try{
        if (typeof(fn)==="string"){
          o[fn].apply(o,args);
        } else {
          fn.apply(o,args);
        };
        if (o.AfterConstruction) o.AfterConstruction();
      } catch($e){
        // do not call BeforeDestruction
        if (o.Destroy) o.Destroy();
        if (o.$final) this.$final();
        throw $e;
      }
      return o;
    };
    c.$destroy = function(fnname){
      if (this.BeforeDestruction) this.BeforeDestruction();
      if (this[fnname]) this[fnname]();
      if (this.$final) this.$final();
    };
    rtl.initClass(c,parent,name,initfn);
  },

  createHelper: function(parent,name,ancestor,initfn){
    // create a helper,
    // ancestor must be null or a helper,
    var c = null;
    if (ancestor != null){
      c = Object.create(ancestor);
      c.$ancestor = ancestor;
      // c.$ancestor === Object.getPrototypeOf(c)
    } else {
      c = { $ancestor: null };
    };
    parent[name] = c;
    c.$class = c; // Note: o.$class === Object.getPrototypeOf(o)
    c.$classname = name;
    parent = rtl.initStruct(c,parent,name);
    c.$fullname = parent.$name+'.'+name;
    // rtti
    var t = c.$module.$rtti.$Helper(c.$name,{ "helper": c });
    c.$rtti = t;
    if (rtl.isObject(ancestor)) t.ancestor = ancestor.$rtti;
    if (!t.ancestor) t.ancestor = null;
    // init members
    initfn.call(c);
  },

  tObjectDestroy: "Destroy",

  free: function(obj,name){
    if (obj[name]==null) return null;
    obj[name].$destroy(rtl.tObjectDestroy);
    obj[name]=null;
  },

  freeLoc: function(obj){
    if (obj==null) return null;
    obj.$destroy(rtl.tObjectDestroy);
    return null;
  },

  recNewT: function(parent,name,initfn,full){
    // create new record type
    var t = {};
    if (parent) parent[name] = t;
    function hide(prop){
      Object.defineProperty(t,prop,{enumerable:false});
    }
    if (full){
      rtl.initStruct(t,parent,name);
      t.$record = t;
      hide('$record');
      hide('$name');
      hide('$parent');
      hide('$module');
    }
    initfn.call(t);
    if (!t.$new){
      t.$new = function(){ return Object.create(this); };
    }
    t.$clone = function(r){ return this.$new().$assign(r); };
    hide('$new');
    hide('$clone');
    hide('$eq');
    hide('$assign');
    return t;
  },

  is: function(instance,type){
    return type.isPrototypeOf(instance) || (instance===type);
  },

  isExt: function(instance,type,mode){
    // mode===1 means instance must be a Pascal class instance
    // mode===2 means instance must be a Pascal class
    // Notes:
    // isPrototypeOf and instanceof return false on equal
    // isPrototypeOf does not work for Date.isPrototypeOf(new Date())
    //   so if isPrototypeOf is false test with instanceof
    // instanceof needs a function on right side
    if (instance == null) return false; // Note: ==null checks for undefined too
    if ((typeof(type) !== 'object') && (typeof(type) !== 'function')) return false;
    if (instance === type){
      if (mode===1) return false;
      if (mode===2) return rtl.isPasClass(instance);
      return true;
    }
    if (type.isPrototypeOf && type.isPrototypeOf(instance)){
      if (mode===1) return rtl.isPasClassInstance(instance);
      if (mode===2) return rtl.isPasClass(instance);
      return true;
    }
    if ((typeof type == 'function') && (instance instanceof type)) return true;
    return false;
  },

  Exception: null,
  EInvalidCast: null,
  EAbstractError: null,
  ERangeError: null,
  EIntOverflow: null,
  EPropWriteOnly: null,

  raiseE: function(typename){
    var t = rtl[typename];
    if (t==null){
      var mod = pas.SysUtils;
      if (!mod) mod = pas.sysutils;
      if (mod){
        t = mod[typename];
        if (!t) t = mod[typename.toLowerCase()];
        if (!t) t = mod['Exception'];
        if (!t) t = mod['exception'];
      }
    }
    if (t){
      if (t.Create){
        throw t.$create("Create");
      } else if (t.create){
        throw t.$create("create");
      }
    }
    if (typename === "EInvalidCast") throw "invalid type cast";
    if (typename === "EAbstractError") throw "Abstract method called";
    if (typename === "ERangeError") throw "range error";
    throw typename;
  },

  as: function(instance,type){
    if((instance === null) || rtl.is(instance,type)) return instance;
    rtl.raiseE("EInvalidCast");
  },

  asExt: function(instance,type,mode){
    if((instance === null) || rtl.isExt(instance,type,mode)) return instance;
    rtl.raiseE("EInvalidCast");
  },

  createInterface: function(module, name, guid, fnnames, ancestor, initfn){
    //console.log('createInterface name="'+name+'" guid="'+guid+'" names='+fnnames);
    var i = ancestor?Object.create(ancestor):{};
    module[name] = i;
    i.$module = module;
    i.$name = name;
    i.$fullname = module.$name+'.'+name;
    i.$guid = guid;
    i.$guidr = null;
    i.$names = fnnames?fnnames:[];
    if (rtl.isFunction(initfn)){
      // rtti
      if (rtl.debug_rtti) rtl.debug('createInterface '+i.$fullname);
      var t = i.$module.$rtti.$Interface(name,{ "interface": i, module: module });
      i.$rtti = t;
      if (ancestor) t.ancestor = ancestor.$rtti;
      if (!t.ancestor) t.ancestor = null;
      initfn.call(i);
    }
    return i;
  },

  strToGUIDR: function(s,g){
    var p = 0;
    function n(l){
      var h = s.substr(p,l);
      p+=l;
      return parseInt(h,16);
    }
    p+=1; // skip {
    g.D1 = n(8);
    p+=1; // skip -
    g.D2 = n(4);
    p+=1; // skip -
    g.D3 = n(4);
    p+=1; // skip -
    if (!g.D4) g.D4=[];
    g.D4[0] = n(2);
    g.D4[1] = n(2);
    p+=1; // skip -
    for(var i=2; i<8; i++) g.D4[i] = n(2);
    return g;
  },

  guidrToStr: function(g){
    if (g.$intf) return g.$intf.$guid;
    var h = rtl.hexStr;
    var s='{'+h(g.D1,8)+'-'+h(g.D2,4)+'-'+h(g.D3,4)+'-'+h(g.D4[0],2)+h(g.D4[1],2)+'-';
    for (var i=2; i<8; i++) s+=h(g.D4[i],2);
    s+='}';
    return s;
  },

  createTGUID: function(guid){
    var TGuid = (pas.System)?pas.System.TGuid:pas.system.tguid;
    var g = rtl.strToGUIDR(guid,TGuid.$new());
    return g;
  },

  getIntfGUIDR: function(intfTypeOrVar){
    if (!intfTypeOrVar) return null;
    if (!intfTypeOrVar.$guidr){
      var g = rtl.createTGUID(intfTypeOrVar.$guid);
      if (!intfTypeOrVar.hasOwnProperty('$guid')) intfTypeOrVar = Object.getPrototypeOf(intfTypeOrVar);
      g.$intf = intfTypeOrVar;
      intfTypeOrVar.$guidr = g;
    }
    return intfTypeOrVar.$guidr;
  },

  addIntf: function (aclass, intf, map){
    function jmp(fn){
      if (typeof(fn)==="function"){
        return function(){ return fn.apply(this.$o,arguments); };
      } else {
        return function(){ rtl.raiseE('EAbstractError'); };
      }
    }
    if(!map) map = {};
    var t = intf;
    var item = Object.create(t);
    if (!aclass.hasOwnProperty('$intfmaps')) aclass.$intfmaps = {};
    aclass.$intfmaps[intf.$guid] = item;
    do{
      var names = t.$names;
      if (!names) break;
      for (var i=0; i<names.length; i++){
        var intfname = names[i];
        var fnname = map[intfname];
        if (!fnname) fnname = intfname;
        //console.log('addIntf: intftype='+t.$name+' index='+i+' intfname="'+intfname+'" fnname="'+fnname+'" old='+typeof(item[intfname]));
        item[intfname] = jmp(aclass[fnname]);
      }
      t = Object.getPrototypeOf(t);
    }while(t!=null);
  },

  getIntfG: function (obj, guid, query){
    if (!obj) return null;
    //console.log('getIntfG: obj='+obj.$classname+' guid='+guid+' query='+query);
    // search
    var maps = obj.$intfmaps;
    if (!maps) return null;
    var item = maps[guid];
    if (!item) return null;
    // check delegation
    //console.log('getIntfG: obj='+obj.$classname+' guid='+guid+' query='+query+' item='+typeof(item));
    if (typeof item === 'function') return item.call(obj); // delegate. Note: COM contains _AddRef
    // check cache
    var intf = null;
    if (obj.$interfaces){
      intf = obj.$interfaces[guid];
      //console.log('getIntfG: obj='+obj.$classname+' guid='+guid+' cache='+typeof(intf));
    }
    if (!intf){ // intf can be undefined!
      intf = Object.create(item);
      intf.$o = obj;
      if (!obj.$interfaces) obj.$interfaces = {};
      obj.$interfaces[guid] = intf;
    }
    if (typeof(query)==='object'){
      // called by queryIntfT
      var o = null;
      if (intf.QueryInterface(rtl.getIntfGUIDR(query),
          {get:function(){ return o; }, set:function(v){ o=v; }}) === 0){
        return o;
      } else {
        return null;
      }
    } else if(query===2){
      // called by TObject.GetInterfaceByStr
      if (intf.$kind === 'com') intf._AddRef();
    }
    return intf;
  },

  getIntfT: function(obj,intftype){
    return rtl.getIntfG(obj,intftype.$guid);
  },

  queryIntfT: function(obj,intftype){
    return rtl.getIntfG(obj,intftype.$guid,intftype);
  },

  queryIntfIsT: function(obj,intftype){
    var i = rtl.queryIntfG(obj,intftype.$guid);
    if (!i) return false;
    if (i.$kind === 'com') i._Release();
    return true;
  },

  asIntfT: function (obj,intftype){
    var i = rtl.getIntfG(obj,intftype.$guid);
    if (i!==null) return i;
    rtl.raiseEInvalidCast();
  },

  intfIsClass: function(intf,classtype){
    return (intf!=null) && (rtl.is(intf.$o,classtype));
  },

  intfAsClass: function(intf,classtype){
    if (intf==null) return null;
    return rtl.as(intf.$o,classtype);
  },

  intfToClass: function(intf,classtype){
    if ((intf!==null) && rtl.is(intf.$o,classtype)) return intf.$o;
    return null;
  },

  // interface reference counting
  intfRefs: { // base object for temporary interface variables
    ref: function(id,intf){
      // called for temporary interface references needing delayed release
      var old = this[id];
      //console.log('rtl.intfRefs.ref: id='+id+' old="'+(old?old.$name:'null')+'" intf="'+(intf?intf.$name:'null')+' $o='+(intf?intf.$o:'null'));
      if (old){
        // called again, e.g. in a loop
        delete this[id];
        old._Release(); // may fail
      }
      this[id]=intf;
      return intf;
    },
    free: function(){
      //console.log('rtl.intfRefs.free...');
      for (var id in this){
        if (this.hasOwnProperty(id)){
          //console.log('rtl.intfRefs.free: id='+id+' '+this[id].$name+' $o='+this[id].$o.$classname);
          this[id]._Release();
        }
      }
    }
  },

  createIntfRefs: function(){
    //console.log('rtl.createIntfRefs');
    return Object.create(rtl.intfRefs);
  },

  setIntfP: function(path,name,value,skipAddRef){
    var old = path[name];
    //console.log('rtl.setIntfP path='+path+' name='+name+' old="'+(old?old.$name:'null')+'" value="'+(value?value.$name:'null')+'"');
    if (old === value) return;
    if (old !== null){
      path[name]=null;
      old._Release();
    }
    if (value !== null){
      if (!skipAddRef) value._AddRef();
      path[name]=value;
    }
  },

  setIntfL: function(old,value,skipAddRef){
    //console.log('rtl.setIntfL old="'+(old?old.$name:'null')+'" value="'+(value?value.$name:'null')+'"');
    if (old !== value){
      if (value!==null){
        if (!skipAddRef) value._AddRef();
      }
      if (old!==null){
        old._Release();  // Release after AddRef, to avoid double Release if Release creates an exception
      }
    } else if (skipAddRef){
      if (old!==null){
        old._Release();  // value has an AddRef
      }
    }
    return value;
  },

  _AddRef: function(intf){
    //if (intf) console.log('rtl._AddRef intf="'+(intf?intf.$name:'null')+'"');
    if (intf) intf._AddRef();
    return intf;
  },

  _Release: function(intf){
    //if (intf) console.log('rtl._Release intf="'+(intf?intf.$name:'null')+'"');
    if (intf) intf._Release();
    return intf;
  },

  checkMethodCall: function(obj,type){
    if (rtl.isObject(obj) && rtl.is(obj,type)) return;
    rtl.raiseE("EInvalidCast");
  },

  oc: function(i){
    // overflow check integer
    if ((Math.floor(i)===i) && (i>=-0x1fffffffffffff) && (i<=0x1fffffffffffff)) return i;
    rtl.raiseE('EIntOverflow');
  },

  rc: function(i,minval,maxval){
    // range check integer
    if ((Math.floor(i)===i) && (i>=minval) && (i<=maxval)) return i;
    rtl.raiseE('ERangeError');
  },

  rcc: function(c,minval,maxval){
    // range check char
    if ((typeof(c)==='string') && (c.length===1)){
      var i = c.charCodeAt(0);
      if ((i>=minval) && (i<=maxval)) return c;
    }
    rtl.raiseE('ERangeError');
  },

  rcSetCharAt: function(s,index,c){
    // range check setCharAt
    if ((typeof(s)!=='string') || (index<0) || (index>=s.length)) rtl.raiseE('ERangeError');
    return rtl.setCharAt(s,index,c);
  },

  rcCharAt: function(s,index){
    // range check charAt
    if ((typeof(s)!=='string') || (index<0) || (index>=s.length)) rtl.raiseE('ERangeError');
    return s.charAt(index);
  },

  rcArrR: function(arr,index){
    // range check read array
    if (Array.isArray(arr) && (typeof(index)==='number') && (index>=0) && (index<arr.length)){
      if (arguments.length>2){
        // arr,index1,index2,...
        arr=arr[index];
        for (var i=2; i<arguments.length; i++) arr=rtl.rcArrR(arr,arguments[i]);
        return arr;
      }
      return arr[index];
    }
    rtl.raiseE('ERangeError');
  },

  rcArrW: function(arr,index,value){
    // range check write array
    // arr,index1,index2,...,value
    for (var i=3; i<arguments.length; i++){
      arr=rtl.rcArrR(arr,index);
      index=arguments[i-1];
      value=arguments[i];
    }
    if (Array.isArray(arr) && (typeof(index)==='number') && (index>=0) && (index<arr.length)){
      return arr[index]=value;
    }
    rtl.raiseE('ERangeError');
  },

  length: function(arr){
    return (arr == null) ? 0 : arr.length;
  },

  arraySetLength: function(arr,defaultvalue,newlength){
    // multi dim: (arr,defaultvalue,dim1,dim2,...)
    var p = arguments;
    function setLength(src,argNo){
      var newlen = p[argNo];
      var a = [];
      a.length = newlength;
      if (argNo === p.length-1){
        var oldlen = src?src.length:0;
        if (rtl.isArray(defaultvalue)){
          for (var i=0; i<newlen; i++) a[i]=(i<oldlen)?src[i]:[]; // array of dyn array
        } else if (rtl.isObject(defaultvalue)) {
          if (rtl.isTRecord(defaultvalue)){
            for (var i=0; i<newlen; i++)
              a[i]=(i<oldlen)?defaultvalue.$clone(src[i]):defaultvalue.$new(); // e.g. record
          } else {
            for (var i=0; i<newlen; i++) a[i]=(i<oldlen)?rtl.refSet(src[i]):{}; // e.g. set
          }
        } else {
          for (var i=0; i<newlen; i++)
            a[i]=(i<oldlen)?src[i]:defaultvalue;
        }
      } else {
        // multi dim array
        for (var i=0; i<newlen; i++) a[i]=setLength(src?src[i]:null,argNo+1);
      }
      return a;
    }
    return setLength(arr,2);
  },

  /*arrayChgLength: function(arr,defaultvalue,newlength){
    // multi dim: (arr,defaultvalue,dim1,dim2,...)
    if (arr == null) arr = [];
    var p = arguments;
    function setLength(a,argNo){
      var oldlen = a.length;
      var newlen = p[argNo];
      if (oldlen!==newlength){
        a.length = newlength;
        if (argNo === p.length-1){
          if (rtl.isArray(defaultvalue)){
            for (var i=oldlen; i<newlen; i++) a[i]=[]; // nested array
          } else if (rtl.isObject(defaultvalue)) {
            if (rtl.isTRecord(defaultvalue)){
              for (var i=oldlen; i<newlen; i++) a[i]=defaultvalue.$new(); // e.g. record
            } else {
              for (var i=oldlen; i<newlen; i++) a[i]={}; // e.g. set
            }
          } else {
            for (var i=oldlen; i<newlen; i++) a[i]=defaultvalue;
          }
        } else {
          for (var i=oldlen; i<newlen; i++) a[i]=[]; // nested array
        }
      }
      if (argNo < p.length-1){
        // multi argNo
        for (var i=0; i<newlen; i++) a[i]=setLength(a[i],argNo+1);
      }
      return a;
    }
    return setLength(arr,2);
  },*/

  arrayEq: function(a,b){
    if (a===null) return b===null;
    if (b===null) return false;
    if (a.length!==b.length) return false;
    for (var i=0; i<a.length; i++) if (a[i]!==b[i]) return false;
    return true;
  },

  arrayClone: function(type,src,srcpos,endpos,dst,dstpos){
    // type: 0 for references, "refset" for calling refSet(), a function for new type()
    // src must not be null
    // This function does not range check.
    if(type === 'refSet') {
      for (; srcpos<endpos; srcpos++) dst[dstpos++] = rtl.refSet(src[srcpos]); // ref set
    } else if (rtl.isTRecord(type)){
      for (; srcpos<endpos; srcpos++) dst[dstpos++] = type.$clone(src[srcpos]); // clone record
    }  else {
      for (; srcpos<endpos; srcpos++) dst[dstpos++] = src[srcpos]; // reference
    };
  },

  arrayConcat: function(type){
    // type: see rtl.arrayClone
    var a = [];
    var l = 0;
    for (var i=1; i<arguments.length; i++){
      var src = arguments[i];
      if (src !== null) l+=src.length;
    };
    a.length = l;
    l=0;
    for (var i=1; i<arguments.length; i++){
      var src = arguments[i];
      if (src === null) continue;
      rtl.arrayClone(type,src,0,src.length,a,l);
      l+=src.length;
    };
    return a;
  },

  arrayConcatN: function(){
    var a = null;
    for (var i=1; i<arguments.length; i++){
      var src = arguments[i];
      if (src === null) continue;
      if (a===null){
        a=src; // Note: concat(a) does not clone
      } else {
        a=a.concat(src);
      }
    };
    return a;
  },

  arrayCopy: function(type, srcarray, index, count){
    // type: see rtl.arrayClone
    // if count is missing, use srcarray.length
    if (srcarray === null) return [];
    if (index < 0) index = 0;
    if (count === undefined) count=srcarray.length;
    var end = index+count;
    if (end>srcarray.length) end = srcarray.length;
    if (index>=end) return [];
    if (type===0){
      return srcarray.slice(index,end);
    } else {
      var a = [];
      a.length = end-index;
      rtl.arrayClone(type,srcarray,index,end,a,0);
      return a;
    }
  },

  setCharAt: function(s,index,c){
    return s.substr(0,index)+c+s.substr(index+1);
  },

  getResStr: function(mod,name){
    var rs = mod.$resourcestrings[name];
    return rs.current?rs.current:rs.org;
  },

  createSet: function(){
    var s = {};
    for (var i=0; i<arguments.length; i++){
      if (arguments[i]!=null){
        s[arguments[i]]=true;
      } else {
        var first=arguments[i+=1];
        var last=arguments[i+=1];
        for(var j=first; j<=last; j++) s[j]=true;
      }
    }
    return s;
  },

  cloneSet: function(s){
    var r = {};
    for (var key in s) r[key]=true;
    return r;
  },

  refSet: function(s){
    Object.defineProperty(s, '$shared', {
      enumerable: false,
      configurable: true,
      writable: true,
      value: true
    });
    return s;
  },

  includeSet: function(s,enumvalue){
    if (s.$shared) s = rtl.cloneSet(s);
    s[enumvalue] = true;
    return s;
  },

  excludeSet: function(s,enumvalue){
    if (s.$shared) s = rtl.cloneSet(s);
    delete s[enumvalue];
    return s;
  },

  diffSet: function(s,t){
    var r = {};
    for (var key in s) if (!t[key]) r[key]=true;
    return r;
  },

  unionSet: function(s,t){
    var r = {};
    for (var key in s) r[key]=true;
    for (var key in t) r[key]=true;
    return r;
  },

  intersectSet: function(s,t){
    var r = {};
    for (var key in s) if (t[key]) r[key]=true;
    return r;
  },

  symDiffSet: function(s,t){
    var r = {};
    for (var key in s) if (!t[key]) r[key]=true;
    for (var key in t) if (!s[key]) r[key]=true;
    return r;
  },

  eqSet: function(s,t){
    for (var key in s) if (!t[key]) return false;
    for (var key in t) if (!s[key]) return false;
    return true;
  },

  neSet: function(s,t){
    return !rtl.eqSet(s,t);
  },

  leSet: function(s,t){
    for (var key in s) if (!t[key]) return false;
    return true;
  },

  geSet: function(s,t){
    for (var key in t) if (!s[key]) return false;
    return true;
  },

  strSetLength: function(s,newlen){
    var oldlen = s.length;
    if (oldlen > newlen){
      return s.substring(0,newlen);
    } else if (s.repeat){
      // Note: repeat needs ECMAScript6!
      return s+' '.repeat(newlen-oldlen);
    } else {
       while (oldlen<newlen){
         s+=' ';
         oldlen++;
       };
       return s;
    }
  },

  spaceLeft: function(s,width){
    var l=s.length;
    if (l>=width) return s;
    if (s.repeat){
      // Note: repeat needs ECMAScript6!
      return ' '.repeat(width-l) + s;
    } else {
      while (l<width){
        s=' '+s;
        l++;
      };
    };
  },

  floatToStr: function(d,w,p){
    // input 1-3 arguments: double, width, precision
    if (arguments.length>2){
      return rtl.spaceLeft(d.toFixed(p),w);
    } else {
	  // exponent width
	  var pad = "";
	  var ad = Math.abs(d);
	  if (ad<1.0e+10) {
		pad='00';
	  } else if (ad<1.0e+100) {
		pad='0';
      }  	
	  if (arguments.length<2) {
	    w=9;		
      } else if (w<9) {
		w=9;
      }		  
      var p = w-8;
      var s=(d>0 ? " " : "" ) + d.toExponential(p);
      s=s.replace(/e(.)/,'E$1'+pad);
      return rtl.spaceLeft(s,w);
    }
  },

  valEnum: function(s, enumType, setCodeFn){
    s = s.toLowerCase();
    for (var key in enumType){
      if((typeof(key)==='string') && (key.toLowerCase()===s)){
        setCodeFn(0);
        return enumType[key];
      }
    }
    setCodeFn(1);
    return 0;
  },

  and: function(a,b){
    var hi = 0x80000000;
    var low = 0x7fffffff;
    var h = (a / hi) & (b / hi);
    var l = (a & low) & (b & low);
    return h*hi + l;
  },

  or: function(a,b){
    var hi = 0x80000000;
    var low = 0x7fffffff;
    var h = (a / hi) | (b / hi);
    var l = (a & low) | (b & low);
    return h*hi + l;
  },

  xor: function(a,b){
    var hi = 0x80000000;
    var low = 0x7fffffff;
    var h = (a / hi) ^ (b / hi);
    var l = (a & low) ^ (b & low);
    return h*hi + l;
  },

  shr: function(a,b){
    if (a<0) a += rtl.hiInt;
    if (a<0x80000000) return a >> b;
    if (b<=0) return a;
    if (b>54) return 0;
    return Math.floor(a / Math.pow(2,b));
  },

  shl: function(a,b){
    if (a<0) a += rtl.hiInt;
    if (b<=0) return a;
    if (b>54) return 0;
    var r = a * Math.pow(2,b);
    if (r <= rtl.hiInt) return r;
    return r % rtl.hiInt;
  },

  initRTTI: function(){
    if (rtl.debug_rtti) rtl.debug('initRTTI');

    // base types
    rtl.tTypeInfo = { name: "tTypeInfo" };
    function newBaseTI(name,kind,ancestor){
      if (!ancestor) ancestor = rtl.tTypeInfo;
      if (rtl.debug_rtti) rtl.debug('initRTTI.newBaseTI "'+name+'" '+kind+' ("'+ancestor.name+'")');
      var t = Object.create(ancestor);
      t.name = name;
      t.kind = kind;
      rtl[name] = t;
      return t;
    };
    function newBaseInt(name,minvalue,maxvalue,ordtype){
      var t = newBaseTI(name,1 /* tkInteger */,rtl.tTypeInfoInteger);
      t.minvalue = minvalue;
      t.maxvalue = maxvalue;
      t.ordtype = ordtype;
      return t;
    };
    newBaseTI("tTypeInfoInteger",1 /* tkInteger */);
    newBaseInt("shortint",-0x80,0x7f,0);
    newBaseInt("byte",0,0xff,1);
    newBaseInt("smallint",-0x8000,0x7fff,2);
    newBaseInt("word",0,0xffff,3);
    newBaseInt("longint",-0x80000000,0x7fffffff,4);
    newBaseInt("longword",0,0xffffffff,5);
    newBaseInt("nativeint",-0x10000000000000,0xfffffffffffff,6);
    newBaseInt("nativeuint",0,0xfffffffffffff,7);
    newBaseTI("char",2 /* tkChar */);
    newBaseTI("string",3 /* tkString */);
    newBaseTI("tTypeInfoEnum",4 /* tkEnumeration */,rtl.tTypeInfoInteger);
    newBaseTI("tTypeInfoSet",5 /* tkSet */);
    newBaseTI("double",6 /* tkDouble */);
    newBaseTI("boolean",7 /* tkBool */);
    newBaseTI("tTypeInfoProcVar",8 /* tkProcVar */);
    newBaseTI("tTypeInfoMethodVar",9 /* tkMethod */,rtl.tTypeInfoProcVar);
    newBaseTI("tTypeInfoArray",10 /* tkArray */);
    newBaseTI("tTypeInfoDynArray",11 /* tkDynArray */);
    newBaseTI("tTypeInfoPointer",15 /* tkPointer */);
    var t = newBaseTI("pointer",15 /* tkPointer */,rtl.tTypeInfoPointer);
    t.reftype = null;
    newBaseTI("jsvalue",16 /* tkJSValue */);
    newBaseTI("tTypeInfoRefToProcVar",17 /* tkRefToProcVar */,rtl.tTypeInfoProcVar);

    // member kinds
    rtl.tTypeMember = {};
    function newMember(name,kind){
      var m = Object.create(rtl.tTypeMember);
      m.name = name;
      m.kind = kind;
      rtl[name] = m;
    };
    newMember("tTypeMemberField",1); // tmkField
    newMember("tTypeMemberMethod",2); // tmkMethod
    newMember("tTypeMemberProperty",3); // tmkProperty

    // base object for storing members: a simple object
    rtl.tTypeMembers = {};

    // tTypeInfoStruct - base object for tTypeInfoClass, tTypeInfoRecord, tTypeInfoInterface
    var tis = newBaseTI("tTypeInfoStruct",0);
    tis.$addMember = function(name,ancestor,options){
      if (rtl.debug_rtti){
        if (!rtl.hasString(name) || (name.charAt()==='$')) throw 'invalid member "'+name+'", this="'+this.name+'"';
        if (!rtl.is(ancestor,rtl.tTypeMember)) throw 'invalid ancestor "'+ancestor+':'+ancestor.name+'", "'+this.name+'.'+name+'"';
        if ((options!=undefined) && (typeof(options)!='object')) throw 'invalid options "'+options+'", "'+this.name+'.'+name+'"';
      };
      var t = Object.create(ancestor);
      t.name = name;
      this.members[name] = t;
      this.names.push(name);
      if (rtl.isObject(options)){
        for (var key in options) if (options.hasOwnProperty(key)) t[key] = options[key];
      };
      return t;
    };
    tis.addField = function(name,type,options){
      var t = this.$addMember(name,rtl.tTypeMemberField,options);
      if (rtl.debug_rtti){
        if (!rtl.is(type,rtl.tTypeInfo)) throw 'invalid type "'+type+'", "'+this.name+'.'+name+'"';
      };
      t.typeinfo = type;
      this.fields.push(name);
      return t;
    };
    tis.addFields = function(){
      var i=0;
      while(i<arguments.length){
        var name = arguments[i++];
        var type = arguments[i++];
        if ((i<arguments.length) && (typeof(arguments[i])==='object')){
          this.addField(name,type,arguments[i++]);
        } else {
          this.addField(name,type);
        };
      };
    };
    tis.addMethod = function(name,methodkind,params,result,options){
      var t = this.$addMember(name,rtl.tTypeMemberMethod,options);
      t.methodkind = methodkind;
      t.procsig = rtl.newTIProcSig(params);
      t.procsig.resulttype = result?result:null;
      this.methods.push(name);
      return t;
    };
    tis.addProperty = function(name,flags,result,getter,setter,options){
      var t = this.$addMember(name,rtl.tTypeMemberProperty,options);
      t.flags = flags;
      t.typeinfo = result;
      t.getter = getter;
      t.setter = setter;
      // Note: in options: params, stored, defaultvalue
      if (rtl.isArray(t.params)) t.params = rtl.newTIParams(t.params);
      this.properties.push(name);
      if (!rtl.isString(t.stored)) t.stored = "";
      return t;
    };
    tis.getField = function(index){
      return this.members[this.fields[index]];
    };
    tis.getMethod = function(index){
      return this.members[this.methods[index]];
    };
    tis.getProperty = function(index){
      return this.members[this.properties[index]];
    };

    newBaseTI("tTypeInfoRecord",12 /* tkRecord */,rtl.tTypeInfoStruct);
    newBaseTI("tTypeInfoClass",13 /* tkClass */,rtl.tTypeInfoStruct);
    newBaseTI("tTypeInfoClassRef",14 /* tkClassRef */);
    newBaseTI("tTypeInfoInterface",18 /* tkInterface */,rtl.tTypeInfoStruct);
    newBaseTI("tTypeInfoHelper",19 /* tkHelper */,rtl.tTypeInfoStruct);
  },

  tSectionRTTI: {
    $module: null,
    $inherited: function(name,ancestor,o){
      if (rtl.debug_rtti){
        rtl.debug('tSectionRTTI.newTI "'+(this.$module?this.$module.$name:"(no module)")
          +'"."'+name+'" ('+ancestor.name+') '+(o?'init':'forward'));
      };
      var t = this[name];
      if (t){
        if (!t.$forward) throw 'duplicate type "'+name+'"';
        if (!ancestor.isPrototypeOf(t)) throw 'typeinfo ancestor mismatch "'+name+'" ancestor="'+ancestor.name+'" t.name="'+t.name+'"';
      } else {
        t = Object.create(ancestor);
        t.name = name;
        t.$module = this.$module;
        this[name] = t;
      }
      if (o){
        delete t.$forward;
        for (var key in o) if (o.hasOwnProperty(key)) t[key]=o[key];
      } else {
        t.$forward = true;
      }
      return t;
    },
    $Scope: function(name,ancestor,o){
      var t=this.$inherited(name,ancestor,o);
      t.members = {};
      t.names = [];
      t.fields = [];
      t.methods = [];
      t.properties = [];
      return t;
    },
    $TI: function(name,kind,o){ var t=this.$inherited(name,rtl.tTypeInfo,o); t.kind = kind; return t; },
    $Int: function(name,o){ return this.$inherited(name,rtl.tTypeInfoInteger,o); },
    $Enum: function(name,o){ return this.$inherited(name,rtl.tTypeInfoEnum,o); },
    $Set: function(name,o){ return this.$inherited(name,rtl.tTypeInfoSet,o); },
    $StaticArray: function(name,o){ return this.$inherited(name,rtl.tTypeInfoArray,o); },
    $DynArray: function(name,o){ return this.$inherited(name,rtl.tTypeInfoDynArray,o); },
    $ProcVar: function(name,o){ return this.$inherited(name,rtl.tTypeInfoProcVar,o); },
    $RefToProcVar: function(name,o){ return this.$inherited(name,rtl.tTypeInfoRefToProcVar,o); },
    $MethodVar: function(name,o){ return this.$inherited(name,rtl.tTypeInfoMethodVar,o); },
    $Record: function(name,o){ return this.$Scope(name,rtl.tTypeInfoRecord,o); },
    $Class: function(name,o){ return this.$Scope(name,rtl.tTypeInfoClass,o); },
    $ClassRef: function(name,o){ return this.$inherited(name,rtl.tTypeInfoClassRef,o); },
    $Pointer: function(name,o){ return this.$inherited(name,rtl.tTypeInfoPointer,o); },
    $Interface: function(name,o){ return this.$Scope(name,rtl.tTypeInfoInterface,o); },
    $Helper: function(name,o){ return this.$Scope(name,rtl.tTypeInfoHelper,o); }
  },

  newTIParam: function(param){
    // param is an array, 0=name, 1=type, 2=optional flags
    var t = {
      name: param[0],
      typeinfo: param[1],
      flags: (rtl.isNumber(param[2]) ? param[2] : 0)
    };
    return t;
  },

  newTIParams: function(list){
    // list: optional array of [paramname,typeinfo,optional flags]
    var params = [];
    if (rtl.isArray(list)){
      for (var i=0; i<list.length; i++) params.push(rtl.newTIParam(list[i]));
    };
    return params;
  },

  newTIProcSig: function(params,result,flags){
    var s = {
      params: rtl.newTIParams(params),
      resulttype: result,
      flags: flags
    };
    return s;
  }
}
rtl.module("System",[],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  this.LineEnding = "\n";
  this.sLineBreak = $mod.LineEnding;
  this.TTextLineBreakStyle = {"0": "tlbsLF", tlbsLF: 0, "1": "tlbsCRLF", tlbsCRLF: 1, "2": "tlbsCR", tlbsCR: 2};
  rtl.recNewT($mod,"TGuid",function () {
    this.D1 = 0;
    this.D2 = 0;
    this.D3 = 0;
    this.$new = function () {
      var r = Object.create(this);
      r.D4 = rtl.arraySetLength(null,0,8);
      return r;
    };
    this.$eq = function (b) {
      return (this.D1 === b.D1) && (this.D2 === b.D2) && (this.D3 === b.D3) && rtl.arrayEq(this.D4,b.D4);
    };
    this.$assign = function (s) {
      this.D1 = s.D1;
      this.D2 = s.D2;
      this.D3 = s.D3;
      this.D4 = s.D4.slice(0);
      return this;
    };
  });
  rtl.recNewT($mod,"TMethod",function () {
    this.Code = null;
    this.Data = null;
    this.$eq = function (b) {
      return (this.Code === b.Code) && (this.Data === b.Data);
    };
    this.$assign = function (s) {
      this.Code = s.Code;
      this.Data = s.Data;
      return this;
    };
  });
  $mod.$rtti.$Class("TObject");
  $mod.$rtti.$ClassRef("TClass",{instancetype: $mod.$rtti["TObject"]});
  rtl.createClass($mod,"TObject",null,function () {
    this.$init = function () {
    };
    this.$final = function () {
    };
    this.Create = function () {
      return this;
    };
    this.Destroy = function () {
    };
    this.Free = function () {
      this.$destroy("Destroy");
    };
    this.ClassType = function () {
      return this;
    };
    this.InheritsFrom = function (aClass) {
      return (aClass!=null) && ((this==aClass) || aClass.isPrototypeOf(this));
    };
    this.MethodName = function (aCode) {
      var Result = "";
      Result = "";
      if (aCode === null) return Result;
      var i = 0;
      var TI = this.$rtti;
      // Callback ?
      if ((typeof aCode["fn"] === "string") && (typeof aCode["scope"] === "object")) return aCode["fn"];
      // Not a callback, check rtti
      while ((Result === "") && (TI != null)) {
        i = 0;
        while ((Result === "") && (i < TI.methods.length)) {
          if (this[TI.getMethod(i).name] === aCode)
            Result=TI.getMethod(i).name;
          i += 1;
        };
        if (Result === "") TI = TI.ancestor;
      };
      return Result;
      return Result;
    };
    this.FieldAddress = function (aName) {
      var Result = null;
      Result = null;
      var aClass = null;
      var i = 0;
      var ClassTI = null;
      var myName = aName.toLowerCase();
      var MemberTI = null;
      aClass = this.$class;
      while (aClass !== null) {
        ClassTI = aClass.$rtti;
        for (var $l1 = 0, $end2 = ClassTI.fields.length - 1; $l1 <= $end2; $l1++) {
          i = $l1;
          MemberTI = ClassTI.getField(i);
          if (MemberTI.name.toLowerCase() === myName) {
             return MemberTI;
          };
        };
        aClass = aClass.$ancestor ? aClass.$ancestor : null;
      };
      return Result;
    };
    this.AfterConstruction = function () {
    };
    this.BeforeDestruction = function () {
    };
    this.DispatchStr = function (aMessage) {
      var aClass = null;
      var Id = undefined;
      if (!rtl.isObject(aMessage)) return;
      Id = aMessage["MsgStr"];
      if (!rtl.isString(Id)) return;
      aClass = this.$class.ClassType();
      while (aClass !== null) {
        var Handlers = aClass.$msgstr;
        if (rtl.isObject(Handlers) && Handlers.hasOwnProperty(Id)){
          this[Handlers[Id]](aMessage);
          return;
        };
        aClass = aClass.$ancestor;
      };
      this.DefaultHandlerStr(aMessage);
    };
    this.DefaultHandlerStr = function (aMessage) {
      if (aMessage) ;
    };
    this.GetInterface = function (iid, obj) {
      var Result = false;
      var i = iid.$intf;
      if (i){
        // iid is the private TGuid of an interface
        i = rtl.getIntfG(this,i.$guid,2);
        if (i){
          obj.set(i);
          return true;
        }
      };
      Result = this.GetInterfaceByStr(rtl.guidrToStr(iid),obj);
      return Result;
    };
    this.GetInterfaceByStr = function (iidstr, obj) {
      var Result = false;
      if (!$mod.IObjectInstance["$str"]) $mod.IObjectInstance["$str"] = rtl.guidrToStr($mod.IObjectInstance);
      if (iidstr == $mod.IObjectInstance["$str"]) {
        obj.set(this);
        return true;
      };
      var i = rtl.getIntfG(this,iidstr,2);
      obj.set(i);
      return i!==null;
      Result = false;
      return Result;
    };
  });
  this.S_OK = 0;
  this.E_NOINTERFACE = -2147467262;
  rtl.createInterface($mod,"IUnknown","{00000000-0000-0000-C000-000000000046}",["QueryInterface","_AddRef","_Release"],null,function () {
    this.$kind = "com";
    var $r = this.$rtti;
    $r.addMethod("QueryInterface",1,[["iid",$mod.$rtti["TGuid"],2],["obj",null,4]],rtl.longint);
    $r.addMethod("_AddRef",1,null,rtl.longint);
    $r.addMethod("_Release",1,null,rtl.longint);
  });
  this.IObjectInstance = $mod.TGuid.$clone({D1: 0xD91C9AF4, D2: 0x3C93, D3: 0x420F, D4: [0xA3,0x03,0xBF,0x5B,0xA8,0x2B,0xFD,0x23]});
  this.IsConsole = false;
  this.OnParamCount = null;
  this.OnParamStr = null;
  this.Frac = function (A) {
    return A % 1;
  };
  this.Random = function (Range) {
    return Math.floor(Math.random()*Range);
  };
  this.Trunc = function (A) {
    if (!Math.trunc) {
      Math.trunc = function(v) {
        v = +v;
        if (!isFinite(v)) return v;
        return (v - v % 1) || (v < 0 ? -0 : v === 0 ? v : 0);
      };
    }
    $mod.Trunc = Math.trunc;
    return Math.trunc(A);
  };
  this.DefaultTextLineBreakStyle = 0;
  this.Int = function (A) {
    var Result = 0.0;
    Result = $mod.Trunc(A);
    return Result;
  };
  this.Copy = function (S, Index, Size) {
    if (Index<1) Index = 1;
    return (Size>0) ? S.substring(Index-1,Index+Size-1) : "";
  };
  this.Copy$1 = function (S, Index) {
    if (Index<1) Index = 1;
    return S.substr(Index-1);
  };
  this.Delete = function (S, Index, Size) {
    var h = "";
    if ((Index < 1) || (Index > S.get().length) || (Size <= 0)) return;
    h = S.get();
    S.set($mod.Copy(h,1,Index - 1) + $mod.Copy$1(h,Index + Size));
  };
  this.Pos = function (Search, InString) {
    return InString.indexOf(Search)+1;
  };
  this.Insert = function (Insertion, Target, Index) {
    var t = "";
    if (Insertion === "") return;
    t = Target.get();
    if (Index < 1) {
      Target.set(Insertion + t)}
     else if (Index > t.length) {
      Target.set(t + Insertion)}
     else Target.set($mod.Copy(t,1,Index - 1) + Insertion + $mod.Copy(t,Index,t.length));
  };
  this.upcase = function (c) {
    return c.toUpperCase();
  };
  this.val = function (S, NI, Code) {
    NI.set($impl.valint(S,-9007199254740991,9007199254740991,Code));
  };
  this.StringOfChar = function (c, l) {
    var Result = "";
    var i = 0;
    if ((l>0) && c.repeat) return c.repeat(l);
    Result = "";
    for (var $l1 = 1, $end2 = l; $l1 <= $end2; $l1++) {
      i = $l1;
      Result = Result + c;
    };
    return Result;
  };
  this.Assigned = function (V) {
    return (V!=undefined) && (V!=null) && (!rtl.isArray(V) || (V.length > 0));
  };
  $mod.$init = function () {
    rtl.exitcode = 0;
  };
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.valint = function (S, MinVal, MaxVal, Code) {
    var Result = 0;
    var x = 0.0;
    x = Number(S);
    if (isNaN(x)) {
      var $tmp1 = $mod.Copy(S,1,1);
      if ($tmp1 === "$") {
        x = Number("0x" + $mod.Copy$1(S,2))}
       else if ($tmp1 === "&") {
        x = Number("0o" + $mod.Copy$1(S,2))}
       else if ($tmp1 === "%") {
        x = Number("0b" + $mod.Copy$1(S,2))}
       else {
        Code.set(1);
        return Result;
      };
    };
    if (isNaN(x) || (x !== $mod.Int(x))) {
      Code.set(1)}
     else if ((x < MinVal) || (x > MaxVal)) {
      Code.set(2)}
     else {
      Result = $mod.Trunc(x);
      Code.set(0);
    };
    return Result;
  };
});
rtl.module("RTLConsts",["System"],function () {
  "use strict";
  var $mod = this;
  $mod.$resourcestrings = {SArgumentMissing: {org: 'Missing argument in format "%s"'}, SInvalidFormat: {org: 'Invalid format specifier : "%s"'}, SInvalidArgIndex: {org: 'Invalid argument index in format: "%s"'}, SListCapacityError: {org: "List capacity (%s) exceeded."}, SListCountError: {org: "List count (%s) out of bounds."}, SListIndexError: {org: "List index (%s) out of bounds"}, SSortedListError: {org: "Operation not allowed on sorted list"}, SDuplicateString: {org: "String list does not allow duplicates"}, SErrFindNeedsSortedList: {org: "Cannot use find on unsorted list"}, SInvalidName: {org: 'Invalid component name: "%s"'}, SInvalidBoolean: {org: '"%s" is not a valid boolean.'}, SDuplicateName: {org: 'Duplicate component name: "%s"'}, SErrInvalidDate: {org: 'Invalid date: "%s"'}, SErrInvalidTimeFormat: {org: 'Invalid time format: "%s"'}, SInvalidDateFormat: {org: 'Invalid date format: "%s"'}, SCantReadPropertyS: {org: 'Cannot read property "%s"'}, SCantWritePropertyS: {org: 'Cannot write property "%s"'}, SErrPropertyNotFound: {org: 'Unknown property: "%s"'}, SIndexedPropertyNeedsParams: {org: 'Indexed property "%s" needs parameters'}, SErrInvalidTypecast: {org: "Invalid class typecast"}, SErrInvalidInteger: {org: 'Invalid integer value: "%s"'}, SErrInvalidFloat: {org: 'Invalid floating-point value: "%s"'}, SInvalidDateTime: {org: "Invalid date-time value: %s"}, SInvalidCurrency: {org: "Invalid currency value: %s"}, SErrInvalidDayOfWeek: {org: "%d is not a valid day of the week"}, SErrInvalidTimeStamp: {org: 'Invalid date\/timestamp : "%s"'}, SErrInvalidDateWeek: {org: "%d %d %d is not a valid dateweek"}, SErrInvalidDayOfYear: {org: "Year %d does not have a day number %d"}, SErrInvalidDateMonthWeek: {org: "Year %d, month %d, Week %d and day %d is not a valid date."}, SErrInvalidDayOfWeekInMonth: {org: "Year %d Month %d NDow %d DOW %d is not a valid date"}, SInvalidJulianDate: {org: "%f Julian cannot be represented as a DateTime"}, SErrInvalidHourMinuteSecMsec: {org: "%d:%d:%d.%d is not a valid time specification"}, SInvalidGUID: {org: '"%s" is not a valid GUID value'}, SEmptyStreamIllegalReader: {org: "Illegal Nil stream for TReader constructor"}, SInvalidPropertyValue: {org: "Invalid value for property"}, SInvalidImage: {org: "Invalid stream format"}, SUnknownProperty: {org: 'Unknown property: "%s"'}, SUnknownPropertyType: {org: "Unknown property type %d"}, SAncestorNotFound: {org: 'Ancestor class for "%s" not found.'}, SUnsupportedPropertyVariantType: {org: "Unsupported property variant type %d"}, SPropertyException: {org: "Error reading %s%s%s: %s"}, SInvalidPropertyPath: {org: "Invalid property path"}, SReadOnlyProperty: {org: "Property is read-only"}, SClassNotFound: {org: 'Class "%s" not found'}, SEmptyStreamIllegalWriter: {org: "Illegal Nil stream for TWriter constructor"}, SErrInvalidPropertyType: {org: "Invalid property type from streamed property: %d"}, SParserExpected: {org: "Wrong token type: %s expected"}, SParserInvalidFloat: {org: "Invalid floating point number: %s"}, SParserInvalidInteger: {org: "Invalid integer number: %s"}, SParserUnterminatedString: {org: "Unterminated string"}, SParserWrongTokenType: {org: "Wrong token type: %s expected but %s found"}, SParserWrongTokenSymbol: {org: "Wrong token symbol: %s expected but %s found"}, SParserLocInfo: {org: " (at %d,%d, stream offset %.8x)"}, SParserUnterminatedBinValue: {org: "Unterminated byte value"}, SParserInvalidProperty: {org: "Invalid property"}};
});
rtl.module("Types",["System"],function () {
  "use strict";
  var $mod = this;
  this.TDirection = {"0": "FromBeginning", FromBeginning: 0, "1": "FromEnd", FromEnd: 1};
  this.TDuplicates = {"0": "dupIgnore", dupIgnore: 0, "1": "dupAccept", dupAccept: 1, "2": "dupError", dupError: 2};
});
rtl.module("JS",["System","Types"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"EJS",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FMessage = "";
    };
    this.Create$1 = function (Msg) {
      this.FMessage = Msg;
      return this;
    };
  });
  this.New = function (aElements) {
    var Result = null;
    var L = 0;
    var I = 0;
    var S = "";
    L = rtl.length(aElements);
    if ((L % 2) === 1) throw $mod.EJS.$create("Create$1",["Number of arguments must be even"]);
    I = 0;
    while (I < L) {
      if (!rtl.isString(aElements[I])) {
        S = String(I);
        throw $mod.EJS.$create("Create$1",["Argument " + S + " must be a string."]);
      };
      I += 2;
    };
    I = 0;
    Result = new Object();
    while (I < L) {
      S = "" + aElements[I];
      Result[S] = aElements[I + 1];
      I += 2;
    };
    return Result;
  };
  this.JSDelete = function (Obj, PropName) {
    return delete Obj[PropName];
  };
  this.isBoolean = function (v) {
    return typeof(v) == 'boolean';
  };
  this.isInteger = function (v) {
    return Math.floor(v)===v;
  };
  this.isNull = function (v) {
    return v === null;
  };
  this.isUndefined = function (v) {
    return v == undefined;
  };
  this.TJSValueType = {"0": "jvtNull", jvtNull: 0, "1": "jvtBoolean", jvtBoolean: 1, "2": "jvtInteger", jvtInteger: 2, "3": "jvtFloat", jvtFloat: 3, "4": "jvtString", jvtString: 4, "5": "jvtObject", jvtObject: 5, "6": "jvtArray", jvtArray: 6};
  this.GetValueType = function (JS) {
    var Result = 0;
    var t = "";
    if ($mod.isNull(JS)) {
      Result = 0}
     else {
      t = typeof(JS);
      if (t === "string") {
        Result = 4}
       else if (t === "boolean") {
        Result = 1}
       else if (t === "object") {
        if (rtl.isArray(JS)) {
          Result = 6}
         else Result = 5;
      } else if (t === "number") if ($mod.isInteger(JS)) {
        Result = 2}
       else Result = 3;
    };
    return Result;
  };
});
rtl.module("SysUtils",["System","RTLConsts","JS"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  this.FreeAndNil = function (Obj) {
    var o = null;
    o = Obj.get();
    if (o === null) return;
    Obj.set(null);
    o.$destroy("Destroy");
  };
  this.TEndian = {"0": "Little", Little: 0, "1": "Big", Big: 1};
  rtl.createClass($mod,"Exception",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.fMessage = "";
    };
    this.Create$1 = function (Msg) {
      this.fMessage = Msg;
      return this;
    };
    this.CreateFmt = function (Msg, Args) {
      this.Create$1($mod.Format(Msg,Args));
      return this;
    };
  });
  rtl.createClass($mod,"EConvertError",$mod.Exception,function () {
  });
  this.Trim = function (S) {
    return S.trim();
  };
  this.TrimLeft = function (S) {
    return S.replace(/^[\s\uFEFF\xA0\x00-\x1f]+/,'');
  };
  this.UpperCase = function (s) {
    return s.toUpperCase();
  };
  this.LowerCase = function (s) {
    return s.toLowerCase();
  };
  this.CompareStr = function (s1, s2) {
    var l1 = s1.length;
    var l2 = s2.length;
    if (l1<=l2){
      var s = s2.substr(0,l1);
      if (s1<s){ return -1;
      } else if (s1>s){ return 1;
      } else { return l1<l2 ? -1 : 0; };
    } else {
      var s = s1.substr(0,l2);
      if (s<s2){ return -1;
      } else { return 1; };
    };
  };
  this.CompareText = function (s1, s2) {
    var l1 = s1.toLowerCase();
    var l2 = s2.toLowerCase();
    if (l1>l2){ return 1;
    } else if (l1<l2){ return -1;
    } else { return 0; };
  };
  this.SameText = function (s1, s2) {
    return s1.toLowerCase() == s2.toLowerCase();
  };
  this.Format = function (Fmt, Args) {
    var Result = "";
    var ChPos = 0;
    var OldPos = 0;
    var ArgPos = 0;
    var DoArg = 0;
    var Len = 0;
    var Hs = "";
    var ToAdd = "";
    var Index = 0;
    var Width = 0;
    var Prec = 0;
    var Left = false;
    var Fchar = "";
    var vq = 0;
    function ReadFormat() {
      var Result = "";
      var Value = 0;
      function ReadInteger() {
        var Code = 0;
        var ArgN = 0;
        if (Value !== -1) return;
        OldPos = ChPos;
        while ((ChPos <= Len) && (Fmt.charAt(ChPos - 1) <= "9") && (Fmt.charAt(ChPos - 1) >= "0")) ChPos += 1;
        if (ChPos > Len) $impl.DoFormatError(1,Fmt);
        if (Fmt.charAt(ChPos - 1) === "*") {
          if (Index === -1) {
            ArgN = ArgPos}
           else {
            ArgN = Index;
            Index += 1;
          };
          if ((ChPos > OldPos) || (ArgN > (rtl.length(Args) - 1))) $impl.DoFormatError(1,Fmt);
          ArgPos = ArgN + 1;
          if (rtl.isNumber(Args[ArgN]) && pas.JS.isInteger(Args[ArgN])) {
            Value = Math.floor(Args[ArgN])}
           else $impl.DoFormatError(1,Fmt);
          ChPos += 1;
        } else {
          if (OldPos < ChPos) {
            pas.System.val(pas.System.Copy(Fmt,OldPos,ChPos - OldPos),{get: function () {
                return Value;
              }, set: function (v) {
                Value = v;
              }},{get: function () {
                return Code;
              }, set: function (v) {
                Code = v;
              }});
            if (Code > 0) $impl.DoFormatError(1,Fmt);
          } else Value = -1;
        };
      };
      function ReadIndex() {
        if (Fmt.charAt(ChPos - 1) !== ":") {
          ReadInteger()}
         else Value = 0;
        if (Fmt.charAt(ChPos - 1) === ":") {
          if (Value === -1) $impl.DoFormatError(2,Fmt);
          Index = Value;
          Value = -1;
          ChPos += 1;
        };
      };
      function ReadLeft() {
        if (Fmt.charAt(ChPos - 1) === "-") {
          Left = true;
          ChPos += 1;
        } else Left = false;
      };
      function ReadWidth() {
        ReadInteger();
        if (Value !== -1) {
          Width = Value;
          Value = -1;
        };
      };
      function ReadPrec() {
        if (Fmt.charAt(ChPos - 1) === ".") {
          ChPos += 1;
          ReadInteger();
          if (Value === -1) Value = 0;
          Prec = Value;
        };
      };
      Index = -1;
      Width = -1;
      Prec = -1;
      Value = -1;
      ChPos += 1;
      if (Fmt.charAt(ChPos - 1) === "%") {
        Result = "%";
        return Result;
      };
      ReadIndex();
      ReadLeft();
      ReadWidth();
      ReadPrec();
      Result = pas.System.upcase(Fmt.charAt(ChPos - 1));
      return Result;
    };
    function Checkarg(AT, err) {
      var Result = false;
      Result = false;
      if (Index === -1) {
        DoArg = ArgPos}
       else DoArg = Index;
      ArgPos = DoArg + 1;
      if ((DoArg > (rtl.length(Args) - 1)) || (pas.JS.GetValueType(Args[DoArg]) !== AT)) {
        if (err) $impl.DoFormatError(3,Fmt);
        ArgPos -= 1;
        return Result;
      };
      Result = true;
      return Result;
    };
    Result = "";
    Len = Fmt.length;
    ChPos = 1;
    OldPos = 1;
    ArgPos = 0;
    while (ChPos <= Len) {
      while ((ChPos <= Len) && (Fmt.charAt(ChPos - 1) !== "%")) ChPos += 1;
      if (ChPos > OldPos) Result = Result + pas.System.Copy(Fmt,OldPos,ChPos - OldPos);
      if (ChPos < Len) {
        Fchar = ReadFormat();
        var $tmp1 = Fchar;
        if ($tmp1 === "D") {
          Checkarg(2,true);
          ToAdd = $mod.IntToStr(Math.floor(Args[DoArg]));
          Width = Math.abs(Width);
          Index = Prec - ToAdd.length;
          if (ToAdd.charAt(0) !== "-") {
            ToAdd = pas.System.StringOfChar("0",Index) + ToAdd}
           else pas.System.Insert(pas.System.StringOfChar("0",Index + 1),{get: function () {
              return ToAdd;
            }, set: function (v) {
              ToAdd = v;
            }},2);
        } else if ($tmp1 === "U") {
          Checkarg(2,true);
          if (Math.floor(Args[DoArg]) < 0) $impl.DoFormatError(3,Fmt);
          ToAdd = $mod.IntToStr(Math.floor(Args[DoArg]));
          Width = Math.abs(Width);
          Index = Prec - ToAdd.length;
          ToAdd = pas.System.StringOfChar("0",Index) + ToAdd;
        } else if ($tmp1 === "E") {
          if (Checkarg(3,false) || Checkarg(2,true)) ToAdd = $mod.FloatToStrF(rtl.getNumber(Args[DoArg]),0,9999,Prec);
        } else if ($tmp1 === "F") {
          if (Checkarg(3,false) || Checkarg(2,true)) ToAdd = $mod.FloatToStrF(rtl.getNumber(Args[DoArg]),0,9999,Prec);
        } else if ($tmp1 === "G") {
          if (Checkarg(3,false) || Checkarg(2,true)) ToAdd = $mod.FloatToStrF(rtl.getNumber(Args[DoArg]),1,Prec,3);
        } else if ($tmp1 === "N") {
          if (Checkarg(3,false) || Checkarg(2,true)) ToAdd = $mod.FloatToStrF(rtl.getNumber(Args[DoArg]),3,9999,Prec);
        } else if ($tmp1 === "M") {
          if (Checkarg(3,false) || Checkarg(2,true)) ToAdd = $mod.FloatToStrF(rtl.getNumber(Args[DoArg]),4,9999,Prec);
        } else if ($tmp1 === "S") {
          Checkarg(4,true);
          Hs = "" + Args[DoArg];
          Index = Hs.length;
          if ((Prec !== -1) && (Index > Prec)) Index = Prec;
          ToAdd = pas.System.Copy(Hs,1,Index);
        } else if ($tmp1 === "P") {
          Checkarg(2,true);
          ToAdd = $mod.IntToHex(Math.floor(Args[DoArg]),31);
        } else if ($tmp1 === "X") {
          Checkarg(2,true);
          vq = Math.floor(Args[DoArg]);
          Index = 31;
          if (Prec > Index) {
            ToAdd = $mod.IntToHex(vq,Index)}
           else {
            Index = 1;
            while ((rtl.shl(1,Index * 4) <= vq) && (Index < 16)) Index += 1;
            if (Index > Prec) Prec = Index;
            ToAdd = $mod.IntToHex(vq,Prec);
          };
        } else if ($tmp1 === "%") ToAdd = "%";
        if (Width !== -1) if (ToAdd.length < Width) if (!Left) {
          ToAdd = pas.System.StringOfChar(" ",Width - ToAdd.length) + ToAdd}
         else ToAdd = ToAdd + pas.System.StringOfChar(" ",Width - ToAdd.length);
        Result = Result + ToAdd;
      };
      ChPos += 1;
      OldPos = ChPos;
    };
    return Result;
  };
  var Alpha = rtl.createSet(null,65,90,null,97,122,95);
  var AlphaNum = rtl.unionSet(Alpha,rtl.createSet(null,48,57));
  var Dot = ".";
  this.IsValidIdent = function (Ident, AllowDots, StrictDots) {
    var Result = false;
    var First = false;
    var I = 0;
    var Len = 0;
    Len = Ident.length;
    if (Len < 1) return false;
    First = true;
    Result = false;
    I = 1;
    while (I <= Len) {
      if (First) {
        if (!(Ident.charCodeAt(I - 1) in Alpha)) return Result;
        First = false;
      } else if (AllowDots && (Ident.charAt(I - 1) === Dot)) {
        if (StrictDots) {
          if (I >= Len) return Result;
          First = true;
        };
      } else if (!(Ident.charCodeAt(I - 1) in AlphaNum)) return Result;
      I = I + 1;
    };
    Result = true;
    return Result;
  };
  this.TStringReplaceFlag = {"0": "rfReplaceAll", rfReplaceAll: 0, "1": "rfIgnoreCase", rfIgnoreCase: 1};
  this.StringReplace = function (aOriginal, aSearch, aReplace, Flags) {
    var Result = "";
    var REFlags = "";
    var REString = "";
    REFlags = "";
    if (0 in Flags) REFlags = "g";
    if (1 in Flags) REFlags = REFlags + "i";
    REString = aSearch.replace(new RegExp($impl.RESpecials,"g"),"\\$1");
    Result = aOriginal.replace(new RegExp(REString,REFlags),aReplace);
    return Result;
  };
  this.IntToStr = function (Value) {
    var Result = "";
    Result = "" + Value;
    return Result;
  };
  this.TryStrToInt$1 = function (S, res) {
    var Result = false;
    var Radix = 10;
    var N = "";
    var J = undefined;
    N = S;
    var $tmp1 = pas.System.Copy(N,1,1);
    if ($tmp1 === "$") {
      Radix = 16}
     else if ($tmp1 === "&") {
      Radix = 8}
     else if ($tmp1 === "%") Radix = 2;
    if (Radix !== 10) pas.System.Delete({get: function () {
        return N;
      }, set: function (v) {
        N = v;
      }},1,1);
    J = parseInt(N,Radix);
    Result = !isNaN(J);
    if (Result) res.set(Math.floor(J));
    return Result;
  };
  this.StrToIntDef = function (S, aDef) {
    var Result = 0;
    var R = 0;
    if ($mod.TryStrToInt$1(S,{get: function () {
        return R;
      }, set: function (v) {
        R = v;
      }})) {
      Result = R}
     else Result = aDef;
    return Result;
  };
  var HexDigits = "0123456789ABCDEF";
  this.IntToHex = function (Value, Digits) {
    var Result = "";
    if (Digits === 0) Digits = 1;
    Result = "";
    while (Value > 0) {
      Result = HexDigits.charAt(((Value & 15) + 1) - 1) + Result;
      Value = Math.floor(Value / 16);
    };
    while (Result.length < Digits) Result = "0" + Result;
    return Result;
  };
  this.TFloatFormat = {"0": "ffFixed", ffFixed: 0, "1": "ffGeneral", ffGeneral: 1, "2": "ffExponent", ffExponent: 2, "3": "ffNumber", ffNumber: 3, "4": "ffCurrency", ffCurrency: 4};
  this.FloatToStrF = function (Value, format, Precision, Digits) {
    var Result = "";
    var DS = "";
    DS = $mod.DecimalSeparator;
    var $tmp1 = format;
    if ($tmp1 === 1) {
      Result = $impl.FormatGeneralFloat(Value,Precision,DS)}
     else if ($tmp1 === 2) {
      Result = $impl.FormatExponentFloat(Value,Precision,Digits,DS)}
     else if ($tmp1 === 0) {
      Result = $impl.FormatFixedFloat(Value,Digits,DS)}
     else if ($tmp1 === 3) {
      Result = $impl.FormatNumberFloat(Value,Digits,DS,$mod.ThousandSeparator)}
     else if ($tmp1 === 4) Result = $impl.FormatNumberCurrency(Value * 10000,Digits,DS,$mod.ThousandSeparator);
    if ((format !== 4) && (Result.length > 1) && (Result.charAt(0) === "-")) $impl.RemoveLeadingNegativeSign({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},DS);
    return Result;
  };
  this.OnGetEnvironmentVariable = null;
  this.OnGetEnvironmentString = null;
  this.OnGetEnvironmentVariableCount = null;
  this.DecimalSeparator = ".";
  this.ThousandSeparator = "";
  this.CurrencyFormat = 0;
  this.NegCurrFormat = 0;
  this.CurrencyDecimals = 2;
  this.CurrencyString = "$";
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.feInvalidFormat = 1;
  $impl.feMissingArgument = 2;
  $impl.feInvalidArgIndex = 3;
  $impl.DoFormatError = function (ErrCode, fmt) {
    var $tmp1 = ErrCode;
    if ($tmp1 === 1) {
      throw $mod.EConvertError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SInvalidFormat"),[fmt]])}
     else if ($tmp1 === 2) {
      throw $mod.EConvertError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SArgumentMissing"),[fmt]])}
     else if ($tmp1 === 3) throw $mod.EConvertError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SInvalidArgIndex"),[fmt]]);
  };
  $impl.maxdigits = 15;
  $impl.ReplaceDecimalSep = function (S, DS) {
    var Result = "";
    var P = 0;
    P = pas.System.Pos(".",S);
    if (P > 0) {
      Result = pas.System.Copy(S,1,P - 1) + DS + pas.System.Copy(S,P + 1,S.length - P)}
     else Result = S;
    return Result;
  };
  $impl.FormatGeneralFloat = function (Value, Precision, DS) {
    var Result = "";
    var P = 0;
    var PE = 0;
    var Q = 0;
    var Exponent = 0;
    if ((Precision === -1) || (Precision > 15)) Precision = 15;
    Result = rtl.floatToStr(Value,Precision + 7);
    Result = $mod.TrimLeft(Result);
    P = pas.System.Pos(".",Result);
    if (P === 0) return Result;
    PE = pas.System.Pos("E",Result);
    if (PE === 0) {
      Result = $impl.ReplaceDecimalSep(Result,DS);
      return Result;
    };
    Q = PE + 2;
    Exponent = 0;
    while (Q <= Result.length) {
      Exponent = ((Exponent * 10) + Result.charCodeAt(Q - 1)) - 48;
      Q += 1;
    };
    if (Result.charAt((PE + 1) - 1) === "-") Exponent = -Exponent;
    if (((P + Exponent) < PE) && (Exponent > -6)) {
      Result = rtl.strSetLength(Result,PE - 1);
      if (Exponent >= 0) {
        for (var $l1 = 0, $end2 = Exponent - 1; $l1 <= $end2; $l1++) {
          Q = $l1;
          Result = rtl.setCharAt(Result,P - 1,Result.charAt((P + 1) - 1));
          P += 1;
        };
        Result = rtl.setCharAt(Result,P - 1,".");
        P = 1;
        if (Result.charAt(P - 1) === "-") P += 1;
        while ((Result.charAt(P - 1) === "0") && (P < Result.length) && (pas.System.Copy(Result,P + 1,DS.length) !== DS)) pas.System.Delete({get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},P,1);
      } else {
        pas.System.Insert(pas.System.Copy("00000",1,-Exponent),{get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},P - 1);
        Result = rtl.setCharAt(Result,P - Exponent - 1,Result.charAt(P - Exponent - 1 - 1));
        Result = rtl.setCharAt(Result,P - 1,".");
        if (Exponent !== -1) Result = rtl.setCharAt(Result,P - Exponent - 1 - 1,"0");
      };
      Q = Result.length;
      while ((Q > 0) && (Result.charAt(Q - 1) === "0")) Q -= 1;
      if (Result.charAt(Q - 1) === ".") Q -= 1;
      if ((Q === 0) || ((Q === 1) && (Result.charAt(0) === "-"))) {
        Result = "0"}
       else Result = rtl.strSetLength(Result,Q);
    } else {
      while (Result.charAt(PE - 1 - 1) === "0") {
        pas.System.Delete({get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},PE - 1,1);
        PE -= 1;
      };
      if (Result.charAt(PE - 1 - 1) === DS) {
        pas.System.Delete({get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},PE - 1,1);
        PE -= 1;
      };
      if (Result.charAt((PE + 1) - 1) === "+") {
        pas.System.Delete({get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},PE + 1,1)}
       else PE += 1;
      while (Result.charAt((PE + 1) - 1) === "0") pas.System.Delete({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},PE + 1,1);
    };
    Result = $impl.ReplaceDecimalSep(Result,DS);
    return Result;
  };
  $impl.FormatExponentFloat = function (Value, Precision, Digits, DS) {
    var Result = "";
    var P = 0;
    DS = $mod.DecimalSeparator;
    if ((Precision === -1) || (Precision > 15)) Precision = 15;
    Result = rtl.floatToStr(Value,Precision + 7);
    while (Result.charAt(0) === " ") pas.System.Delete({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},1,1);
    P = pas.System.Pos("E",Result);
    if (P === 0) {
      Result = $impl.ReplaceDecimalSep(Result,DS);
      return Result;
    };
    P += 2;
    if (Digits > 4) Digits = 4;
    Digits = (Result.length - P - Digits) + 1;
    if (Digits < 0) {
      pas.System.Insert(pas.System.Copy("0000",1,-Digits),{get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},P)}
     else while ((Digits > 0) && (Result.charAt(P - 1) === "0")) {
      pas.System.Delete({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},P,1);
      if (P > Result.length) {
        pas.System.Delete({get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},P - 2,2);
        break;
      };
      Digits -= 1;
    };
    Result = $impl.ReplaceDecimalSep(Result,DS);
    return Result;
  };
  $impl.FormatFixedFloat = function (Value, Digits, DS) {
    var Result = "";
    if (Digits === -1) {
      Digits = 2}
     else if (Digits > 18) Digits = 18;
    Result = rtl.floatToStr(Value,0,Digits);
    if ((Result !== "") && (Result.charAt(0) === " ")) pas.System.Delete({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},1,1);
    Result = $impl.ReplaceDecimalSep(Result,DS);
    return Result;
  };
  $impl.FormatNumberFloat = function (Value, Digits, DS, TS) {
    var Result = "";
    var P = 0;
    if (Digits === -1) {
      Digits = 2}
     else if (Digits > 15) Digits = 15;
    Result = rtl.floatToStr(Value,0,Digits);
    if ((Result !== "") && (Result.charAt(0) === " ")) pas.System.Delete({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},1,1);
    P = pas.System.Pos(".",Result);
    Result = $impl.ReplaceDecimalSep(Result,DS);
    P -= 3;
    if ((TS !== "") && (TS !== "\x00")) while (P > 1) {
      if (Result.charAt(P - 1 - 1) !== "-") pas.System.Insert(TS,{get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},P);
      P -= 3;
    };
    return Result;
  };
  $impl.RemoveLeadingNegativeSign = function (AValue, DS) {
    var Result = false;
    var i = 0;
    var TS = "";
    var StartPos = 0;
    Result = false;
    StartPos = 2;
    TS = $mod.ThousandSeparator;
    for (var $l1 = StartPos, $end2 = AValue.get().length; $l1 <= $end2; $l1++) {
      i = $l1;
      Result = (AValue.get().charCodeAt(i - 1) in rtl.createSet(48,DS.charCodeAt(),69,43)) || (AValue.get().charAt(i - 1) === TS);
      if (!Result) break;
    };
    if (Result && (AValue.get().charAt(0) === "-")) pas.System.Delete(AValue,1,1);
    return Result;
  };
  $impl.FormatNumberCurrency = function (Value, Digits, DS, TS) {
    var Result = "";
    var Negative = false;
    var P = 0;
    if (Digits === -1) {
      Digits = $mod.CurrencyDecimals}
     else if (Digits > 18) Digits = 18;
    Result = rtl.floatToStr(Value / 10000,0,Digits);
    Negative = Result.charAt(0) === "-";
    if (Negative) pas.System.Delete({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},1,1);
    P = pas.System.Pos(".",Result);
    if (TS !== "") {
      if (P !== 0) {
        Result = $impl.ReplaceDecimalSep(Result,DS)}
       else P = Result.length + 1;
      P -= 3;
      while (P > 1) {
        pas.System.Insert(TS,{get: function () {
            return Result;
          }, set: function (v) {
            Result = v;
          }},P);
        P -= 3;
      };
    };
    if (Negative) $impl.RemoveLeadingNegativeSign({get: function () {
        return Result;
      }, set: function (v) {
        Result = v;
      }},DS);
    if (!Negative) {
      var $tmp1 = $mod.CurrencyFormat;
      if ($tmp1 === 0) {
        Result = $mod.CurrencyString + Result}
       else if ($tmp1 === 1) {
        Result = Result + $mod.CurrencyString}
       else if ($tmp1 === 2) {
        Result = $mod.CurrencyString + " " + Result}
       else if ($tmp1 === 3) Result = Result + " " + $mod.CurrencyString;
    } else {
      var $tmp2 = $mod.NegCurrFormat;
      if ($tmp2 === 0) {
        Result = "(" + $mod.CurrencyString + Result + ")"}
       else if ($tmp2 === 1) {
        Result = "-" + $mod.CurrencyString + Result}
       else if ($tmp2 === 2) {
        Result = $mod.CurrencyString + "-" + Result}
       else if ($tmp2 === 3) {
        Result = $mod.CurrencyString + Result + "-"}
       else if ($tmp2 === 4) {
        Result = "(" + Result + $mod.CurrencyString + ")"}
       else if ($tmp2 === 5) {
        Result = "-" + Result + $mod.CurrencyString}
       else if ($tmp2 === 6) {
        Result = Result + "-" + $mod.CurrencyString}
       else if ($tmp2 === 7) {
        Result = Result + $mod.CurrencyString + "-"}
       else if ($tmp2 === 8) {
        Result = "-" + Result + " " + $mod.CurrencyString}
       else if ($tmp2 === 9) {
        Result = "-" + $mod.CurrencyString + " " + Result}
       else if ($tmp2 === 10) {
        Result = Result + " " + $mod.CurrencyString + "-"}
       else if ($tmp2 === 11) {
        Result = $mod.CurrencyString + " " + Result + "-"}
       else if ($tmp2 === 12) {
        Result = $mod.CurrencyString + " " + "-" + Result}
       else if ($tmp2 === 13) {
        Result = Result + "-" + " " + $mod.CurrencyString}
       else if ($tmp2 === 14) {
        Result = "(" + $mod.CurrencyString + " " + Result + ")"}
       else if ($tmp2 === 15) Result = "(" + Result + " " + $mod.CurrencyString + ")";
    };
    return Result;
  };
  $impl.RESpecials = "([\\+\\[\\]\\(\\)\\\\\\.\\*])";
});
rtl.module("TypInfo",["System","SysUtils","Types","RTLConsts","JS"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  this.TTypeKind = {"0": "tkUnknown", tkUnknown: 0, "1": "tkInteger", tkInteger: 1, "2": "tkChar", tkChar: 2, "3": "tkString", tkString: 3, "4": "tkEnumeration", tkEnumeration: 4, "5": "tkSet", tkSet: 5, "6": "tkDouble", tkDouble: 6, "7": "tkBool", tkBool: 7, "8": "tkProcVar", tkProcVar: 8, "9": "tkMethod", tkMethod: 9, "10": "tkArray", tkArray: 10, "11": "tkDynArray", tkDynArray: 11, "12": "tkRecord", tkRecord: 12, "13": "tkClass", tkClass: 13, "14": "tkClassRef", tkClassRef: 14, "15": "tkPointer", tkPointer: 15, "16": "tkJSValue", tkJSValue: 16, "17": "tkRefToProcVar", tkRefToProcVar: 17, "18": "tkInterface", tkInterface: 18, "19": "tkHelper", tkHelper: 19};
  this.tkFloat = 6;
  this.pfGetFunction = 1;
  this.pfHasIndex = 16;
  rtl.createClass($mod,"EPropertyError",pas.SysUtils.Exception,function () {
  });
  this.GetPropInfos = function (aTIStruct) {
    var Result = [];
    var C = null;
    var i = 0;
    var Names = null;
    var PropName = "";
    Result = [];
    C = aTIStruct;
    Names = new Object();
    while (C !== null) {
      for (var $l1 = 0, $end2 = C.properties.length - 1; $l1 <= $end2; $l1++) {
        i = $l1;
        PropName = C.properties[i];
        if (Names.hasOwnProperty(PropName)) continue;
        Result.push(C.members[PropName]);
        Names[PropName] = true;
      };
      if (!rtl.isExt(C,rtl.tTypeInfoClass)) break;
      C = C.ancestor;
    };
    return Result;
  };
  this.GetPropList$2 = function (AClass) {
    var Result = [];
    Result = $mod.GetPropInfos(AClass.$rtti);
    return Result;
  };
  this.GetPropList$3 = function (Instance) {
    var Result = [];
    Result = $mod.GetPropList$2(Instance.$class.ClassType());
    return Result;
  };
  this.IsStoredProp = function (Instance, PropInfo) {
    var Result = false;
    var $tmp1 = PropInfo.flags & 12;
    if ($tmp1 === 0) {
      Result = true}
     else if ($tmp1 === 4) {
      Result = false}
     else if ($tmp1 === 8) {
      Result = !(Instance[PropInfo.stored] == false)}
     else {
      Result = Instance[PropInfo.stored]();
    };
    return Result;
  };
  this.GetJSValueProp$1 = function (Instance, PropInfo) {
    var Result = undefined;
    var gk = 0;
    gk = $impl.GetPropGetterKind(PropInfo);
    var $tmp1 = gk;
    if ($tmp1 === 0) {
      throw $mod.EPropertyError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SCantReadPropertyS"),[PropInfo.name]])}
     else if ($tmp1 === 1) {
      Result = Instance[PropInfo.getter]}
     else if ($tmp1 === 2) {
      if ((16 & PropInfo.flags) > 0) {
        Result = Instance[PropInfo.getter](PropInfo.index)}
       else Result = Instance[PropInfo.getter]()}
     else if ($tmp1 === 3) throw $mod.EPropertyError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SIndexedPropertyNeedsParams"),[PropInfo.name]]);
    return Result;
  };
  this.GetJSValueProp$3 = function (Instance, PropInfo) {
    var Result = undefined;
    Result = $mod.GetJSValueProp$1(Instance,PropInfo);
    return Result;
  };
  this.GetOrdProp$1 = function (Instance, PropInfo) {
    var Result = 0;
    var o = null;
    var Key = "";
    var n = 0;
    if (PropInfo.typeinfo.kind === 5) {
      o = rtl.getObject($mod.GetJSValueProp$3(Instance,PropInfo));
      Result = 0;
      for (Key in o) {
        n = parseInt(Key,10);
        if (n < 32) Result = Result + (1 << n);
      };
    } else Result = Math.floor($mod.GetJSValueProp$3(Instance,PropInfo));
    return Result;
  };
  this.GetEnumName = function (TypeInfo, Value) {
    var Result = "";
    Result = TypeInfo.enumtype[Value];
    return Result;
  };
  this.GetStrProp$1 = function (Instance, PropInfo) {
    var Result = "";
    Result = "" + $mod.GetJSValueProp$3(Instance,PropInfo);
    return Result;
  };
  this.GetFloatProp$1 = function (Instance, PropInfo) {
    var Result = 0.0;
    Result = rtl.getNumber($mod.GetJSValueProp$3(Instance,PropInfo));
    return Result;
  };
  this.GetObjectProp$2 = function (Instance, PropInfo) {
    var Result = null;
    Result = $mod.GetObjectProp$3(Instance,PropInfo,null);
    return Result;
  };
  this.GetObjectProp$3 = function (Instance, PropInfo, MinClass) {
    var Result = null;
    var O = null;
    O = rtl.getObject($mod.GetJSValueProp$3(Instance,PropInfo));
    if ((MinClass !== null) && !O.$class.InheritsFrom(MinClass)) {
      Result = null}
     else Result = O;
    return Result;
  };
  this.GetMethodProp = function (Instance, PropInfo) {
    var Result = pas.System.TMethod.$new();
    var v = undefined;
    Result.Code = null;
    Result.Data = null;
    v = $mod.GetJSValueProp$3(Instance,PropInfo);
    if (!rtl.isFunction(v)) return Result;
    Result.Data = rtl.getObject(v)["scope"];
    Result.Code = rtl.getObject(v)["fn"];
    return Result;
  };
  this.GetInterfaceProp$1 = function (Instance, PropInfo) {
    var Result = null;
    var gk = 0;
    var $ok = false;
    try {
      if (PropInfo.typeinfo.kind !== 18) throw pas.SysUtils.Exception.$create("Create$1",["Cannot get RAW interface from IInterface interface"]);
      gk = $impl.GetPropGetterKind(PropInfo);
      var $tmp1 = gk;
      if ($tmp1 === 0) {
        throw $mod.EPropertyError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SCantReadPropertyS"),[PropInfo.name]])}
       else if ($tmp1 === 1) {
        Result = rtl.setIntfL(Result,rtl.getObject(Instance[PropInfo.getter]))}
       else if ($tmp1 === 2) {
        if ((16 & PropInfo.flags) > 0) {
          Result = rtl.setIntfL(Result,Instance[PropInfo.getter](PropInfo.index),true)}
         else Result = rtl.setIntfL(Result,Instance[PropInfo.getter](),true)}
       else if ($tmp1 === 3) throw $mod.EPropertyError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SIndexedPropertyNeedsParams"),[PropInfo.name]]);
      $ok = true;
    } finally {
      if (!$ok) rtl._Release(Result);
    };
    return Result;
  };
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.TGetterKind = {"0": "gkNone", gkNone: 0, "1": "gkField", gkField: 1, "2": "gkFunction", gkFunction: 2, "3": "gkFunctionWithParams", gkFunctionWithParams: 3};
  $impl.GetPropGetterKind = function (PropInfo) {
    var Result = 0;
    if (PropInfo.getter === "") {
      Result = 0}
     else if ((1 & PropInfo.flags) > 0) {
      if (rtl.length(PropInfo.params) > 0) {
        Result = 3}
       else Result = 2;
    } else Result = 1;
    return Result;
  };
});
rtl.module("Classes",["System","RTLConsts","Types","SysUtils","JS","TypInfo"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $mod.$rtti.$MethodVar("TNotifyEvent",{procsig: rtl.newTIProcSig([["Sender",pas.System.$rtti["TObject"]]]), methodkind: 0});
  rtl.createClass($mod,"EStreamError",pas.SysUtils.Exception,function () {
  });
  rtl.createClass($mod,"EFilerError",$mod.EStreamError,function () {
  });
  rtl.createClass($mod,"EReadError",$mod.EFilerError,function () {
  });
  rtl.createClass($mod,"EWriteError",$mod.EFilerError,function () {
  });
  rtl.createClass($mod,"EListError",pas.SysUtils.Exception,function () {
  });
  rtl.createClass($mod,"EStringListError",$mod.EListError,function () {
  });
  rtl.createClass($mod,"EComponentError",pas.SysUtils.Exception,function () {
  });
  rtl.createClass($mod,"TFPList",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FList = [];
      this.FCount = 0;
      this.FCapacity = 0;
    };
    this.$final = function () {
      this.FList = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.Get = function (Index) {
      var Result = undefined;
      if ((Index < 0) || (Index >= this.FCount)) this.RaiseIndexError(Index);
      Result = this.FList[Index];
      return Result;
    };
    this.SetCapacity = function (NewCapacity) {
      if (NewCapacity < this.FCount) this.$class.Error(rtl.getResStr(pas.RTLConsts,"SListCapacityError"),"" + NewCapacity);
      if (NewCapacity === this.FCapacity) return;
      this.FList = rtl.arraySetLength(this.FList,undefined,NewCapacity);
      this.FCapacity = NewCapacity;
    };
    this.SetCount = function (NewCount) {
      if (NewCount < 0) this.$class.Error(rtl.getResStr(pas.RTLConsts,"SListCountError"),"" + NewCount);
      if (NewCount > this.FCount) {
        if (NewCount > this.FCapacity) this.SetCapacity(NewCount);
      };
      this.FCount = NewCount;
    };
    this.RaiseIndexError = function (Index) {
      this.$class.Error(rtl.getResStr(pas.RTLConsts,"SListIndexError"),"" + Index);
    };
    this.Destroy = function () {
      this.Clear();
      pas.System.TObject.Destroy.call(this);
    };
    this.Add = function (Item) {
      var Result = 0;
      if (this.FCount === this.FCapacity) this.Expand();
      this.FList[this.FCount] = Item;
      Result = this.FCount;
      this.FCount += 1;
      return Result;
    };
    this.Clear = function () {
      if (rtl.length(this.FList) > 0) {
        this.SetCount(0);
        this.SetCapacity(0);
      };
    };
    this.Delete = function (Index) {
      if ((Index < 0) || (Index >= this.FCount)) this.$class.Error(rtl.getResStr(pas.RTLConsts,"SListIndexError"),"" + Index);
      this.FCount = this.FCount - 1;
      this.FList.splice(Index,1);
      this.FCapacity -= 1;
    };
    this.Error = function (Msg, Data) {
      throw $mod.EListError.$create("CreateFmt",[Msg,[Data]]);
    };
    this.Expand = function () {
      var Result = null;
      var IncSize = 0;
      if (this.FCount < this.FCapacity) return this;
      IncSize = 4;
      if (this.FCapacity > 3) IncSize = IncSize + 4;
      if (this.FCapacity > 8) IncSize = IncSize + 8;
      if (this.FCapacity > 127) IncSize += this.FCapacity >>> 2;
      this.SetCapacity(this.FCapacity + IncSize);
      Result = this;
      return Result;
    };
    this.IndexOf = function (Item) {
      var Result = 0;
      var C = 0;
      Result = 0;
      C = this.FCount;
      while ((Result < C) && (this.FList[Result] != Item)) Result += 1;
      if (Result >= C) Result = -1;
      return Result;
    };
    this.IndexOfItem = function (Item, Direction) {
      var Result = 0;
      if (Direction === 0) {
        Result = this.IndexOf(Item)}
       else {
        Result = this.FCount - 1;
        while ((Result >= 0) && (this.FList[Result] != Item)) Result = Result - 1;
      };
      return Result;
    };
    this.Last = function () {
      var Result = undefined;
      if (this.FCount === 0) {
        Result = null}
       else Result = this.Get(this.FCount - 1);
      return Result;
    };
    this.Remove = function (Item) {
      var Result = 0;
      Result = this.IndexOf(Item);
      if (Result !== -1) this.Delete(Result);
      return Result;
    };
  });
  this.TListNotification = {"0": "lnAdded", lnAdded: 0, "1": "lnExtracted", lnExtracted: 1, "2": "lnDeleted", lnDeleted: 2};
  rtl.createClass($mod,"TList",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FList = null;
    };
    this.$final = function () {
      this.FList = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.Get = function (Index) {
      var Result = undefined;
      Result = this.FList.Get(Index);
      return Result;
    };
    this.Notify = function (aValue, Action) {
      if (pas.System.Assigned(aValue)) ;
      if (Action === 1) ;
    };
    this.GetCount = function () {
      var Result = 0;
      Result = this.FList.FCount;
      return Result;
    };
    this.Create$1 = function () {
      pas.System.TObject.Create.call(this);
      this.FList = $mod.TFPList.$create("Create");
      return this;
    };
    this.Destroy = function () {
      if (this.FList != null) this.Clear();
      pas.SysUtils.FreeAndNil({p: this, get: function () {
          return this.p.FList;
        }, set: function (v) {
          this.p.FList = v;
        }});
    };
    this.Add = function (Item) {
      var Result = 0;
      Result = this.FList.Add(Item);
      if (pas.System.Assigned(Item)) this.Notify(Item,0);
      return Result;
    };
    this.Clear = function () {
      while (this.FList.FCount > 0) this.Delete(this.GetCount() - 1);
    };
    this.Delete = function (Index) {
      var V = undefined;
      V = this.FList.Get(Index);
      this.FList.Delete(Index);
      if (pas.System.Assigned(V)) this.Notify(V,2);
    };
  });
  rtl.createClass($mod,"TPersistent",pas.System.TObject,function () {
    this.AssignError = function (Source) {
      var SourceName = "";
      if (Source !== null) {
        SourceName = Source.$classname}
       else SourceName = "Nil";
      throw pas.SysUtils.EConvertError.$create("Create$1",["Cannot assign a " + SourceName + " to a " + this.$classname + "."]);
    };
    this.DefineProperties = function (Filer) {
      if (Filer === null) return;
    };
    this.AssignTo = function (Dest) {
      Dest.AssignError(this);
    };
    this.GetOwner = function () {
      var Result = null;
      Result = null;
      return Result;
    };
    this.Assign = function (Source) {
      if (Source !== null) {
        Source.AssignTo(this)}
       else this.AssignError(null);
    };
  });
  rtl.createClass($mod,"TStrings",$mod.TPersistent,function () {
    this.$init = function () {
      $mod.TPersistent.$init.call(this);
      this.FSpecialCharsInited = false;
      this.FAlwaysQuote = false;
      this.FQuoteChar = "";
      this.FDelimiter = "";
      this.FNameValueSeparator = "";
      this.FUpdateCount = 0;
      this.FLBS = 0;
      this.FSkipLastLineBreak = false;
      this.FLineBreak = "";
    };
    this.DoSetTextStr = function (Value, DoClear) {
      var S = "";
      var P = 0;
      try {
        this.BeginUpdate();
        if (DoClear) this.Clear();
        P = 1;
        while (this.GetNextLinebreak(Value,{get: function () {
            return S;
          }, set: function (v) {
            S = v;
          }},{get: function () {
            return P;
          }, set: function (v) {
            P = v;
          }})) this.Add(S);
      } finally {
        this.EndUpdate();
      };
    };
    this.GetLineBreak = function () {
      var Result = "";
      this.CheckSpecialChars();
      Result = this.FLineBreak;
      return Result;
    };
    this.GetSkipLastLineBreak = function () {
      var Result = false;
      this.CheckSpecialChars();
      Result = this.FSkipLastLineBreak;
      return Result;
    };
    this.Error = function (Msg, Data) {
      throw $mod.EStringListError.$create("CreateFmt",[Msg,[pas.SysUtils.IntToStr(Data)]]);
    };
    this.GetCapacity = function () {
      var Result = 0;
      Result = this.GetCount();
      return Result;
    };
    this.GetObject = function (Index) {
      var Result = null;
      if (Index === 0) ;
      Result = null;
      return Result;
    };
    this.GetTextStr = function () {
      var Result = "";
      var I = 0;
      var S = "";
      var NL = "";
      this.CheckSpecialChars();
      if (this.FLineBreak !== pas.System.sLineBreak) {
        NL = this.FLineBreak}
       else {
        var $tmp1 = this.FLBS;
        if ($tmp1 === 0) {
          NL = "\n"}
         else if ($tmp1 === 1) {
          NL = "\r\n"}
         else if ($tmp1 === 2) NL = "\r";
      };
      Result = "";
      for (var $l2 = 0, $end3 = this.GetCount() - 1; $l2 <= $end3; $l2++) {
        I = $l2;
        S = this.Get(I);
        Result = Result + S;
        if ((I < (this.GetCount() - 1)) || !this.GetSkipLastLineBreak()) Result = Result + NL;
      };
      return Result;
    };
    this.Put = function (Index, S) {
      var Obj = null;
      Obj = this.GetObject(Index);
      this.Delete(Index);
      this.InsertObject(Index,S,Obj);
    };
    this.PutObject = function (Index, AObject) {
      if (Index === 0) return;
      if (AObject === null) return;
    };
    this.SetTextStr = function (Value) {
      this.CheckSpecialChars();
      this.DoSetTextStr(Value,true);
    };
    this.SetUpdateState = function (Updating) {
      if (Updating) ;
    };
    this.DoCompareText = function (s1, s2) {
      var Result = 0;
      Result = pas.SysUtils.CompareText(s1,s2);
      return Result;
    };
    this.CheckSpecialChars = function () {
      if (!this.FSpecialCharsInited) {
        this.FQuoteChar = '"';
        this.FDelimiter = ",";
        this.FNameValueSeparator = "=";
        this.FLBS = pas.System.DefaultTextLineBreakStyle;
        this.FSpecialCharsInited = true;
        this.FLineBreak = pas.System.sLineBreak;
      };
    };
    this.GetNextLinebreak = function (Value, S, P) {
      var Result = false;
      var PP = 0;
      S.set("");
      Result = false;
      if ((Value.length - P.get()) < 0) return Result;
      PP = Value.indexOf(this.GetLineBreak(),P.get() - 1) + 1;
      if (PP < 1) PP = Value.length + 1;
      S.set(pas.System.Copy(Value,P.get(),PP - P.get()));
      P.set(PP + this.GetLineBreak().length);
      Result = true;
      return Result;
    };
    this.Create$1 = function () {
      pas.System.TObject.Create.call(this);
      this.FAlwaysQuote = false;
      return this;
    };
    this.Destroy = function () {
      pas.System.TObject.Destroy.call(this);
    };
    this.Add = function (S) {
      var Result = 0;
      Result = this.GetCount();
      this.Insert(this.GetCount(),S);
      return Result;
    };
    this.AddObject = function (S, AObject) {
      var Result = 0;
      Result = this.Add(S);
      this.PutObject(Result,AObject);
      return Result;
    };
    this.AddStrings = function (TheStrings) {
      var Runner = 0;
      for (var $l1 = 0, $end2 = TheStrings.GetCount() - 1; $l1 <= $end2; $l1++) {
        Runner = $l1;
        this.AddObject(TheStrings.Get(Runner),TheStrings.GetObject(Runner));
      };
    };
    this.Assign = function (Source) {
      var S = null;
      if ($mod.TStrings.isPrototypeOf(Source)) {
        S = Source;
        this.BeginUpdate();
        try {
          this.Clear();
          this.FSpecialCharsInited = S.FSpecialCharsInited;
          this.FQuoteChar = S.FQuoteChar;
          this.FDelimiter = S.FDelimiter;
          this.FNameValueSeparator = S.FNameValueSeparator;
          this.FLBS = S.FLBS;
          this.FLineBreak = S.FLineBreak;
          this.AddStrings(S);
        } finally {
          this.EndUpdate();
        };
      } else $mod.TPersistent.Assign.call(this,Source);
    };
    this.BeginUpdate = function () {
      if (this.FUpdateCount === 0) this.SetUpdateState(true);
      this.FUpdateCount += 1;
    };
    this.EndUpdate = function () {
      if (this.FUpdateCount > 0) this.FUpdateCount -= 1;
      if (this.FUpdateCount === 0) this.SetUpdateState(false);
    };
    this.IndexOf = function (S) {
      var Result = 0;
      Result = 0;
      while ((Result < this.GetCount()) && (this.DoCompareText(this.Get(Result),S) !== 0)) Result = Result + 1;
      if (Result === this.GetCount()) Result = -1;
      return Result;
    };
    this.InsertObject = function (Index, S, AObject) {
      this.Insert(Index,S);
      this.PutObject(Index,AObject);
    };
  });
  rtl.recNewT($mod,"TStringItem",function () {
    this.FString = "";
    this.FObject = null;
    this.$eq = function (b) {
      return (this.FString === b.FString) && (this.FObject === b.FObject);
    };
    this.$assign = function (s) {
      this.FString = s.FString;
      this.FObject = s.FObject;
      return this;
    };
  });
  this.TStringsSortStyle = {"0": "sslNone", sslNone: 0, "1": "sslUser", sslUser: 1, "2": "sslAuto", sslAuto: 2};
  rtl.createClass($mod,"TStringList",$mod.TStrings,function () {
    this.$init = function () {
      $mod.TStrings.$init.call(this);
      this.FList = [];
      this.FCount = 0;
      this.FOnChange = null;
      this.FOnChanging = null;
      this.FDuplicates = 0;
      this.FCaseSensitive = false;
      this.FForceSort = false;
      this.FOwnsObjects = false;
      this.FSortStyle = 0;
    };
    this.$final = function () {
      this.FList = undefined;
      this.FOnChange = undefined;
      this.FOnChanging = undefined;
      $mod.TStrings.$final.call(this);
    };
    this.ExchangeItemsInt = function (Index1, Index2) {
      var S = "";
      var O = null;
      S = this.FList[Index1].FString;
      O = this.FList[Index1].FObject;
      this.FList[Index1].FString = this.FList[Index2].FString;
      this.FList[Index1].FObject = this.FList[Index2].FObject;
      this.FList[Index2].FString = S;
      this.FList[Index2].FObject = O;
    };
    this.GetSorted = function () {
      var Result = false;
      Result = this.FSortStyle in rtl.createSet(1,2);
      return Result;
    };
    this.Grow = function () {
      var NC = 0;
      NC = this.GetCapacity();
      if (NC >= 256) {
        NC = NC + Math.floor(NC / 4)}
       else if (NC === 0) {
        NC = 4}
       else NC = NC * 4;
      this.SetCapacity(NC);
    };
    this.InternalClear = function (FromIndex, ClearOnly) {
      var I = 0;
      if (FromIndex < this.FCount) {
        if (this.FOwnsObjects) {
          for (var $l1 = FromIndex, $end2 = this.FCount - 1; $l1 <= $end2; $l1++) {
            I = $l1;
            this.FList[I].FString = "";
            pas.SysUtils.FreeAndNil({p: this.FList[I], get: function () {
                return this.p.FObject;
              }, set: function (v) {
                this.p.FObject = v;
              }});
          };
        } else {
          for (var $l3 = FromIndex, $end4 = this.FCount - 1; $l3 <= $end4; $l3++) {
            I = $l3;
            this.FList[I].FString = "";
          };
        };
        this.FCount = FromIndex;
      };
      if (!ClearOnly) this.SetCapacity(0);
    };
    this.QuickSort = function (L, R, CompareFn) {
      var Pivot = 0;
      var vL = 0;
      var vR = 0;
      if ((R - L) <= 1) {
        if (L < R) if (CompareFn(this,L,R) > 0) this.ExchangeItems(L,R);
        return;
      };
      vL = L;
      vR = R;
      Pivot = L + pas.System.Random(R - L);
      while (vL < vR) {
        while ((vL < Pivot) && (CompareFn(this,vL,Pivot) <= 0)) vL += 1;
        while ((vR > Pivot) && (CompareFn(this,vR,Pivot) > 0)) vR -= 1;
        this.ExchangeItems(vL,vR);
        if (Pivot === vL) {
          Pivot = vR}
         else if (Pivot === vR) Pivot = vL;
      };
      if ((Pivot - 1) >= L) this.QuickSort(L,Pivot - 1,CompareFn);
      if ((Pivot + 1) <= R) this.QuickSort(Pivot + 1,R,CompareFn);
    };
    this.SetSorted = function (Value) {
      if (Value) {
        this.SetSortStyle(2)}
       else this.SetSortStyle(0);
    };
    this.SetSortStyle = function (AValue) {
      if (this.FSortStyle === AValue) return;
      if (AValue === 2) this.Sort();
      this.FSortStyle = AValue;
    };
    this.CheckIndex = function (AIndex) {
      if ((AIndex < 0) || (AIndex >= this.FCount)) this.Error(rtl.getResStr(pas.RTLConsts,"SListIndexError"),AIndex);
    };
    this.ExchangeItems = function (Index1, Index2) {
      this.ExchangeItemsInt(Index1,Index2);
    };
    this.Changed = function () {
      if (this.FUpdateCount === 0) {
        if (this.FOnChange != null) this.FOnChange(this);
      };
    };
    this.Changing = function () {
      if (this.FUpdateCount === 0) if (this.FOnChanging != null) this.FOnChanging(this);
    };
    this.Get = function (Index) {
      var Result = "";
      this.CheckIndex(Index);
      Result = this.FList[Index].FString;
      return Result;
    };
    this.GetCapacity = function () {
      var Result = 0;
      Result = rtl.length(this.FList);
      return Result;
    };
    this.GetCount = function () {
      var Result = 0;
      Result = this.FCount;
      return Result;
    };
    this.GetObject = function (Index) {
      var Result = null;
      this.CheckIndex(Index);
      Result = this.FList[Index].FObject;
      return Result;
    };
    this.Put = function (Index, S) {
      if (this.GetSorted()) this.Error(rtl.getResStr(pas.RTLConsts,"SSortedListError"),0);
      this.CheckIndex(Index);
      this.Changing();
      this.FList[Index].FString = S;
      this.Changed();
    };
    this.PutObject = function (Index, AObject) {
      this.CheckIndex(Index);
      this.Changing();
      this.FList[Index].FObject = AObject;
      this.Changed();
    };
    this.SetCapacity = function (NewCapacity) {
      if (NewCapacity < 0) this.Error(rtl.getResStr(pas.RTLConsts,"SListCapacityError"),NewCapacity);
      if (NewCapacity !== this.GetCapacity()) this.FList = rtl.arraySetLength(this.FList,$mod.TStringItem,NewCapacity);
    };
    this.SetUpdateState = function (Updating) {
      if (Updating) {
        this.Changing()}
       else this.Changed();
    };
    this.InsertItem = function (Index, S) {
      this.InsertItem$1(Index,S,null);
    };
    this.InsertItem$1 = function (Index, S, O) {
      var It = $mod.TStringItem.$new();
      this.Changing();
      if (this.FCount === this.GetCapacity()) this.Grow();
      It.FString = S;
      It.FObject = O;
      this.FList.splice(Index,0,It);
      this.FCount += 1;
      this.Changed();
    };
    this.DoCompareText = function (s1, s2) {
      var Result = 0;
      if (this.FCaseSensitive) {
        Result = pas.SysUtils.CompareStr(s1,s2)}
       else Result = pas.SysUtils.CompareText(s1,s2);
      return Result;
    };
    this.Destroy = function () {
      this.InternalClear(0,false);
      $mod.TStrings.Destroy.call(this);
    };
    this.Add = function (S) {
      var Result = 0;
      if (!(this.FSortStyle === 2)) {
        Result = this.FCount}
       else if (this.Find(S,{get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }})) {
        var $tmp1 = this.FDuplicates;
        if ($tmp1 === 0) {
          return Result}
         else if ($tmp1 === 2) this.Error(rtl.getResStr(pas.RTLConsts,"SDuplicateString"),0);
      };
      this.InsertItem(Result,S);
      return Result;
    };
    this.Clear = function () {
      if (this.FCount === 0) return;
      this.Changing();
      this.InternalClear(0,false);
      this.Changed();
    };
    this.Delete = function (Index) {
      this.CheckIndex(Index);
      this.Changing();
      if (this.FOwnsObjects) pas.SysUtils.FreeAndNil({p: this.FList[Index], get: function () {
          return this.p.FObject;
        }, set: function (v) {
          this.p.FObject = v;
        }});
      this.FList.splice(Index,1);
      this.FList[this.GetCount() - 1].FString = "";
      this.FList[this.GetCount() - 1].FObject = null;
      this.FCount -= 1;
      this.Changed();
    };
    this.Find = function (S, Index) {
      var Result = false;
      var L = 0;
      var R = 0;
      var I = 0;
      var CompareRes = 0;
      Result = false;
      Index.set(-1);
      if (!this.GetSorted()) throw $mod.EListError.$create("Create$1",[rtl.getResStr(pas.RTLConsts,"SErrFindNeedsSortedList")]);
      L = 0;
      R = this.GetCount() - 1;
      while (L <= R) {
        I = L + Math.floor((R - L) / 2);
        CompareRes = this.DoCompareText(S,this.FList[I].FString);
        if (CompareRes > 0) {
          L = I + 1}
         else {
          R = I - 1;
          if (CompareRes === 0) {
            Result = true;
            if (this.FDuplicates !== 1) L = I;
          };
        };
      };
      Index.set(L);
      return Result;
    };
    this.IndexOf = function (S) {
      var Result = 0;
      if (!this.GetSorted()) {
        Result = $mod.TStrings.IndexOf.call(this,S)}
       else if (!this.Find(S,{get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }})) Result = -1;
      return Result;
    };
    this.Insert = function (Index, S) {
      if (this.FSortStyle === 2) {
        this.Error(rtl.getResStr(pas.RTLConsts,"SSortedListError"),0)}
       else {
        if ((Index < 0) || (Index > this.FCount)) this.Error(rtl.getResStr(pas.RTLConsts,"SListIndexError"),Index);
        this.InsertItem(Index,S);
      };
    };
    this.Sort = function () {
      this.CustomSort($impl.StringListAnsiCompare);
    };
    this.CustomSort = function (CompareFn) {
      if ((this.FForceSort || !(this.FSortStyle === 2)) && (this.FCount > 1)) {
        this.Changing();
        this.QuickSort(0,this.FCount - 1,CompareFn);
        this.Changed();
      };
    };
  });
  rtl.createClass($mod,"TCollectionItem",$mod.TPersistent,function () {
    this.$init = function () {
      $mod.TPersistent.$init.call(this);
      this.FCollection = null;
      this.FID = 0;
    };
    this.$final = function () {
      this.FCollection = undefined;
      $mod.TPersistent.$final.call(this);
    };
    this.SetCollection = function (Value) {
      if (Value !== this.FCollection) {
        if (this.FCollection !== null) this.FCollection.RemoveItem(this);
        if (Value !== null) Value.InsertItem(this);
      };
    };
    this.GetOwner = function () {
      var Result = null;
      Result = this.FCollection;
      return Result;
    };
    this.Create$1 = function (ACollection) {
      pas.System.TObject.Create.call(this);
      this.SetCollection(ACollection);
      return this;
    };
    this.Destroy = function () {
      this.SetCollection(null);
      pas.System.TObject.Destroy.call(this);
    };
  });
  this.TCollectionNotification = {"0": "cnAdded", cnAdded: 0, "1": "cnExtracting", cnExtracting: 1, "2": "cnDeleting", cnDeleting: 2};
  rtl.createClass($mod,"TCollection",$mod.TPersistent,function () {
    this.$init = function () {
      $mod.TPersistent.$init.call(this);
      this.FItemClass = null;
      this.FItems = null;
      this.FUpdateCount = 0;
      this.FNextID = 0;
    };
    this.$final = function () {
      this.FItemClass = undefined;
      this.FItems = undefined;
      $mod.TPersistent.$final.call(this);
    };
    this.GetCount = function () {
      var Result = 0;
      Result = this.FItems.FCount;
      return Result;
    };
    this.InsertItem = function (Item) {
      if (!this.FItemClass.isPrototypeOf(Item)) return;
      this.FItems.Add(Item);
      Item.FCollection = this;
      Item.FID = this.FNextID;
      this.FNextID += 1;
      this.SetItemName(Item);
      this.Notify(Item,0);
      this.Changed();
    };
    this.RemoveItem = function (Item) {
      var I = 0;
      this.Notify(Item,1);
      I = this.FItems.IndexOfItem(Item,1);
      if (I !== -1) this.FItems.Delete(I);
      Item.FCollection = null;
      this.Changed();
    };
    this.DoClear = function () {
      var Item = null;
      while (this.FItems.FCount > 0) {
        Item = rtl.getObject(this.FItems.Last());
        if (Item != null) Item.$destroy("Destroy");
      };
    };
    this.Changed = function () {
      if (this.FUpdateCount === 0) this.Update(null);
    };
    this.GetItem = function (Index) {
      var Result = null;
      Result = rtl.getObject(this.FItems.Get(Index));
      return Result;
    };
    this.SetItemName = function (Item) {
      if (Item === null) ;
    };
    this.Update = function (Item) {
      if (Item === null) ;
    };
    this.Notify = function (Item, Action) {
      if (Item === null) ;
      if (Action === 0) ;
    };
    this.Create$1 = function (AItemClass) {
      pas.System.TObject.Create.call(this);
      this.FItemClass = AItemClass;
      this.FItems = $mod.TFPList.$create("Create");
      return this;
    };
    this.Destroy = function () {
      this.FUpdateCount = 1;
      try {
        this.DoClear();
      } finally {
        this.FUpdateCount = 0;
      };
      if (this.FItems != null) this.FItems.$destroy("Destroy");
      pas.System.TObject.Destroy.call(this);
    };
    this.Owner = function () {
      var Result = null;
      Result = this.GetOwner();
      return Result;
    };
    this.Add = function () {
      var Result = null;
      Result = this.FItemClass.$create("Create$1",[this]);
      return Result;
    };
    this.Assign = function (Source) {
      var I = 0;
      if ($mod.TCollection.isPrototypeOf(Source)) {
        this.Clear();
        for (var $l1 = 0, $end2 = Source.GetCount() - 1; $l1 <= $end2; $l1++) {
          I = $l1;
          this.Add().Assign(Source.GetItem(I));
        };
        return;
      } else $mod.TPersistent.Assign.call(this,Source);
    };
    this.BeginUpdate = function () {
      this.FUpdateCount += 1;
    };
    this.Clear = function () {
      if (this.FItems.FCount === 0) return;
      this.BeginUpdate();
      try {
        this.DoClear();
      } finally {
        this.EndUpdate();
      };
    };
    this.EndUpdate = function () {
      if (this.FUpdateCount > 0) this.FUpdateCount -= 1;
      if (this.FUpdateCount === 0) this.Changed();
    };
    this.Delete = function (Index) {
      var Item = null;
      Item = rtl.getObject(this.FItems.Get(Index));
      this.Notify(Item,2);
      if (Item != null) Item.$destroy("Destroy");
    };
  });
  rtl.createClass($mod,"TOwnedCollection",$mod.TCollection,function () {
    this.$init = function () {
      $mod.TCollection.$init.call(this);
      this.FOwner = null;
    };
    this.$final = function () {
      this.FOwner = undefined;
      $mod.TCollection.$final.call(this);
    };
    this.GetOwner = function () {
      var Result = null;
      Result = this.FOwner;
      return Result;
    };
    this.Create$2 = function (AOwner, AItemClass) {
      this.FOwner = AOwner;
      $mod.TCollection.Create$1.call(this,AItemClass);
      return this;
    };
  });
  this.TOperation = {"0": "opInsert", opInsert: 0, "1": "opRemove", opRemove: 1};
  this.TComponentStateItem = {"0": "csLoading", csLoading: 0, "1": "csReading", csReading: 1, "2": "csWriting", csWriting: 2, "3": "csDestroying", csDestroying: 3, "4": "csDesigning", csDesigning: 4, "5": "csAncestor", csAncestor: 5, "6": "csUpdating", csUpdating: 6, "7": "csFixups", csFixups: 7, "8": "csFreeNotification", csFreeNotification: 8, "9": "csInline", csInline: 9, "10": "csDesignInstance", csDesignInstance: 10};
  this.TComponentStyleItem = {"0": "csInheritable", csInheritable: 0, "1": "csCheckPropAvail", csCheckPropAvail: 1, "2": "csSubComponent", csSubComponent: 2, "3": "csTransient", csTransient: 3};
  rtl.createClass($mod,"TComponent",$mod.TPersistent,function () {
    this.$init = function () {
      $mod.TPersistent.$init.call(this);
      this.FOwner = null;
      this.FName = "";
      this.FTag = 0;
      this.FComponents = null;
      this.FFreeNotifies = null;
      this.FComponentState = {};
      this.FComponentStyle = {};
    };
    this.$final = function () {
      this.FOwner = undefined;
      this.FComponents = undefined;
      this.FFreeNotifies = undefined;
      this.FComponentState = undefined;
      this.FComponentStyle = undefined;
      $mod.TPersistent.$final.call(this);
    };
    this.Insert = function (AComponent) {
      if (!(this.FComponents != null)) this.FComponents = $mod.TFPList.$create("Create");
      this.FComponents.Add(AComponent);
      AComponent.FOwner = this;
    };
    this.Remove = function (AComponent) {
      AComponent.FOwner = null;
      if (this.FComponents != null) {
        this.FComponents.Remove(AComponent);
        if (this.FComponents.FCount === 0) {
          this.FComponents.$destroy("Destroy");
          this.FComponents = null;
        };
      };
    };
    this.RemoveNotification = function (AComponent) {
      if (this.FFreeNotifies !== null) {
        this.FFreeNotifies.Remove(AComponent);
        if (this.FFreeNotifies.FCount === 0) {
          this.FFreeNotifies.$destroy("Destroy");
          this.FFreeNotifies = null;
          this.FComponentState = rtl.excludeSet(this.FComponentState,8);
        };
      };
    };
    this.SetReference = function (Enable) {
      var aField = null;
      var aValue = null;
      var aOwner = null;
      if (this.FName === "") return;
      if (this.FOwner != null) {
        aOwner = this.FOwner;
        aField = this.FOwner.$class.FieldAddress(this.FName);
        if (aField != null) {
          if (Enable) {
            aValue = this}
           else aValue = null;
          aOwner["" + aField["name"]] = aValue;
        };
      };
    };
    this.ChangeName = function (NewName) {
      this.FName = NewName;
    };
    this.GetChildren = function (Proc, Root) {
      if (Proc === null) ;
      if (Root === null) ;
    };
    this.GetOwner = function () {
      var Result = null;
      Result = this.FOwner;
      return Result;
    };
    this.Notification = function (AComponent, Operation) {
      var C = 0;
      if (Operation === 1) this.RemoveFreeNotification(AComponent);
      if (!(this.FComponents != null)) return;
      C = this.FComponents.FCount - 1;
      while (C >= 0) {
        rtl.getObject(this.FComponents.Get(C)).Notification(AComponent,Operation);
        C -= 1;
        if (C >= this.FComponents.FCount) C = this.FComponents.FCount - 1;
      };
    };
    this.SetDesigning = function (Value, SetChildren) {
      var Runner = 0;
      if (Value) {
        this.FComponentState = rtl.includeSet(this.FComponentState,4)}
       else this.FComponentState = rtl.excludeSet(this.FComponentState,4);
      if ((this.FComponents != null) && SetChildren) for (var $l1 = 0, $end2 = this.FComponents.FCount - 1; $l1 <= $end2; $l1++) {
        Runner = $l1;
        rtl.getObject(this.FComponents.Get(Runner)).SetDesigning(Value,true);
      };
    };
    this.SetName = function (NewName) {
      if (this.FName === NewName) return;
      if ((NewName !== "") && !pas.SysUtils.IsValidIdent(NewName,false,false)) throw $mod.EComponentError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SInvalidName"),[NewName]]);
      if (this.FOwner != null) {
        this.FOwner.ValidateRename(this,this.FName,NewName)}
       else this.ValidateRename(null,this.FName,NewName);
      this.SetReference(false);
      this.ChangeName(NewName);
      this.SetReference(true);
    };
    this.ValidateRename = function (AComponent, CurName, NewName) {
      if ((AComponent !== null) && (pas.SysUtils.CompareText(CurName,NewName) !== 0) && (AComponent.FOwner === this) && (this.FindComponent(NewName) !== null)) throw $mod.EComponentError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SDuplicateName"),[NewName]]);
      if ((4 in this.FComponentState) && (this.FOwner !== null)) this.FOwner.ValidateRename(AComponent,CurName,NewName);
    };
    this.ValidateContainer = function (AComponent) {
      AComponent.ValidateInsert(this);
    };
    this.ValidateInsert = function (AComponent) {
      if (AComponent === null) ;
    };
    this._AddRef = function () {
      var Result = 0;
      Result = -1;
      return Result;
    };
    this._Release = function () {
      var Result = 0;
      Result = -1;
      return Result;
    };
    this.Create$1 = function (AOwner) {
      this.FComponentStyle = rtl.createSet(0);
      if (AOwner != null) AOwner.InsertComponent(this);
      return this;
    };
    this.Destroy = function () {
      var I = 0;
      var C = null;
      this.Destroying();
      if (this.FFreeNotifies != null) {
        I = this.FFreeNotifies.FCount - 1;
        while (I >= 0) {
          C = rtl.getObject(this.FFreeNotifies.Get(I));
          this.FFreeNotifies.Delete(I);
          C.Notification(this,1);
          if (this.FFreeNotifies === null) {
            I = 0}
           else if (I > this.FFreeNotifies.FCount) I = this.FFreeNotifies.FCount;
          I -= 1;
        };
        pas.SysUtils.FreeAndNil({p: this, get: function () {
            return this.p.FFreeNotifies;
          }, set: function (v) {
            this.p.FFreeNotifies = v;
          }});
      };
      this.DestroyComponents();
      if (this.FOwner !== null) this.FOwner.RemoveComponent(this);
      pas.System.TObject.Destroy.call(this);
    };
    this.BeforeDestruction = function () {
      if (!(3 in this.FComponentState)) this.Destroying();
    };
    this.DestroyComponents = function () {
      var acomponent = null;
      while (this.FComponents != null) {
        acomponent = rtl.getObject(this.FComponents.Last());
        this.Remove(acomponent);
        acomponent.$destroy("Destroy");
      };
    };
    this.Destroying = function () {
      var Runner = 0;
      if (3 in this.FComponentState) return;
      this.FComponentState = rtl.includeSet(this.FComponentState,3);
      if (this.FComponents != null) for (var $l1 = 0, $end2 = this.FComponents.FCount - 1; $l1 <= $end2; $l1++) {
        Runner = $l1;
        rtl.getObject(this.FComponents.Get(Runner)).Destroying();
      };
    };
    this.QueryInterface = function (IID, Obj) {
      var Result = 0;
      if (this.GetInterface(IID,Obj)) {
        Result = 0}
       else Result = -2147467262;
      return Result;
    };
    this.WriteState = function (Writer) {
      Writer.WriteComponentData(this);
    };
    this.FindComponent = function (AName) {
      var Result = null;
      var I = 0;
      Result = null;
      if ((AName === "") || !(this.FComponents != null)) return Result;
      for (var $l1 = 0, $end2 = this.FComponents.FCount - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        if (pas.SysUtils.CompareText(rtl.getObject(this.FComponents.Get(I)).FName,AName) === 0) {
          Result = rtl.getObject(this.FComponents.Get(I));
          return Result;
        };
      };
      return Result;
    };
    this.RemoveFreeNotification = function (AComponent) {
      this.RemoveNotification(AComponent);
      AComponent.RemoveNotification(this);
    };
    this.InsertComponent = function (AComponent) {
      AComponent.ValidateContainer(this);
      this.ValidateRename(AComponent,"",AComponent.FName);
      this.Insert(AComponent);
      if (4 in this.FComponentState) AComponent.SetDesigning(true,true);
      this.Notification(AComponent,0);
    };
    this.RemoveComponent = function (AComponent) {
      this.Notification(AComponent,1);
      this.Remove(AComponent);
      AComponent.SetDesigning(false,true);
      this.ValidateRename(AComponent,AComponent.FName,"");
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Name",6,rtl.string,"FName","SetName");
    $r.addProperty("Tag",0,rtl.nativeint,"FTag","FTag",{Default: 0});
  });
  this.TSeekOrigin = {"0": "soBeginning", soBeginning: 0, "1": "soCurrent", soCurrent: 1, "2": "soEnd", soEnd: 2};
  rtl.createClass($mod,"TStream",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FEndian = 0;
    };
    this.MakeInt = function (B, aSize, Signed) {
      var Result = 0;
      var Mem = null;
      var A = null;
      var D = null;
      var isLittle = false;
      isLittle = this.FEndian === 0;
      Mem = new ArrayBuffer(rtl.length(B));
      A = new Uint8Array(Mem);
      A.set(B);
      D = new DataView(Mem);
      if (Signed) {
        var $tmp1 = aSize;
        if ($tmp1 === 1) {
          Result = D.getInt8(0)}
         else if ($tmp1 === 2) {
          Result = D.getInt16(0,isLittle)}
         else if ($tmp1 === 4) {
          Result = D.getInt32(0,isLittle)}
         else if ($tmp1 === 8) Result = Math.round(D.getFloat64(0,isLittle));
      } else {
        var $tmp2 = aSize;
        if ($tmp2 === 1) {
          Result = D.getUint8(0)}
         else if ($tmp2 === 2) {
          Result = D.getUint16(0,isLittle)}
         else if ($tmp2 === 4) {
          Result = D.getUint32(0,isLittle)}
         else if ($tmp2 === 8) Result = Math.round(D.getFloat64(0,isLittle));
      };
      return Result;
    };
    this.MakeBytes = function (B, aSize, Signed) {
      var Result = [];
      var Mem = null;
      var A = null;
      var D = null;
      var isLittle = false;
      isLittle = this.FEndian === 0;
      Mem = new ArrayBuffer(aSize);
      D = new DataView(Mem);
      if (Signed) {
        var $tmp1 = aSize;
        if ($tmp1 === 1) {
          D.setInt8(0,B)}
         else if ($tmp1 === 2) {
          D.setInt16(0,B,isLittle)}
         else if ($tmp1 === 4) {
          D.setInt32(0,B,isLittle)}
         else if ($tmp1 === 8) D.setFloat64(0,B,isLittle);
      } else {
        var $tmp2 = aSize;
        if ($tmp2 === 1) {
          D.setUint8(0,B)}
         else if ($tmp2 === 2) {
          D.setUint16(0,B,isLittle)}
         else if ($tmp2 === 4) {
          D.setUint32(0,B,isLittle)}
         else if ($tmp2 === 8) D.setFloat64(0,B,isLittle);
      };
      Result = rtl.arraySetLength(Result,0,aSize);
      A = new Uint8Array(Mem);
      Result = $mod.TMemoryStream.MemoryToBytes$1(A);
      return Result;
    };
    this.GetPosition = function () {
      var Result = 0;
      Result = this.Seek(0,1);
      return Result;
    };
    this.SetPosition = function (Pos) {
      this.Seek(Pos,0);
    };
    this.GetSize = function () {
      var Result = 0;
      var p = 0;
      p = this.Seek(0,1);
      Result = this.Seek(0,2);
      this.Seek(p,0);
      return Result;
    };
    this.ReadMaxSizeData = function (Buffer, aSize, aCount) {
      var Result = 0;
      var CP = 0;
      if (aCount <= aSize) {
        Result = this.Read({get: function () {
            return Buffer;
          }, set: function (v) {
            Buffer = v;
          }},aCount)}
       else {
        Result = this.Read({get: function () {
            return Buffer;
          }, set: function (v) {
            Buffer = v;
          }},aSize);
        CP = this.GetPosition();
        Result = (Result + this.Seek(aCount - aSize,1)) - CP;
      };
      return Result;
    };
    this.WriteMaxSizeData = function (Buffer, aSize, aCount) {
      var Result = 0;
      var CP = 0;
      if (aCount <= aSize) {
        Result = this.Write(Buffer,aCount)}
       else {
        Result = this.Write(Buffer,aSize);
        CP = this.GetPosition();
        Result = (Result + this.Seek(aCount - aSize,1)) - CP;
      };
      return Result;
    };
    this.Read = function (Buffer, Count) {
      var Result = 0;
      Result = this.Read$1(Buffer.get(),0,Count);
      return Result;
    };
    this.Write = function (Buffer, Count) {
      var Result = 0;
      Result = this.Write$1(Buffer,0,Count);
      return Result;
    };
    this.ReadData$4 = function (Buffer, Count) {
      var Result = 0;
      var W = 0;
      Result = this.ReadData$12({get: function () {
          return W;
        }, set: function (v) {
          W = v;
        }},Count);
      if (Result === 2) Buffer.set(String.fromCharCode(W));
      return Result;
    };
    this.ReadData$8 = function (Buffer, Count) {
      var Result = 0;
      var B = [];
      B = rtl.arraySetLength(B,0,Count);
      Result = this.ReadMaxSizeData(B,1,Count);
      if (Result >= 1) Buffer.set(this.MakeInt(B,1,false));
      return Result;
    };
    this.ReadData$12 = function (Buffer, Count) {
      var Result = 0;
      var B = [];
      B = rtl.arraySetLength(B,0,Count);
      Result = this.ReadMaxSizeData(B,2,Count);
      if (Result >= 2) Buffer.set(this.MakeInt(B,2,false));
      return Result;
    };
    this.ReadData$16 = function (Buffer, Count) {
      var Result = 0;
      var B = [];
      B = rtl.arraySetLength(B,0,Count);
      Result = this.ReadMaxSizeData(B,4,Count);
      if (Result >= 4) Buffer.set(this.MakeInt(B,4,false));
      return Result;
    };
    this.ReadData$18 = function (Buffer, Count) {
      var Result = 0;
      var B = [];
      B = rtl.arraySetLength(B,0,Count);
      Result = this.ReadMaxSizeData(B,8,8);
      if (Result >= 8) Buffer.set(this.MakeInt(B,8,true));
      return Result;
    };
    this.ReadData$22 = function (Buffer, Count) {
      var Result = 0;
      var B = [];
      var Mem = null;
      var A = null;
      var D = null;
      B = rtl.arraySetLength(B,0,Count);
      Result = this.ReadMaxSizeData(B,8,Count);
      if (Result >= 8) {
        Mem = new ArrayBuffer(8);
        A = new Uint8Array(Mem);
        A.set(B);
        D = new DataView(Mem);
        Buffer.set(D.getFloat64(0));
      };
      return Result;
    };
    this.ReadBuffer = function (Buffer, Count) {
      this.ReadBuffer$1(Buffer,0,Count);
    };
    this.ReadBuffer$1 = function (Buffer, Offset, Count) {
      if (this.Read$1(Buffer.get(),Offset,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$2 = function (Buffer) {
      this.ReadBufferData$3(Buffer,2);
    };
    this.ReadBufferData$3 = function (Buffer, Count) {
      if (this.ReadData$4(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$6 = function (Buffer) {
      this.ReadBufferData$7(Buffer,1);
    };
    this.ReadBufferData$7 = function (Buffer, Count) {
      if (this.ReadData$8(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$10 = function (Buffer) {
      this.ReadBufferData$11(Buffer,2);
    };
    this.ReadBufferData$11 = function (Buffer, Count) {
      if (this.ReadData$12(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$14 = function (Buffer) {
      this.ReadBufferData$15(Buffer,4);
    };
    this.ReadBufferData$15 = function (Buffer, Count) {
      if (this.ReadData$16(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$16 = function (Buffer) {
      this.ReadBufferData$17(Buffer,8);
    };
    this.ReadBufferData$17 = function (Buffer, Count) {
      if (this.ReadData$18(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.ReadBufferData$20 = function (Buffer) {
      this.ReadBufferData$21(Buffer,8);
    };
    this.ReadBufferData$21 = function (Buffer, Count) {
      if (this.ReadData$22(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SReadError")]);
    };
    this.WriteData$4 = function (Buffer, Count) {
      var Result = 0;
      var U = 0;
      U = Buffer.charCodeAt();
      Result = this.WriteData$12(U,Count);
      return Result;
    };
    this.WriteData$6 = function (Buffer, Count) {
      var Result = 0;
      Result = this.WriteMaxSizeData(this.MakeBytes(Buffer,1,true),1,Count);
      return Result;
    };
    this.WriteData$8 = function (Buffer, Count) {
      var Result = 0;
      Result = this.WriteMaxSizeData(this.MakeBytes(Buffer,1,false),1,Count);
      return Result;
    };
    this.WriteData$12 = function (Buffer, Count) {
      var Result = 0;
      Result = this.WriteMaxSizeData(this.MakeBytes(Buffer,2,true),2,Count);
      return Result;
    };
    this.WriteData$16 = function (Buffer, Count) {
      var Result = 0;
      Result = this.WriteMaxSizeData(this.MakeBytes(Buffer,4,false),4,Count);
      return Result;
    };
    this.WriteData$18 = function (Buffer, Count) {
      var Result = 0;
      Result = this.WriteMaxSizeData(this.MakeBytes(Buffer,8,true),8,Count);
      return Result;
    };
    this.WriteData$22 = function (Buffer, Count) {
      var Result = 0;
      var Mem = null;
      var A = null;
      var D = null;
      var B = [];
      var I = 0;
      Mem = new ArrayBuffer(8);
      D = new DataView(Mem);
      D.setFloat64(0,Buffer);
      B = rtl.arraySetLength(B,0,8);
      A = new Uint8Array(Mem);
      for (I = 0; I <= 7; I++) B[I] = A[I];
      Result = this.WriteMaxSizeData(B,8,Count);
      return Result;
    };
    this.WriteBufferData$4 = function (Buffer) {
      this.WriteBufferData$5(Buffer,2);
    };
    this.WriteBufferData$5 = function (Buffer, Count) {
      if (this.WriteData$4(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$6 = function (Buffer) {
      this.WriteBufferData$7(Buffer,1);
    };
    this.WriteBufferData$7 = function (Buffer, Count) {
      if (this.WriteData$6(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$8 = function (Buffer) {
      this.WriteBufferData$9(Buffer,1);
    };
    this.WriteBufferData$9 = function (Buffer, Count) {
      if (this.WriteData$8(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$12 = function (Buffer) {
      this.WriteBufferData$13(Buffer,2);
    };
    this.WriteBufferData$13 = function (Buffer, Count) {
      if (this.WriteData$12(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$14 = function (Buffer) {
      this.WriteBufferData$15(Buffer,4);
    };
    this.WriteBufferData$15 = function (Buffer, Count) {
      if (this.WriteData$16(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$16 = function (Buffer) {
      this.WriteBufferData$17(Buffer,8);
    };
    this.WriteBufferData$17 = function (Buffer, Count) {
      if (this.WriteData$18(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteBufferData$20 = function (Buffer) {
      this.WriteBufferData$21(Buffer,8);
    };
    this.WriteBufferData$21 = function (Buffer, Count) {
      if (this.WriteData$22(Buffer,Count) !== Count) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SWriteError")]);
    };
    this.WriteComponent = function (Instance) {
      this.WriteDescendent(Instance,null);
    };
    this.WriteDescendent = function (Instance, Ancestor) {
      var Driver = null;
      var Writer = null;
      Driver = $mod.TBinaryObjectWriter.$create("Create$1",[this]);
      try {
        Writer = $mod.TWriter.$create("Create$1",[Driver]);
        try {
          Writer.WriteDescendent(Instance,Ancestor);
        } finally {
          Writer.$destroy("Destroy");
        };
      } finally {
        Driver = rtl.freeLoc(Driver);
      };
    };
    this.ReadByte = function () {
      var Result = 0;
      this.ReadBufferData$7({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},1);
      return Result;
    };
    this.ReadDWord = function () {
      var Result = 0;
      this.ReadBufferData$15({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }},4);
      return Result;
    };
  });
  rtl.createClass($mod,"TCustomMemoryStream",$mod.TStream,function () {
    this.$init = function () {
      $mod.TStream.$init.call(this);
      this.FMemory = null;
      this.FDataView = null;
      this.FDataArray = null;
      this.FSize = 0;
      this.FPosition = 0;
      this.FSizeBoundsSeek = false;
    };
    this.$final = function () {
      this.FMemory = undefined;
      this.FDataView = undefined;
      this.FDataArray = undefined;
      $mod.TStream.$final.call(this);
    };
    this.GetDataArray = function () {
      var Result = null;
      if (this.FDataArray === null) this.FDataArray = new Uint8Array(this.FMemory);
      Result = this.FDataArray;
      return Result;
    };
    this.GetDataView = function () {
      var Result = null;
      if (this.FDataView === null) this.FDataView = new DataView(this.FMemory);
      Result = this.FDataView;
      return Result;
    };
    this.GetSize = function () {
      var Result = 0;
      Result = this.FSize;
      return Result;
    };
    this.GetPosition = function () {
      var Result = 0;
      Result = this.FPosition;
      return Result;
    };
    this.SetPointer = function (Ptr, ASize) {
      this.FMemory = Ptr;
      this.FSize = ASize;
      this.FDataView = null;
      this.FDataArray = null;
    };
    this.MemoryToBytes = function (Mem) {
      var Result = [];
      Result = this.MemoryToBytes$1(new Uint8Array(Mem));
      return Result;
    };
    this.MemoryToBytes$1 = function (Mem) {
      var Result = [];
      var I = 0;
      for (var $l1 = 0, $end2 = Mem.length - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        Result[I] = Mem[I];
      };
      return Result;
    };
    this.BytesToMemory = function (aBytes) {
      var Result = null;
      var a = null;
      Result = new ArrayBuffer(rtl.length(aBytes));
      a = new Uint8Array(Result);
      a.set(aBytes);
      return Result;
    };
    this.Read$1 = function (Buffer, Offset, Count) {
      var Result = 0;
      var I = 0;
      var Src = 0;
      var Dest = 0;
      Result = 0;
      if ((this.FSize > 0) && (this.FPosition < this.FSize) && (this.FPosition >= 0)) {
        Result = Count;
        if (Result > (this.FSize - this.FPosition)) Result = this.FSize - this.FPosition;
        Src = this.FPosition;
        Dest = Offset;
        I = 0;
        while (I < Result) {
          Buffer[Dest] = this.GetDataView().getUint8(Src);
          Src += 1;
          Dest += 1;
          I += 1;
        };
        this.FPosition = this.FPosition + Result;
      };
      return Result;
    };
    this.Seek = function (Offset, Origin) {
      var Result = 0;
      var $tmp1 = Origin;
      if ($tmp1 === 0) {
        this.FPosition = Offset}
       else if ($tmp1 === 2) {
        this.FPosition = this.FSize + Offset}
       else if ($tmp1 === 1) this.FPosition = this.FPosition + Offset;
      if (this.FSizeBoundsSeek && (this.FPosition > this.FSize)) this.FPosition = this.FSize;
      Result = this.FPosition;
      return Result;
    };
  });
  rtl.createClass($mod,"TMemoryStream",$mod.TCustomMemoryStream,function () {
    this.$init = function () {
      $mod.TCustomMemoryStream.$init.call(this);
      this.FCapacity = 0;
    };
    this.SetCapacity = function (NewCapacity) {
      this.SetPointer(this.Realloc({get: function () {
          return NewCapacity;
        }, set: function (v) {
          NewCapacity = v;
        }}),this.FSize);
      this.FCapacity = NewCapacity;
    };
    this.Realloc = function (NewCapacity) {
      var Result = null;
      var GC = 0;
      var DestView = null;
      if (NewCapacity.get() < 0) {
        NewCapacity.set(0)}
       else {
        GC = this.FCapacity + Math.floor(this.FCapacity / 4);
        if ((NewCapacity.get() > this.FCapacity) && (NewCapacity.get() < GC)) NewCapacity.set(GC);
        NewCapacity.set((NewCapacity.get() + (4096 - 1)) & ~(4096 - 1));
      };
      if (NewCapacity.get() === this.FCapacity) {
        Result = this.FMemory}
       else if (NewCapacity.get() === 0) {
        Result = null}
       else {
        Result = new ArrayBuffer(NewCapacity.get());
        if (Result === null) throw $mod.EStreamError.$create("Create$1",[rtl.getResStr(pas.Classes,"SMemoryStreamError")]);
        DestView = new Uint8Array(Result);
        DestView.set(this.GetDataArray());
      };
      return Result;
    };
    this.Destroy = function () {
      this.Clear();
      pas.System.TObject.Destroy.call(this);
    };
    this.Clear = function () {
      this.FSize = 0;
      this.FPosition = 0;
      this.SetCapacity(0);
    };
    this.Write$1 = function (Buffer, Offset, Count) {
      var Result = 0;
      var NewPos = 0;
      if ((Count === 0) || (this.FPosition < 0)) return 0;
      NewPos = this.FPosition + Count;
      if (NewPos > this.FSize) {
        if (NewPos > this.FCapacity) this.SetCapacity(NewPos);
        this.FSize = NewPos;
      };
      this.GetDataArray().set(rtl.arrayCopy(0,Buffer,Offset,Count),this.FPosition);
      this.FPosition = NewPos;
      Result = Count;
      return Result;
    };
  });
  rtl.createClass($mod,"TBytesStream",$mod.TMemoryStream,function () {
    this.GetBytes = function () {
      var Result = [];
      Result = $mod.TMemoryStream.MemoryToBytes(this.FMemory);
      return Result;
    };
    this.Create$1 = function (ABytes) {
      pas.System.TObject.Create.call(this);
      this.SetPointer($mod.TMemoryStream.BytesToMemory(ABytes),rtl.length(ABytes));
      this.FCapacity = rtl.length(ABytes);
      return this;
    };
  });
  rtl.createClass($mod,"TStringStream",$mod.TMemoryStream,function () {
    this.GetDataString = function () {
      var Result = "";
      var a = null;
      Result = "";
      a = new Uint16Array(this.FMemory.slice(0,this.GetSize()));
      //    Result=String.fromCharCode.apply(null, new Uint16Array(a));
      Result=String.fromCharCode.apply(null, a);
      return Result;
    };
    this.Create$1 = function (aString) {
      var $Self = this;
      function StrToBuf(aLen) {
        var Result = null;
        var I = 0;
        Result = new ArrayBuffer(aLen * 2);
        var $with1 = new Uint16Array(Result);
        for (var $l2 = 0, $end3 = aLen - 1; $l2 <= $end3; $l2++) {
          I = $l2;
          $with1[I] = aString.charCodeAt(I);
        };
        return Result;
      };
      var Len = 0;
      pas.System.TObject.Create.call($Self);
      Len = aString.length;
      $Self.SetPointer(StrToBuf(Len),Len * 2);
      $Self.FCapacity = Len * 2;
      return this;
    };
  });
  this.TFilerFlag = {"0": "ffInherited", ffInherited: 0, "1": "ffChildPos", ffChildPos: 1, "2": "ffInline", ffInline: 2};
  rtl.createClass($mod,"TFiler",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FRoot = null;
      this.FLookupRoot = null;
      this.FAncestor = null;
      this.FIgnoreChildren = false;
    };
    this.$final = function () {
      this.FRoot = undefined;
      this.FLookupRoot = undefined;
      this.FAncestor = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.SetRoot = function (ARoot) {
      this.FRoot = ARoot;
    };
  });
  this.TValueType = {"0": "vaNull", vaNull: 0, "1": "vaList", vaList: 1, "2": "vaInt8", vaInt8: 2, "3": "vaInt16", vaInt16: 3, "4": "vaInt32", vaInt32: 4, "5": "vaDouble", vaDouble: 5, "6": "vaString", vaString: 6, "7": "vaIdent", vaIdent: 7, "8": "vaFalse", vaFalse: 8, "9": "vaTrue", vaTrue: 9, "10": "vaBinary", vaBinary: 10, "11": "vaSet", vaSet: 11, "12": "vaNil", vaNil: 12, "13": "vaCollection", vaCollection: 13, "14": "vaCurrency", vaCurrency: 14, "15": "vaDate", vaDate: 15, "16": "vaNativeInt", vaNativeInt: 16};
  rtl.createClass($mod,"TAbstractObjectWriter",pas.System.TObject,function () {
  });
  rtl.createClass($mod,"TBinaryObjectWriter",$mod.TAbstractObjectWriter,function () {
    this.$init = function () {
      $mod.TAbstractObjectWriter.$init.call(this);
      this.FStream = null;
    };
    this.$final = function () {
      this.FStream = undefined;
      $mod.TAbstractObjectWriter.$final.call(this);
    };
    this.WriteWord = function (w) {
      this.FStream.WriteBufferData$12(w);
    };
    this.WriteDWord = function (lw) {
      this.FStream.WriteBufferData$14(lw);
    };
    this.WriteValue = function (Value) {
      var b = 0;
      b = Value;
      this.FStream.WriteBufferData$8(b);
    };
    this.Create$1 = function (Stream) {
      pas.System.TObject.Create.call(this);
      if (Stream === null) throw $mod.EWriteError.$create("Create$1",[rtl.getResStr(pas.RTLConsts,"SEmptyStreamIllegalWriter")]);
      this.FStream = Stream;
      return this;
    };
    this.WriteSignature = function () {
      this.FStream.WriteBufferData$14(809914452);
    };
    this.BeginCollection = function () {
      this.WriteValue(13);
    };
    this.BeginComponent = function (Component, Flags, ChildPos) {
      var Prefix = 0;
      if (rtl.neSet(Flags,{})) {
        Prefix = 0;
        if (0 in Flags) Prefix = Prefix | 0x1;
        if (1 in Flags) Prefix = Prefix | 0x2;
        if (2 in Flags) Prefix = Prefix | 0x4;
        Prefix = Prefix | 0xf0;
        this.FStream.WriteBufferData$8(Prefix);
        if (1 in Flags) this.WriteInteger(ChildPos);
      };
      this.WriteStr(Component.$classname);
      this.WriteStr(Component.FName);
    };
    this.BeginList = function () {
      this.WriteValue(1);
    };
    this.EndList = function () {
      this.WriteValue(0);
    };
    this.BeginProperty = function (PropName) {
      this.WriteStr(PropName);
    };
    this.EndProperty = function () {
    };
    this.WriteBoolean = function (Value) {
      if (Value) {
        this.WriteValue(9)}
       else this.WriteValue(8);
    };
    this.WriteFloat = function (Value) {
      this.WriteValue(5);
      this.FStream.WriteBufferData$20(Value);
    };
    this.WriteIdent = function (Ident) {
      if (pas.SysUtils.UpperCase(Ident) === "NIL") {
        this.WriteValue(12)}
       else if (pas.SysUtils.UpperCase(Ident) === "FALSE") {
        this.WriteValue(8)}
       else if (pas.SysUtils.UpperCase(Ident) === "TRUE") {
        this.WriteValue(9)}
       else if (pas.SysUtils.UpperCase(Ident) === "NULL") {
        this.WriteValue(0)}
       else {
        this.WriteValue(7);
        this.WriteStr(Ident);
      };
    };
    this.WriteInteger = function (Value) {
      var s = 0;
      var i = 0;
      var l = 0;
      if ((Value >= -128) && (Value <= 127)) {
        this.WriteValue(2);
        s = Value;
        this.FStream.WriteBufferData$6(s);
      } else if ((Value >= -32768) && (Value <= 32767)) {
        this.WriteValue(3);
        i = Value;
        this.WriteWord(i & 65535);
      } else if ((Value >= -0x80000000) && (Value <= 0x7fffffff)) {
        this.WriteValue(4);
        l = Value;
        this.WriteDWord(l >>> 0);
      } else {
        this.WriteValue($mod.TValueType.vaNativeInt);
        this.FStream.WriteBufferData$16(Value);
      };
    };
    this.WriteMethodName = function (Name) {
      if (Name.length > 0) {
        this.WriteValue(7);
        this.WriteStr(Name);
      } else this.WriteValue(12);
    };
    this.WriteSet = function (Value, SetType) {
      var i = 0;
      var b = 0;
      this.WriteValue(11);
      b = 1;
      for (i = 0; i <= 31; i++) {
        if ((Value & b) !== 0) {
          this.WriteStr(pas.TypInfo.GetEnumName(SetType,i));
        };
        b = b << 1;
      };
      this.WriteStr("");
    };
    this.WriteStr = function (Value) {
      var len = 0;
      var i = 0;
      var b = 0;
      len = Value.length;
      if (len > 255) len = 255;
      b = len;
      this.FStream.WriteBufferData$8(b);
      for (var $l1 = 1, $end2 = len; $l1 <= $end2; $l1++) {
        i = $l1;
        this.FStream.WriteBufferData$4(Value.charAt(i - 1));
      };
    };
    this.WriteString = function (Value) {
      var i = 0;
      var len = 0;
      len = Value.length;
      this.WriteValue(6);
      this.WriteDWord(len);
      for (var $l1 = 1, $end2 = len; $l1 <= $end2; $l1++) {
        i = $l1;
        this.FStream.WriteBufferData$4(Value.charAt(i - 1));
      };
    };
    this.WriteVariant = function (VarValue) {
      if (pas.JS.isUndefined(VarValue)) {
        this.WriteValue(12)}
       else if (pas.JS.isNull(VarValue)) {
        this.WriteValue(0)}
       else if (rtl.isNumber(VarValue)) {
        if (pas.System.Frac(rtl.getNumber(VarValue)) === 0) {
          this.WriteInteger(Math.floor(VarValue))}
         else this.WriteFloat(rtl.getNumber(VarValue));
      } else if (pas.JS.isBoolean(VarValue)) {
        this.WriteBoolean(!(VarValue == false))}
       else if (rtl.isString(VarValue)) {
        this.WriteString("" + VarValue)}
       else throw $mod.EWriteError.$create("Create$1",[rtl.getResStr(pas.RTLConsts,"SUnsupportedPropertyVariantType")]);
    };
  });
  rtl.createClass($mod,"TWriter",$mod.TFiler,function () {
    this.$init = function () {
      $mod.TFiler.$init.call(this);
      this.FDriver = null;
      this.FDestroyDriver = false;
      this.FRootAncestor = null;
      this.FPropPath = "";
      this.FAncestors = null;
      this.FAncestorPos = 0;
      this.FCurrentPos = 0;
      this.FOnFindAncestor = null;
      this.FOnWriteMethodProperty = null;
      this.FOnWriteStringProperty = null;
    };
    this.$final = function () {
      this.FDriver = undefined;
      this.FRootAncestor = undefined;
      this.FAncestors = undefined;
      this.FOnFindAncestor = undefined;
      this.FOnWriteMethodProperty = undefined;
      this.FOnWriteStringProperty = undefined;
      $mod.TFiler.$final.call(this);
    };
    this.AddToAncestorList = function (Component) {
      this.FAncestors.AddObject(Component.FName,$impl.TPosComponent.$create("Create$1",[this.FAncestors.GetCount(),Component]));
    };
    this.WriteComponentData = function (Instance) {
      var Flags = {};
      Flags = {};
      if ((this.FAncestor != null) && (!(9 in Instance.FComponentState) || ((5 in Instance.FComponentState) && (this.FAncestors !== null)))) {
        Flags = rtl.createSet(0)}
       else if (9 in Instance.FComponentState) Flags = rtl.createSet(2);
      if ((this.FAncestors !== null) && ((this.FCurrentPos !== this.FAncestorPos) || (this.FAncestor === null))) Flags = rtl.includeSet(Flags,1);
      this.FDriver.BeginComponent(Instance,rtl.refSet(Flags),this.FCurrentPos);
      if (this.FAncestors !== null) this.FCurrentPos += 1;
      this.WriteProperties(Instance);
      this.WriteListEnd();
      if (!this.FIgnoreChildren) this.WriteChildren(Instance);
    };
    this.DetermineAncestor = function (Component) {
      var I = 0;
      if (!(this.FAncestors != null)) return;
      I = this.FAncestors.IndexOf(Component.FName);
      if (I === -1) {
        this.FAncestor = null;
        this.FAncestorPos = -1;
      } else {
        var $with1 = this.FAncestors.GetObject(I);
        this.FAncestor = $with1.FComponent;
        this.FAncestorPos = $with1.FPos;
      };
    };
    this.DoFindAncestor = function (Component) {
      var C = null;
      if (this.FOnFindAncestor != null) if ((this.FAncestor === null) || $mod.TComponent.isPrototypeOf(this.FAncestor)) {
        C = this.FAncestor;
        this.FOnFindAncestor(this,Component,Component.FName,{get: function () {
            return C;
          }, set: function (v) {
            C = v;
          }},{p: this, get: function () {
            return this.p.FRootAncestor;
          }, set: function (v) {
            this.p.FRootAncestor = v;
          }});
        this.FAncestor = C;
      };
    };
    this.SetRoot = function (ARoot) {
      $mod.TFiler.SetRoot.call(this,ARoot);
      this.FLookupRoot = ARoot;
    };
    this.WriteProperty = function (Instance, PropInfo) {
      var HasAncestor = false;
      var PropType = null;
      var N = 0;
      var Value = 0;
      var DefValue = 0;
      var Ident = "";
      var IntToIdentFn = null;
      var FloatValue = 0.0;
      var DefFloatValue = 0.0;
      var MethodValue = pas.System.TMethod.$new();
      var DefMethodValue = pas.System.TMethod.$new();
      var StrValue = "";
      var DefStrValue = "";
      var AncestorObj = null;
      var C = null;
      var Component = null;
      var ObjValue = null;
      var SavedAncestor = null;
      var Key = "";
      var SavedPropPath = "";
      var Name = "";
      var lMethodName = "";
      var VarValue = undefined;
      var DefVarValue = undefined;
      var BoolValue = false;
      var DefBoolValue = false;
      var Handled = false;
      var O = null;
      var intfValue = null;
      try {
        if (PropInfo.getter === "") return;
        PropType = PropInfo.typeinfo;
        if (PropInfo.setter === "") {
          if (PropType.kind !== 13) return;
          ObjValue = pas.TypInfo.GetObjectProp$2(Instance,PropInfo);
          if (!ObjValue.$class.InheritsFrom($mod.TComponent) || !(2 in ObjValue.FComponentStyle)) return;
        };
        HasAncestor = (this.FAncestor != null) && ((Instance === this.FRoot) || (Instance.$class.ClassType() === this.FAncestor.$class.ClassType()));
        var $tmp1 = PropType.kind;
        if (($tmp1 === 1) || ($tmp1 === 2) || ($tmp1 === 4) || ($tmp1 === 5)) {
          Value = pas.TypInfo.GetOrdProp$1(Instance,PropInfo);
          if (HasAncestor) {
            DefValue = pas.TypInfo.GetOrdProp$1(this.FAncestor,PropInfo)}
           else {
            if (PropType.kind !== 5) {
              DefValue = Math.floor(PropInfo.Default)}
             else {
              O = rtl.getObject(PropInfo.Default);
              DefValue = 0;
              for (Key in O) {
                N = parseInt(Key,10);
                if (N < 32) DefValue = DefValue + (1 << N);
              };
            };
          };
          if ((Value !== DefValue) || (DefValue === 0x80000000)) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            var $tmp2 = PropType.kind;
            if ($tmp2 === 1) {
              IntToIdentFn = $mod.FindIntToIdent(PropInfo.typeinfo);
              if ((IntToIdentFn != null) && IntToIdentFn(Value,{get: function () {
                  return Ident;
                }, set: function (v) {
                  Ident = v;
                }})) {
                this.WriteIdent(Ident)}
               else this.WriteInteger(Value);
            } else if ($tmp2 === 2) {
              this.WriteChar(String.fromCharCode(Value))}
             else if ($tmp2 === 5) {
              this.FDriver.WriteSet(Value,PropType.comptype);
            } else if ($tmp2 === 4) this.WriteIdent(pas.TypInfo.GetEnumName(PropType,Value));
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === pas.TypInfo.TTypeKind.tkDouble) {
          FloatValue = pas.TypInfo.GetFloatProp$1(Instance,PropInfo);
          if (HasAncestor) {
            DefFloatValue = pas.TypInfo.GetFloatProp$1(this.FAncestor,PropInfo)}
           else {
            DefFloatValue = rtl.getNumber(PropInfo.Default);
          };
          if ((FloatValue !== DefFloatValue) || (!HasAncestor && (pas.System.Int(DefFloatValue) === 0x80000000))) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            this.WriteFloat(FloatValue);
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === 9) {
          MethodValue.$assign(pas.TypInfo.GetMethodProp(Instance,PropInfo));
          if (HasAncestor) {
            DefMethodValue.$assign(pas.TypInfo.GetMethodProp(this.FAncestor,PropInfo))}
           else {
            DefMethodValue.Data = null;
            DefMethodValue.Code = null;
          };
          Handled = false;
          if (this.FOnWriteMethodProperty != null) this.FOnWriteMethodProperty(this,Instance,PropInfo,MethodValue,DefMethodValue,{get: function () {
              return Handled;
            }, set: function (v) {
              Handled = v;
            }});
          if (rtl.isString(MethodValue.Code)) {
            lMethodName = MethodValue.Code}
           else lMethodName = this.FLookupRoot.$class.MethodName(MethodValue.Code);
          if (!Handled && (MethodValue.Code !== DefMethodValue.Code) && (!(MethodValue.Code != null) || (lMethodName.length > 0))) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            if (MethodValue.Code != null) {
              this.FDriver.WriteMethodName(lMethodName)}
             else this.FDriver.WriteMethodName("");
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === 3) {
          StrValue = pas.TypInfo.GetStrProp$1(Instance,PropInfo);
          if (HasAncestor) {
            DefStrValue = pas.TypInfo.GetStrProp$1(this.FAncestor,PropInfo)}
           else {
            DefValue = Math.floor(PropInfo.Default);
            DefStrValue = rtl.strSetLength(DefStrValue,0);
          };
          if ((StrValue !== DefStrValue) || (!HasAncestor && (DefValue === 0x80000000))) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            if (this.FOnWriteStringProperty != null) this.FOnWriteStringProperty(this,Instance,PropInfo,{get: function () {
                return StrValue;
              }, set: function (v) {
                StrValue = v;
              }});
            this.WriteString(StrValue);
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === 16) {
          VarValue = pas.TypInfo.GetJSValueProp$3(Instance,PropInfo);
          if (HasAncestor) {
            DefVarValue = pas.TypInfo.GetJSValueProp$3(this.FAncestor,PropInfo)}
           else DefVarValue = null;
          if (VarValue != DefVarValue) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            this.WriteVariant(VarValue);
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === 13) {
          ObjValue = pas.TypInfo.GetObjectProp$2(Instance,PropInfo);
          if (HasAncestor) {
            AncestorObj = pas.TypInfo.GetObjectProp$2(this.FAncestor,PropInfo);
            if ($mod.TComponent.isPrototypeOf(AncestorObj) && $mod.TComponent.isPrototypeOf(ObjValue)) {
              if ((AncestorObj !== ObjValue) && (AncestorObj.FOwner === this.FRootAncestor) && (ObjValue.FOwner === this.FRoot) && (pas.SysUtils.UpperCase(AncestorObj.FName) === pas.SysUtils.UpperCase(ObjValue.FName))) {
                AncestorObj = ObjValue;
              };
            };
          } else AncestorObj = null;
          if (!(ObjValue != null)) {
            if (ObjValue !== AncestorObj) {
              this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
              this.FDriver.WriteIdent("NIL");
              this.FDriver.EndProperty();
            };
          } else if (ObjValue.$class.InheritsFrom($mod.TPersistent)) {
            if (ObjValue.$class.InheritsFrom($mod.TComponent) && (!(2 in ObjValue.FComponentStyle) || ((ObjValue.FOwner !== Instance) && (ObjValue.FOwner !== null)))) {
              Component = ObjValue;
              if ((ObjValue !== AncestorObj) && !(3 in Component.FComponentStyle)) {
                Name = "";
                C = Component;
                while ((C !== null) && (C.FName !== "")) {
                  if (Name !== "") Name = "." + Name;
                  if (C.FOwner === this.FLookupRoot) {
                    Name = C.FName + Name;
                    break;
                  } else if (C === this.FLookupRoot) {
                    Name = "Owner" + Name;
                    break;
                  };
                  Name = C.FName + Name;
                  C = C.FOwner;
                };
                if ((C === null) && (Component.FOwner === null)) if (Name !== "") Name = Name + ".Owner";
                if (Name.length > 0) {
                  this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
                  this.WriteIdent(Name);
                  this.FDriver.EndProperty();
                };
              };
            } else {
              SavedAncestor = this.FAncestor;
              SavedPropPath = this.FPropPath;
              try {
                this.FPropPath = this.FPropPath + PropInfo.name + ".";
                if (HasAncestor) this.FAncestor = pas.TypInfo.GetObjectProp$2(this.FAncestor,PropInfo);
                this.WriteProperties(ObjValue);
              } finally {
                this.FAncestor = SavedAncestor;
                this.FPropPath = SavedPropPath;
              };
              if (ObjValue.$class.InheritsFrom($mod.TCollection)) {
                if (!HasAncestor || !$mod.CollectionsEqual$1(ObjValue,pas.TypInfo.GetObjectProp$2(this.FAncestor,PropInfo),this.FRoot,this.FRootAncestor)) {
                  this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
                  SavedPropPath = this.FPropPath;
                  try {
                    this.FPropPath = rtl.strSetLength(this.FPropPath,0);
                    this.WriteCollection(ObjValue);
                  } finally {
                    this.FPropPath = SavedPropPath;
                    this.FDriver.EndProperty();
                  };
                };
              };
            };
          };
        } else if ($tmp1 === 7) {
          BoolValue = pas.TypInfo.GetOrdProp$1(Instance,PropInfo) !== 0;
          if (HasAncestor) {
            DefBoolValue = pas.TypInfo.GetOrdProp$1(this.FAncestor,PropInfo) !== 0}
           else {
            DefBoolValue = PropInfo.Default != 0;
            DefValue = Math.floor(PropInfo.Default);
          };
          if ((BoolValue !== DefBoolValue) || (DefValue === 0x80000000)) {
            this.FDriver.BeginProperty(this.FPropPath + PropInfo.name);
            this.WriteBoolean(BoolValue);
            this.FDriver.EndProperty();
          };
        } else if ($tmp1 === 18) {
          intfValue = rtl.setIntfL(intfValue,pas.TypInfo.GetInterfaceProp$1(Instance,PropInfo),true);
        };
      } finally {
        rtl._Release(intfValue);
      };
    };
    this.WriteProperties = function (Instance) {
      var PropCount = 0;
      var i = 0;
      var PropList = [];
      PropList = pas.TypInfo.GetPropList$3(Instance);
      PropCount = rtl.length(PropList);
      if (PropCount > 0) for (var $l1 = 0, $end2 = PropCount - 1; $l1 <= $end2; $l1++) {
        i = $l1;
        if (pas.TypInfo.IsStoredProp(Instance,PropList[i])) this.WriteProperty(Instance,PropList[i]);
      };
      Instance.DefineProperties(this);
    };
    this.WriteChildren = function (Component) {
      var SRoot = null;
      var SRootA = null;
      var SList = null;
      var SPos = 0;
      var I = 0;
      var SAncestorPos = 0;
      var O = null;
      SRoot = this.FRoot;
      SRootA = this.FRootAncestor;
      SList = this.FAncestors;
      SPos = this.FCurrentPos;
      SAncestorPos = this.FAncestorPos;
      try {
        this.FAncestors = null;
        this.FCurrentPos = 0;
        this.FAncestorPos = -1;
        if (9 in Component.FComponentState) this.FRoot = Component;
        if ($mod.TComponent.isPrototypeOf(this.FAncestor)) {
          this.FAncestors = $mod.TStringList.$create("Create$1");
          if (9 in this.FAncestor.FComponentState) this.FRootAncestor = this.FAncestor;
          this.FAncestor.GetChildren(rtl.createCallback(this,"AddToAncestorList"),this.FRootAncestor);
          this.FAncestors.SetSorted(true);
        };
        try {
          Component.GetChildren(rtl.createCallback(this,"WriteComponent"),this.FRoot);
        } finally {
          if (this.FAncestors != null) for (var $l1 = 0, $end2 = this.FAncestors.GetCount() - 1; $l1 <= $end2; $l1++) {
            I = $l1;
            O = this.FAncestors.GetObject(I);
            this.FAncestors.PutObject(I,null);
            O = rtl.freeLoc(O);
          };
          pas.SysUtils.FreeAndNil({p: this, get: function () {
              return this.p.FAncestors;
            }, set: function (v) {
              this.p.FAncestors = v;
            }});
        };
      } finally {
        this.FAncestors = SList;
        this.FRoot = SRoot;
        this.FRootAncestor = SRootA;
        this.FCurrentPos = SPos;
        this.FAncestorPos = SAncestorPos;
      };
    };
    this.CreateDriver = function (Stream) {
      var Result = null;
      Result = $mod.TBinaryObjectWriter.$create("Create$1",[Stream]);
      return Result;
    };
    this.Create$1 = function (ADriver) {
      pas.System.TObject.Create.call(this);
      this.FDriver = ADriver;
      return this;
    };
    this.Create$2 = function (Stream) {
      pas.System.TObject.Create.call(this);
      if (Stream === null) throw $mod.EWriteError.$create("Create$1",[rtl.getResStr(pas.RTLConsts,"SEmptyStreamIllegalWriter")]);
      this.FDriver = this.CreateDriver(Stream);
      this.FDestroyDriver = true;
      return this;
    };
    this.Destroy = function () {
      if (this.FDestroyDriver) rtl.free(this,"FDriver");
      pas.System.TObject.Destroy.call(this);
    };
    this.WriteBoolean = function (Value) {
      this.FDriver.WriteBoolean(Value);
    };
    this.WriteCollection = function (Value) {
      var i = 0;
      this.FDriver.BeginCollection();
      if (Value != null) for (var $l1 = 0, $end2 = Value.GetCount() - 1; $l1 <= $end2; $l1++) {
        i = $l1;
        this.WriteListBegin();
        this.WriteProperties(Value.GetItem(i));
        this.WriteListEnd();
      };
      this.WriteListEnd();
    };
    this.WriteComponent = function (Component) {
      var SA = null;
      var SR = null;
      var SRA = null;
      SR = this.FRoot;
      SA = this.FAncestor;
      SRA = this.FRootAncestor;
      try {
        Component.FComponentState = rtl.unionSet(Component.FComponentState,rtl.createSet(2));
        try {
          this.DetermineAncestor(Component);
          this.DoFindAncestor(Component);
          Component.WriteState(this);
          this.FDriver.EndList();
        } finally {
          Component.FComponentState = rtl.diffSet(Component.FComponentState,rtl.createSet(2));
        };
      } finally {
        this.FAncestor = SA;
        this.FRoot = SR;
        this.FRootAncestor = SRA;
      };
    };
    this.WriteChar = function (Value) {
      this.WriteString(Value);
    };
    this.WriteDescendent = function (ARoot, AAncestor) {
      this.FRoot = ARoot;
      this.FAncestor = AAncestor;
      this.FRootAncestor = AAncestor;
      this.FLookupRoot = ARoot;
      this.WriteSignature();
      this.WriteComponent(ARoot);
    };
    this.WriteFloat = function (Value) {
      this.FDriver.WriteFloat(Value);
    };
    this.WriteIdent = function (Ident) {
      this.FDriver.WriteIdent(Ident);
    };
    this.WriteInteger = function (Value) {
      this.FDriver.WriteInteger(Value);
    };
    this.WriteListBegin = function () {
      this.FDriver.BeginList();
    };
    this.WriteListEnd = function () {
      this.FDriver.EndList();
    };
    this.WriteSignature = function () {
      this.FDriver.WriteSignature();
    };
    this.WriteString = function (Value) {
      this.FDriver.WriteString(Value);
    };
    this.WriteVariant = function (VarValue) {
      this.FDriver.WriteVariant(VarValue);
    };
  });
  this.TObjectTextEncoding = {"0": "oteDFM", oteDFM: 0, "1": "oteLFM", oteLFM: 1};
  rtl.createClass($mod,"TObjectStreamConverter",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FIndent = "";
      this.FInput = null;
      this.FOutput = null;
      this.FEncoding = 0;
    };
    this.$final = function () {
      this.FInput = undefined;
      this.FOutput = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.OutLn = function (s) {
      this.OutStr(s + pas.System.LineEnding);
    };
    this.OutStr = function (s) {
      var I = 0;
      for (var $l1 = 1, $end2 = s.length; $l1 <= $end2; $l1++) {
        I = $l1;
        this.FOutput.WriteBufferData$4(s.charAt(I - 1));
      };
    };
    this.OutString = function (s) {
      this.OutStr(s);
    };
    this.ReadWord = function () {
      var Result = 0;
      this.FInput.ReadBufferData$10({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }});
      return Result;
    };
    this.ReadDWord = function () {
      var Result = 0;
      this.FInput.ReadBufferData$14({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }});
      return Result;
    };
    this.ReadDouble = function () {
      var Result = 0.0;
      this.FInput.ReadBufferData$20({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }});
      return Result;
    };
    this.ReadInt = function (ValueType) {
      var Result = 0;
      var $tmp1 = ValueType;
      if ($tmp1 === 2) {
        Result = ((this.FInput.ReadByte() & 255) << 24) >> 24}
       else if ($tmp1 === 3) {
        Result = ((this.ReadWord() & 65535) << 16) >> 16}
       else if ($tmp1 === 4) {
        Result = this.ReadDWord() & 0xFFFFFFFF}
       else if ($tmp1 === 16) Result = this.ReadNativeInt();
      return Result;
    };
    this.ReadInt$1 = function () {
      var Result = 0;
      Result = this.ReadInt(this.FInput.ReadByte());
      return Result;
    };
    this.ReadNativeInt = function () {
      var Result = 0;
      this.FInput.ReadBufferData$16({get: function () {
          return Result;
        }, set: function (v) {
          Result = v;
        }});
      return Result;
    };
    this.ReadStr = function () {
      var Result = "";
      var l = 0;
      var i = 0;
      var c = "";
      this.FInput.ReadBufferData$6({get: function () {
          return l;
        }, set: function (v) {
          l = v;
        }});
      Result = rtl.strSetLength(Result,l);
      for (var $l1 = 1, $end2 = l; $l1 <= $end2; $l1++) {
        i = $l1;
        this.FInput.ReadBufferData$2({get: function () {
            return c;
          }, set: function (v) {
            c = v;
          }});
        Result = rtl.setCharAt(Result,i - 1,c);
      };
      return Result;
    };
    this.ReadString = function (StringType) {
      var Result = "";
      var i = 0;
      var C = "";
      Result = "";
      if (StringType !== 6) throw $mod.EFilerError.$create("Create$1",["Invalid string type passed to ReadString"]);
      i = this.ReadDWord();
      Result = rtl.strSetLength(Result,i);
      for (var $l1 = 1, $end2 = Result.length; $l1 <= $end2; $l1++) {
        i = $l1;
        this.FInput.ReadBufferData$2({get: function () {
            return C;
          }, set: function (v) {
            C = v;
          }});
        Result = rtl.setCharAt(Result,i - 1,C);
      };
      return Result;
    };
    this.ProcessBinary = function () {
      var ToDo = 0;
      var DoNow = 0;
      var i = 0;
      var lbuf = [];
      var s = "";
      ToDo = this.ReadDWord();
      lbuf = rtl.arraySetLength(lbuf,0,32);
      this.OutLn("{");
      while (ToDo > 0) {
        DoNow = ToDo;
        if (DoNow > 32) DoNow = 32;
        ToDo -= DoNow;
        s = this.FIndent + "  ";
        this.FInput.ReadBuffer({get: function () {
            return lbuf;
          }, set: function (v) {
            lbuf = v;
          }},DoNow);
        for (var $l1 = 0, $end2 = DoNow - 1; $l1 <= $end2; $l1++) {
          i = $l1;
          s = s + pas.SysUtils.IntToHex(lbuf[i],2);
        };
        this.OutLn(s);
      };
      this.OutLn(this.FIndent + "}");
    };
    this.ProcessValue = function (ValueType, Indent) {
      var s = "";
      var IsFirst = false;
      var ext = 0.0;
      var $tmp1 = ValueType;
      if ($tmp1 === 1) {
        this.OutStr("(");
        IsFirst = true;
        while (true) {
          ValueType = this.FInput.ReadByte();
          if (ValueType === 0) break;
          if (IsFirst) {
            this.OutLn("");
            IsFirst = false;
          };
          this.OutStr(Indent + "  ");
          this.ProcessValue(ValueType,Indent + "  ");
        };
        this.OutLn(Indent + ")");
      } else if ($tmp1 === 2) {
        this.OutLn(pas.SysUtils.IntToStr(((this.FInput.ReadByte() & 255) << 24) >> 24))}
       else if ($tmp1 === 3) {
        this.OutLn(pas.SysUtils.IntToStr(((this.ReadWord() & 65535) << 16) >> 16))}
       else if ($tmp1 === 4) {
        this.OutLn(pas.SysUtils.IntToStr(this.ReadDWord() & 0xFFFFFFFF))}
       else if ($tmp1 === 16) {
        this.OutLn(pas.SysUtils.IntToStr(this.ReadNativeInt()))}
       else if ($tmp1 === 5) {
        ext = this.ReadDouble();
        s = rtl.floatToStr(ext);
        this.OutLn(s);
      } else if ($tmp1 === 6) {
        this.OutString("'" + pas.SysUtils.StringReplace(this.ReadString(6),"'","''",rtl.createSet(0)) + "'");
        this.OutLn("");
      } else if ($tmp1 === 7) {
        this.OutLn(this.ReadStr())}
       else if ($tmp1 === 8) {
        this.OutLn("False")}
       else if ($tmp1 === 9) {
        this.OutLn("True")}
       else if ($tmp1 === 10) {
        this.ProcessBinary()}
       else if ($tmp1 === 11) {
        this.OutStr("[");
        IsFirst = true;
        while (true) {
          s = this.ReadStr();
          if (s.length === 0) break;
          if (!IsFirst) this.OutStr(", ");
          IsFirst = false;
          this.OutStr(s);
        };
        this.OutLn("]");
      } else if ($tmp1 === 12) {
        this.OutLn("nil")}
       else if ($tmp1 === 13) {
        this.OutStr("<");
        while (this.FInput.ReadByte() !== 0) {
          this.OutLn(Indent);
          this.FInput.Seek(-1,1);
          this.OutStr(Indent + "  item");
          ValueType = this.FInput.ReadByte();
          if (ValueType !== 1) this.OutStr("[" + pas.SysUtils.IntToStr(this.ReadInt(ValueType)) + "]");
          this.OutLn("");
          this.ReadPropList(Indent + "    ");
          this.OutStr(Indent + "  end");
        };
        this.OutLn(">");
      } else {
        throw $mod.EReadError.$create("CreateFmt",[rtl.getResStr(pas.RTLConsts,"SErrInvalidPropertyType"),[ValueType]]);
      };
    };
    this.ReadObject = function (indent) {
      var b = 0;
      var ObjClassName = "";
      var ObjName = "";
      var ChildPos = 0;
      b = this.FInput.ReadByte();
      if ((b & 0xf0) === 0xf0) {
        if ((b & 2) !== 0) ChildPos = this.ReadInt$1();
      } else {
        b = 0;
        this.FInput.Seek(-1,1);
      };
      ObjClassName = this.ReadStr();
      ObjName = this.ReadStr();
      this.OutStr(indent);
      if ((b & 1) !== 0) {
        this.OutStr("inherited")}
       else if ((b & 4) !== 0) {
        this.OutStr("inline")}
       else this.OutStr("object");
      this.OutStr(" ");
      if (ObjName !== "") this.OutStr(ObjName + ": ");
      this.OutStr(ObjClassName);
      if ((b & 2) !== 0) this.OutStr("[" + pas.SysUtils.IntToStr(ChildPos) + "]");
      this.OutLn("");
      this.ReadPropList(indent + "  ");
      while (this.FInput.ReadByte() !== 0) {
        this.FInput.Seek(-1,1);
        this.ReadObject(indent + "  ");
      };
      this.OutLn(indent + "end");
    };
    this.ReadPropList = function (indent) {
      while (this.FInput.ReadByte() !== 0) {
        this.FInput.Seek(-1,1);
        this.OutStr(indent + this.ReadStr() + " = ");
        this.ProcessValue(this.FInput.ReadByte(),indent);
      };
    };
    this.ObjectBinaryToText$1 = function (aInput, aOutput, aEncoding) {
      this.FInput = aInput;
      this.FOutput = aOutput;
      this.FEncoding = aEncoding;
      this.Execute();
    };
    this.Execute = function () {
      if (this.FIndent === "") this.FIndent = "  ";
      if (!(this.FInput != null)) throw $mod.EReadError.$create("Create$1",["Missing input stream"]);
      if (!(this.FOutput != null)) throw $mod.EReadError.$create("Create$1",["Missing output stream"]);
      if (this.FInput.ReadDWord() !== 809914452) throw $mod.EReadError.$create("Create$1",["Illegal stream image"]);
      this.ReadObject("");
    };
  });
  this.FindIntToIdent = function (AIntegerType) {
    var Result = null;
    var i = 0;
    Result = null;
    if (!($impl.IntConstList != null)) return Result;
    var $with1 = $impl.IntConstList;
    for (var $l2 = 0, $end3 = $with1.FCount - 1; $l2 <= $end3; $l2++) {
      i = $l2;
      if (rtl.getObject($with1.Get(i)).IntegerType === AIntegerType) return rtl.getObject($with1.Get(i)).IntToIdentFn;
    };
    return Result;
  };
  this.CollectionsEqual$1 = function (C1, C2, Owner1, Owner2) {
    var Result = false;
    function stream_collection(s, c, o) {
      var w = null;
      w = $mod.TWriter.$create("Create$2",[s]);
      try {
        w.SetRoot(o);
        w.FLookupRoot = o;
        w.WriteCollection(c);
      } finally {
        w = rtl.freeLoc(w);
      };
    };
    var s1 = null;
    var s2 = null;
    var b1 = [];
    var b2 = [];
    var I = 0;
    var Len = 0;
    Result = false;
    if ((C1.$class.ClassType() !== C2.$class.ClassType()) || (C1.GetCount() !== C2.GetCount())) return Result;
    if (C1.GetCount() === 0) {
      Result = true;
      return Result;
    };
    s2 = null;
    s1 = $mod.TBytesStream.$create("Create");
    try {
      s2 = $mod.TBytesStream.$create("Create");
      stream_collection(s1,C1,Owner1);
      stream_collection(s2,C2,Owner2);
      Result = s1.GetSize() === s2.GetSize();
      if (Result) {
        b1 = s1.GetBytes();
        b2 = s2.GetBytes();
        I = 0;
        Len = s1.GetSize();
        while (Result && (I < Len)) {
          Result = b1[I] === b2[I];
          I += 1;
        };
      };
    } finally {
      s2 = rtl.freeLoc(s2);
      s1 = rtl.freeLoc(s1);
    };
    return Result;
  };
  this.ObjectBinaryToText = function (aInput, aOutput) {
    $mod.ObjectBinaryToText$1(aInput,aOutput,1);
  };
  this.ObjectBinaryToText$1 = function (aInput, aOutput, aEncoding) {
    var Conv = null;
    Conv = $mod.TObjectStreamConverter.$create("Create");
    try {
      Conv.ObjectBinaryToText$1(aInput,aOutput,aEncoding);
    } finally {
      Conv = rtl.freeLoc(Conv);
    };
  };
  this.vaInt64 = 16;
  $mod.$init = function () {
    $impl.ClassList = Object.create(null);
  };
},[],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.IntConstList = null;
  rtl.createClass($impl,"TIntConst",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.IntegerType = null;
      this.IntToIdentFn = null;
    };
    this.$final = function () {
      this.IntToIdentFn = undefined;
      pas.System.TObject.$final.call(this);
    };
  });
  $impl.StringListAnsiCompare = function (List, Index1, Index) {
    var Result = 0;
    Result = List.DoCompareText(List.FList[Index1].FString,List.FList[Index].FString);
    return Result;
  };
  $impl.TMSGrow = 4096;
  $impl.FilerSignatureInt = 809914452;
  rtl.createClass($impl,"TPosComponent",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FPos = 0;
      this.FComponent = null;
    };
    this.$final = function () {
      this.FComponent = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.Create$1 = function (APos, AComponent) {
      this.FPos = APos;
      this.FComponent = AComponent;
      return this;
    };
  });
  $impl.ClassList = null;
  $mod.$resourcestrings = {SReadError: {org: "Could not read data from stream"}, SWriteError: {org: "Could not write data to stream"}, SMemoryStreamError: {org: "Could not allocate memory"}};
});
rtl.module("Web",["System","Types","JS"],function () {
  "use strict";
  var $mod = this;
});
rtl.module("CustApp",["System","Classes","SysUtils","Types","JS"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"TCustomApplication",pas.Classes.TComponent,function () {
    this.$init = function () {
      pas.Classes.TComponent.$init.call(this);
      this.FExceptObjectJS = undefined;
      this.FTerminated = false;
      this.FOptionChar = "";
      this.FCaseSensitiveOptions = false;
      this.FStopOnException = false;
      this.FExceptionExitCode = 0;
      this.FExceptObject = null;
    };
    this.$final = function () {
      this.FExceptObject = undefined;
      pas.Classes.TComponent.$final.call(this);
    };
    this.Create$1 = function (AOwner) {
      pas.Classes.TComponent.Create$1.call(this,AOwner);
      this.FOptionChar = "-";
      this.FCaseSensitiveOptions = true;
      this.FStopOnException = false;
      return this;
    };
    this.HandleException = function (Sender) {
      this.ShowException(this.FExceptObject);
      if (this.FStopOnException) this.Terminate$1(this.FExceptionExitCode);
      if (Sender === null) ;
    };
    this.Initialize = function () {
      this.FTerminated = false;
    };
    this.Run = function () {
      do {
        this.FExceptObject = null;
        this.FExceptObjectJS = null;
        try {
          this.DoRun();
        } catch ($e) {
          if (pas.SysUtils.Exception.isPrototypeOf($e)) {
            var E = $e;
            this.FExceptObject = E;
            this.FExceptObjectJS = E;
            this.HandleException(this);
          } else {
            this.FExceptObject = null;
            this.FExceptObjectJS = $e;
          }
        };
        break;
      } while (!this.FTerminated);
    };
    this.Terminate$1 = function (AExitCode) {
      this.FTerminated = true;
      rtl.exitcode = AExitCode;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
});
rtl.module("browserapp",["System","Classes","SysUtils","Types","JS","Web","CustApp"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  rtl.createClass($mod,"TBrowserApplication",pas.CustApp.TCustomApplication,function () {
    this.DoRun = function () {
    };
    this.ShowException = function (E) {
      var S = "";
      if (E !== null) {
        S = E.$classname + ": " + E.fMessage}
       else if (this.FExceptObjectJS) S = rtl.getObject(this.FExceptObjectJS).toString();
      window.alert("Unhandled exception caught:" + S);
    };
    this.HandleException = function (Sender) {
      if (pas.SysUtils.Exception.isPrototypeOf(this.FExceptObject)) this.ShowException(this.FExceptObject);
      pas.CustApp.TCustomApplication.HandleException.call(this,Sender);
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  this.ReloadEnvironmentStrings = function () {
    var I = 0;
    var S = "";
    var A = [];
    var P = [];
    if ($impl.EnvNames != null) pas.SysUtils.FreeAndNil({p: $impl, get: function () {
        return this.p.EnvNames;
      }, set: function (v) {
        this.p.EnvNames = v;
      }});
    $impl.EnvNames = new Object();
    S = window.location.search;
    S = pas.System.Copy(S,2,S.length - 1);
    A = S.split("&");
    for (var $l1 = 0, $end2 = rtl.length(A) - 1; $l1 <= $end2; $l1++) {
      I = $l1;
      P = A[I].split("=");
      if (rtl.length(P) === 2) {
        $impl.EnvNames[decodeURIComponent(P[0])] = decodeURIComponent(P[1])}
       else if (rtl.length(P) === 1) $impl.EnvNames[decodeURIComponent(P[0])] = "";
    };
  };
  $mod.$init = function () {
    pas.System.IsConsole = true;
    pas.System.OnParamCount = $impl.GetParamCount;
    pas.System.OnParamStr = $impl.GetParamStr;
    $mod.ReloadEnvironmentStrings();
    $impl.ReloadParamStrings();
    pas.SysUtils.OnGetEnvironmentVariable = $impl.MyGetEnvironmentVariable;
    pas.SysUtils.OnGetEnvironmentVariableCount = $impl.MyGetEnvironmentVariableCount;
    pas.SysUtils.OnGetEnvironmentString = $impl.MyGetEnvironmentString;
  };
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.EnvNames = null;
  $impl.Params = [];
  $impl.ReloadParamStrings = function () {
    $impl.Params = rtl.arraySetLength($impl.Params,"",1);
    $impl.Params[0] = window.location.pathname;
  };
  $impl.GetParamCount = function () {
    var Result = 0;
    Result = rtl.length($impl.Params) - 1;
    return Result;
  };
  $impl.GetParamStr = function (Index) {
    var Result = "";
    Result = $impl.Params[Index];
    return Result;
  };
  $impl.MyGetEnvironmentVariable = function (EnvVar) {
    var Result = "";
    Result = "" + $impl.EnvNames[EnvVar];
    return Result;
  };
  $impl.MyGetEnvironmentVariableCount = function () {
    var Result = 0;
    Result = rtl.length(Object.getOwnPropertyNames($impl.EnvNames));
    return Result;
  };
  $impl.MyGetEnvironmentString = function (Index) {
    var Result = "";
    Result = "" + $impl.EnvNames[Object.getOwnPropertyNames($impl.EnvNames)[Index]];
    return Result;
  };
});
rtl.module("libjquery",["System","JS","Web"],function () {
  "use strict";
  var $mod = this;
});
rtl.module("webwidget",["System","Classes","SysUtils","JS","Web"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  this.SElementData = "wwElement";
  this.STopElementData = $mod.SElementData + "Top";
  this.SContentElementData = $mod.SElementData + "Content";
  this.SElementClass = "wwClass";
  this.sEventAbort = "abort";
  this.SEventAnimationCancel = "animationcancel";
  this.SEventAnimationEnd = "animationend";
  this.SEventAnimationIteration = "animationiteration";
  this.SEventAnimationStart = "animationstart";
  this.sEventAuxClick = "auxclick";
  this.sEventBlur = "blur";
  this.SEventCancel = "cancel";
  this.SEventCanPlay = "canplay";
  this.SEventCanPlayThrough = "canplaythrough";
  this.SEventChange = "change";
  this.sEventClick = "click";
  this.sEventCompositionEnd = "compositionend";
  this.sEventCompositionStart = "compositionstart";
  this.sEventCompositionUpdate = "compositionupdate";
  this.sEventContextMenu = "contextmenu";
  this.sEventCopy = "copy";
  this.sEventCut = "cut";
  this.sEventCueChange = "cuechange";
  this.sEventDblClick = "dblclick";
  this.sEventDurationChange = "durationchange";
  this.sEventEnded = "ended";
  this.sEventError = "error";
  this.sEventFocus = "focus";
  this.sEventFocusIn = "focusin";
  this.sEventFocusOut = "focusout";
  this.SEventGotPointerCapture = "gotpointercapture";
  this.SEventInput = "input";
  this.SEventInvalid = "invalid";
  this.sEventKeyDown = "keydown";
  this.sEventKeyPress = "keypress";
  this.sEventKeyUp = "keyup";
  this.sEventLoad = "load";
  this.sEventLoadedData = "loadeddata";
  this.sEventLoadedMetaData = "loadedmetadata";
  this.sEventLoadend = "loadend";
  this.sEventLoadStart = "loadstart";
  this.SEventLostPointerCapture = "lostpointercapture";
  this.sEventMouseDown = "mousedown";
  this.sEventMouseEnter = "mouseenter";
  this.sEventMouseLeave = "mouseleave";
  this.sEventMouseMove = "mousemove";
  this.sEventMouseOut = "mouseout";
  this.sEventMouseUp = "mouseup";
  this.sEventOverFlow = "overflow";
  this.sEventPaste = "paste";
  this.sEventPause = "pause";
  this.sEventPlay = "play";
  this.SEventPointerCancel = "pointercancel";
  this.SEventPointerDown = "pointerdown";
  this.SEventPointerEnter = "pointerenter";
  this.SEventPointerLeave = "pointerleave";
  this.SEventPointerMove = "pointermove";
  this.SEventPointerOut = "pointerout";
  this.SEventPointerOver = "pointerover";
  this.SEventPointerUp = "pointerup";
  this.sEventReset = "reset";
  this.sEventResize = "resize";
  this.sEventScroll = "scroll";
  this.sEventSelect = "select";
  this.sEventSubmit = "submit";
  this.sEventTouchStart = "touchstart";
  this.SEventTransitionCancel = "transitioncancel";
  this.SEventTransitionEnd = "transitionend";
  this.SEventTransitionRun = "transitionrun";
  this.SEventTransitionStart = "transitionstart";
  this.SEventWheel = "wheel";
  this.SErrNotRendered = "Cannot perform this operation: Widget not rendered";
  rtl.createClass($mod,"EWidgets",pas.SysUtils.Exception,function () {
  });
  $mod.$rtti.$MethodVar("THTMLNotifyEvent",{procsig: rtl.newTIProcSig([["Sender",pas.System.$rtti["TObject"]],["Event",pas.Web.$rtti["TJSEvent"]]]), methodkind: 0});
  rtl.recNewT($mod,"TEventDispatch",function () {
    this.MsgStr = "";
    this.HTMLEvent = null;
    this.EventHandler = null;
    this.$eq = function (b) {
      return (this.MsgStr === b.MsgStr) && (this.HTMLEvent === b.HTMLEvent) && rtl.eqCallback(this.EventHandler,b.EventHandler);
    };
    this.$assign = function (s) {
      this.MsgStr = s.MsgStr;
      this.HTMLEvent = s.HTMLEvent;
      this.EventHandler = s.EventHandler;
      return this;
    };
  });
  this.TStylePriority = {"0": "spNone", spNone: 0, "1": "spImportant", spImportant: 1};
  $mod.$rtti.$Enum("TStylePriority",{minvalue: 0, maxvalue: 1, ordtype: 1, enumtype: this.TStylePriority});
  rtl.createClass($mod,"TStyleItem",pas.Classes.TCollectionItem,function () {
    this.$init = function () {
      pas.Classes.TCollectionItem.$init.call(this);
      this.FPriority = 0;
      this.FName = "";
      this.FValue = "";
      this.FImported = false;
    };
    this.SetPriority = function (AValue) {
      if (this.FPriority === AValue) return;
      this.FPriority = AValue;
      this.MarkDirty();
    };
    this.SetValue = function (AValue) {
      if (this.FValue === AValue) return;
      this.FValue = AValue;
      this.MarkDirty();
    };
    this.SetName = function (AValue) {
      if (AValue === this.FName) return;
      this.FName = AValue;
      this.MarkDirty();
    };
    this.MarkDirty = function () {
      if (this.FCollection != null) this.FCollection.MarkDirty(this);
    };
    this.Assign = function (Source) {
      var SI = null;
      if ($mod.TStyleItem.isPrototypeOf(Source)) {
        SI = rtl.as(Source,$mod.TStyleItem);
        this.FName = SI.FName;
        this.FValue = SI.FValue;
        this.FImported = SI.FImported;
        this.MarkDirty();
      } else pas.Classes.TPersistent.Assign.call(this,Source);
    };
    var $r = this.$rtti;
    $r.addProperty("Name",2,rtl.string,"FName","SetName");
    $r.addProperty("Value",2,rtl.string,"FValue","SetValue");
    $r.addProperty("Priority",2,$mod.$rtti["TStylePriority"],"FPriority","SetPriority");
  });
  rtl.createClass($mod,"TWebWidgetStyles",pas.Classes.TOwnedCollection,function () {
    this.GetStyleItem = function (aIndex) {
      var Result = null;
      Result = this.GetItem(aIndex);
      return Result;
    };
    this.MarkDirty = function (aItem) {
      var El = null;
      if (this.Widget() != null) {
        El = this.Widget().GetElement();
        if (El != null) this.ApplyToDOM(El,aItem);
      };
    };
    var Prios = ["","important"];
    this.ApplyToDOM = function (aElement, aItem) {
      if (aItem.FName !== "") if (aItem.FValue !== "") {
        aElement.style.setProperty(aItem.FName,aItem.FValue,Prios[aItem.FPriority])}
       else aElement.style.removeProperty(aItem.FName);
    };
    this.Widget = function () {
      var Result = null;
      Result = this.Owner();
      return Result;
    };
    this.Add$1 = function (aName, aValue) {
      var Result = null;
      Result = rtl.as(this.Add(),$mod.TStyleItem);
      Result.FName = aName;
      if (aValue !== "") Result.SetValue(aValue);
      return Result;
    };
    this.EnsureStyle = function (aName, aValue) {
      var Result = null;
      Result = this.FindStyle(aName);
      if (Result === null) {
        Result = this.Add$1(aName,aValue)}
       else if (aValue !== "") Result.SetValue(aValue);
      return Result;
    };
    this.IndexOfStyle = function (aName) {
      var Result = 0;
      Result = this.GetCount() - 1;
      while ((Result >= 0) && !pas.SysUtils.SameText(aName,this.GetStyleItem(Result).FName)) Result -= 1;
      return Result;
    };
    this.FindStyle = function (aName) {
      var Result = null;
      var Idx = 0;
      Idx = this.IndexOfStyle(aName);
      if (Idx === -1) {
        Result = null}
       else Result = this.GetStyleItem(Idx);
      return Result;
    };
    this.RemoveStyle = function (aName) {
      var Result = "";
      var I = 0;
      var el = null;
      I = this.IndexOfStyle(aName);
      if (I !== -1) {
        Result = this.GetStyleItem(I).FValue;
        this.Delete(I);
      };
      if (this.Widget() != null) {
        el = this.Widget().GetElement();
        if (el != null) {
          if (Result === "") Result = el.style.getPropertyValue(aName);
          el.style.removeProperty(aName);
        };
      };
      return Result;
    };
    this.RefreshFromDOM = function (aElement, DoClear) {
      var S = null;
      var I = 0;
      var K = "";
      var W = null;
      var itm = null;
      if (aElement === null) {
        W = this.Widget();
        if (W != null) aElement = W.GetElement();
        if (aElement === null) return;
      };
      if (DoClear) this.Clear();
      S = aElement.style;
      for (var $l1 = 0, $end2 = S.length - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        K = S.item(I);
        itm = this.FindStyle(K);
        if (itm === null) {
          itm = this.Add$1(K,"");
          itm.FImported = true;
        };
        itm.FValue = S.getPropertyValue(K);
        var $tmp3 = pas.SysUtils.LowerCase(S.getPropertyPriority(K));
        if ($tmp3 === "important") itm.FPriority = 1;
      };
    };
    this.ClearImported = function () {
      var I = 0;
      I = this.GetCount() - 1;
      while (I >= 0) {
        if (this.GetStyleItem(I).FImported) this.Delete(I);
        I -= 1;
      };
    };
    this.ApplyToDOM$1 = function (aElement) {
      var I = 0;
      if ((aElement === null) && (this.Widget() !== null)) aElement = this.Widget().GetElement();
      if (aElement !== null) for (var $l1 = 0, $end2 = this.GetCount() - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        this.ApplyToDOM(aElement,this.GetStyleItem(I));
      };
    };
  });
  this.TStyleRefresh = {"0": "srOnElementID", srOnElementID: 0, "1": "srAlways", srAlways: 1, "2": "srNever", srNever: 2};
  $mod.$rtti.$Enum("TStyleRefresh",{minvalue: 0, maxvalue: 2, ordtype: 1, enumtype: this.TStyleRefresh});
  rtl.createClass($mod,"TCustomWebWidget",pas.Classes.TComponent,function () {
    this.MaxEvents = 66;
    this.WidgetID = 0;
    this.FEventNames = [$mod.sEventAbort,$mod.SEventAnimationCancel,$mod.SEventAnimationEnd,$mod.SEventAnimationIteration,$mod.SEventAnimationStart,$mod.sEventAuxClick,$mod.sEventBlur,$mod.SEventCancel,$mod.SEventCanPlay,$mod.SEventCanPlayThrough,$mod.SEventChange,$mod.sEventClick,$mod.sEventCompositionEnd,$mod.sEventCompositionStart,$mod.sEventCompositionUpdate,$mod.sEventContextMenu,$mod.sEventCopy,$mod.sEventCut,$mod.sEventCueChange,$mod.sEventDblClick,$mod.sEventDurationChange,$mod.sEventEnded,$mod.sEventError,$mod.sEventFocus,$mod.sEventFocusIn,$mod.sEventFocusOut,$mod.SEventGotPointerCapture,$mod.SEventInput,$mod.SEventInvalid,$mod.sEventKeyDown,$mod.sEventKeyPress,$mod.sEventKeyUp,$mod.sEventLoad,$mod.sEventLoadedData,$mod.sEventLoadedMetaData,$mod.sEventLoadend,$mod.sEventLoadStart,$mod.SEventLostPointerCapture,$mod.sEventMouseDown,$mod.sEventMouseEnter,$mod.sEventMouseLeave,$mod.sEventMouseMove,$mod.sEventMouseOut,$mod.sEventMouseUp,$mod.sEventOverFlow,$mod.sEventPaste,$mod.sEventPause,$mod.sEventPlay,$mod.SEventPointerCancel,$mod.SEventPointerDown,$mod.SEventPointerEnter,$mod.SEventPointerLeave,$mod.SEventPointerMove,$mod.SEventPointerOut,$mod.SEventPointerOver,$mod.SEventPointerUp,$mod.sEventReset,$mod.sEventResize,$mod.sEventScroll,$mod.sEventSelect,$mod.sEventSubmit,$mod.sEventTouchStart,$mod.SEventTransitionCancel,$mod.SEventTransitionEnd,$mod.SEventTransitionRun,$mod.SEventTransitionStart,$mod.SEventWheel];
    this.$init = function () {
      pas.Classes.TComponent.$init.call(this);
      this.FAfterRenderHTML = null;
      this.FAfterUnRenderHTML = null;
      this.FBeforeRenderHTML = null;
      this.FBeforeUnRenderHTML = null;
      this.FParent = null;
      this.FMyHook = null;
      this.FParentElement = null;
      this.FElement = null;
      this.FOwnsElement = false;
      this.FParentID = "";
      this.FElementID = "";
      this.FChildren = null;
      this.FClasses = "";
      this.FMyEvents = null;
      this.FStyleRefresh = 0;
      this.FStyles = null;
      this.FVisible = false;
      this.FDisplay = "";
    };
    this.$final = function () {
      this.FAfterRenderHTML = undefined;
      this.FAfterUnRenderHTML = undefined;
      this.FBeforeRenderHTML = undefined;
      this.FBeforeUnRenderHTML = undefined;
      this.FParent = undefined;
      this.FMyHook = undefined;
      this.FParentElement = undefined;
      this.FElement = undefined;
      this.FChildren = undefined;
      this.FMyEvents = undefined;
      this.FStyles = undefined;
      pas.Classes.TComponent.$final.call(this);
    };
    this.GetChildCount = function () {
      var Result = 0;
      Result = this.FChildren.length;
      return Result;
    };
    this.GetChild = function (aIndex) {
      var Result = null;
      if ((aIndex < 0) || (aIndex >= this.FChildren.length)) throw pas.Classes.EListError.$create("CreateFmt",[rtl.getResStr(pas.webwidget,"SErrInvalidChildIndex"),[aIndex,this.FChildren.length - 1]]);
      Result = rtl.getObject(this.FChildren[aIndex]);
      return Result;
    };
    this.GetClasses = function () {
      var Result = "";
      if (this.GetRendered()) this.FClasses = this.FElement.className;
      Result = this.FClasses;
      return Result;
    };
    this.GetElement = function () {
      var Result = null;
      var El = null;
      if (this.FElement === null) {
        if (this.FElementID !== "") {
          El = this.$class.FindElement(this.FElementID);
          if (El != null) this.ApplyWidgetSettings(El);
          this.FElement = El;
          this.ApplyData();
        };
      };
      Result = this.FElement;
      return Result;
    };
    this.GetExternalElement = function () {
      var Result = false;
      Result = this.FElementID !== "";
      return Result;
    };
    this.GetHTMLEvent = function (AIndex) {
      var Result = null;
      var Fun = undefined;
      Result = null;
      if ((this.FMyEvents != null) && (AIndex >= 0) && (AIndex <= 66)) {
        Fun = this.FMyEvents[this.FEventNames[AIndex]];
        if (!pas.JS.isUndefined(Fun)) Result = Fun;
      };
      return Result;
    };
    this.GetIsElementDirty = function () {
      var Result = false;
      Result = this.GetRendered();
      return Result;
    };
    this.GetParent = function () {
      var Result = null;
      Result = this.FParent;
      return Result;
    };
    this.GetParentElement = function () {
      var Result = null;
      var El = null;
      if (this.FParentElement === null) {
        El = this.GetTopElement();
        if (El != null) {
          this.FParentElement = El.parentElement}
         else if (this.FParentID !== "") {
          this.FParentElement = this.$class.FindElement(this.FParentID)}
         else if (this.FParent != null) {
          this.FParentElement = this.FParent.GetContentElement()}
         else this.FParentElement = this.$class.DefaultParentElement();
      };
      Result = this.FParentElement;
      return Result;
    };
    this.GetParentID = function () {
      var Result = "";
      var E = null;
      Result = "";
      E = this.GetParentElement();
      if (E != null) {
        Result = E.id}
       else Result = this.FParentID;
      return Result;
    };
    this.GetElementID = function () {
      var Result = "";
      var El = null;
      El = this.GetElement();
      if (El != null) {
        Result = El.id}
       else Result = this.FElementID;
      return Result;
    };
    this.GetRendered = function () {
      var Result = false;
      Result = this.FElement !== null;
      return Result;
    };
    this.GetVisible = function () {
      var Result = false;
      Result = this.FVisible;
      return Result;
    };
    this.SetClasses = function (AValue) {
      this.FClasses = this.$class.AddClasses(AValue,this.WidgetClasses(),false);
      if (this.GetRendered()) this.FElement.className = this.FClasses;
    };
    this.SetDataset = function (aName, AValue) {
      var El = null;
      El = this.GetElement();
      if (El === null) throw $mod.EWidgets.$create("Create$1",[$mod.SErrNotRendered]);
      El.dataset[aName] = AValue;
    };
    this.SetElementID = function (AValue) {
      if (this.FElementID === AValue) return;
      if (AValue !== "") {
        if (this.FParentID !== "") throw $mod.EWidgets.$create("Create$1",[rtl.getResStr(pas.webwidget,"SErrCannotSetParentAndElementID")]);
        if (this.$class.FixedElement() !== null) throw $mod.EWidgets.$create("Create$1",[rtl.getResStr(pas.webwidget,"SErrElementIDNotAllowed")]);
        this.FElementID = AValue;
      } else {
        this.FElementID = AValue;
        if (this.GetRendered()) this.UnRender(this.GetParentElement());
      };
    };
    this.SetHTMLEvent = function (AIndex, AValue) {
      var EventName = "";
      if ((AIndex < 0) || (AIndex > 66)) return;
      EventName = this.FEventNames[AIndex];
      if (AValue != null) {
        this.AddEvent(EventName,AValue)}
       else this.DeleteEvent(EventName);
    };
    this.SetParent = function (AValue) {
      var ReRender = false;
      if (AValue === this.FParent) return;
      if (this.$class.FixedParent() !== null) throw $mod.EWidgets.$create("Create$1",[rtl.getResStr(pas.webwidget,"SErrParentNotAllowed")]);
      if (this.FParent != null) this.FParent.RemoveChild(this);
      ReRender = this.GetRendered();
      if (ReRender) this.UnRender(this.GetParentElement());
      this.InvalidateParentElement();
      if (AValue != null) {
        this.FParentID = "";
        AValue.AddChild(this);
      };
      if (ReRender && (this.GetParentElement() != null)) {
        this.FElement = this.RenderHTML(this.GetParentElement());
        if (this.FElement != null) this.ApplyData();
      };
    };
    this.SetParentID = function (AValue) {
      var ReRender = false;
      if (this.FParentID === AValue) return;
      if (AValue !== "") {
        if (this.FElementID !== "") throw $mod.EWidgets.$create("Create$1",[rtl.getResStr(pas.webwidget,"SErrCannotSetParentAndElementID")]);
        if (this.$class.FixedParent() !== null) throw $mod.EWidgets.$create("Create$1",[rtl.getResStr(pas.webwidget,"SErrParentIDNotAllowed")]);
      };
      ReRender = this.GetRendered();
      if (ReRender) this.UnRender(this.GetParentElement());
      if ((AValue !== "") && (this.FParent != null)) this.FParent.RemoveChild(this);
      this.FParentID = AValue;
      this.InvalidateParentElement();
      if (ReRender && (this.GetParentElement() != null)) this.EnsureElement();
    };
    this.AddChild = function (aValue) {
      if (aValue === null) return;
      aValue.FParent = this;
      if (this.FChildren.indexOf(aValue) === -1) this.FChildren.push(aValue);
    };
    this.RemoveChild = function (aValue) {
      var I = 0;
      if (aValue === null) return;
      I = this.FChildren.indexOf(aValue);
      if (I >= 0) {
        this.FChildren.splice(I,1);
        aValue.FParent = null;
      };
    };
    this.SetStyles = function (AValue) {
      if (this.FStyles === AValue) return;
      this.FStyles.Assign(AValue);
    };
    this.SetVisible = function (AValue) {
      var el = null;
      if (AValue === this.FVisible) return;
      el = this.GetElement();
      if (el != null) {
        this.ApplyVisible(el,AValue)}
       else this.FVisible = AValue;
    };
    this.EventEntry = function (aEvent) {
      var R = $mod.TEventDispatch.$new();
      var Fun = undefined;
      R.MsgStr = aEvent.type;
      R.HTMLEvent = aEvent;
      if (this.FMyEvents != null) {
        Fun = this.FMyEvents[R.MsgStr]}
       else Fun = null;
      if (!(pas.JS.isUndefined(Fun) || pas.JS.isNull(Fun))) R.EventHandler = Fun;
      this.DispatchStr(R);
      if (R.EventHandler !== null) R.EventHandler(this,R.HTMLEvent);
    };
    this.RemoveEvent = function (aElement, aEvent) {
      aElement.RemoveEventListener(aEvent,this.FMyHook);
    };
    this.HookupEvent = function (aElement, aEvent) {
      aElement.addEventListener(aEvent,this.FMyHook);
    };
    this.HookupEvents = function (aElement) {
      var Event = "";
      if (this.FMyEvents != null) for (var $in1 = Object.keys(this.FMyEvents), $l2 = 0, $end3 = rtl.length($in1) - 1; $l2 <= $end3; $l2++) {
        Event = $in1[$l2];
        this.HookupEvent(aElement,Event);
      };
    };
    this.AddEvent = function (aName, AHandler) {
      var el = null;
      if (this.FMyEvents === null) this.FMyEvents = new Object();
      this.FMyEvents[aName] = AHandler;
      el = this.GetElement();
      if (el != null) this.HookupEvent(el,aName);
    };
    this.DeleteEvent = function (aName) {
      var el = null;
      if ((this.FMyEvents !== null) && this.FMyEvents.hasOwnProperty(aName)) pas.JS.JSDelete(this.FMyEvents,aName);
      el = this.GetElement();
      if (el != null) this.RemoveEvent(el,aName);
    };
    this.FixedParent = function () {
      var Result = null;
      Result = null;
      return Result;
    };
    this.DefaultParentElement = function () {
      var Result = null;
      Result = null;
      return Result;
    };
    this.DefaultParent = function () {
      var Result = null;
      Result = null;
      return Result;
    };
    this.FixedElement = function () {
      var Result = null;
      Result = null;
      return Result;
    };
    this.GenerateID = function () {
      var Result = "";
      $mod.TCustomWebWidget.WidgetID += 1;
      Result = "ww-" + pas.SysUtils.IntToStr(this.WidgetID);
      return Result;
    };
    this.FindElement = function (aID) {
      var Result = null;
      Result = document.getElementById(aID);
      return Result;
    };
    this.CreateElement = function (aTag, aID) {
      var Result = null;
      Result = document.createElement(aTag);
      if (aID !== "") Result.id = aID;
      return Result;
    };
    this.GetContentElement = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.GetTopElement = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.DisplayElementName = function () {
      var Result = "";
      Result = this.FName;
      if (Result === "") Result = " <" + this.HTMLTag() + ">";
      if (this.FElement != null) Result = Result + "#" + this.FElement.id;
      Result = Result + " (Type: " + this.$classname + ")";
      return Result;
    };
    this.EnsureElement = function () {
      var Result = null;
      var P = null;
      Result = this.GetElement();
      if (Result === null) {
        if (this.GetParent() != null) this.GetParent().EnsureElement();
        P = this.GetParentElement();
        if ((P === null) && (this.$class.FixedElement() === null)) {
          throw $mod.EWidgets.$create("CreateFmt",[rtl.getResStr(pas.webwidget,"SErrCannotRenderWithoutParent"),[this.DisplayElementName()]])}
         else {
          Result = this.RenderHTML(P);
          this.FOwnsElement = true;
          this.FElement = Result;
          this.ApplyData();
        };
      };
      return Result;
    };
    this.InvalidateParentElement = function () {
      this.FParentElement = null;
    };
    this.InvalidateElement = function () {
      if (this.FStyles.GetCount() > 0) this.FStyles.ClearImported();
      this.FElement = null;
    };
    this.WidgetClasses = function () {
      var Result = "";
      Result = "";
      return Result;
    };
    this.CreateStyles = function () {
      var Result = null;
      Result = $mod.TWebWidgetStyles.$create("Create$2",[this,$mod.TStyleItem]);
      return Result;
    };
    this.ApplyVisible = function (aElement, AValue) {
      if (AValue) {
        if (this.FDisplay !== "") {
          aElement.style.setProperty("display",this.FDisplay)}
         else aElement.style.removeProperty("display");
      } else {
        this.FDisplay = aElement.style.getPropertyValue("display");
        aElement.style.setProperty("display","none");
      };
      this.FVisible = AValue;
    };
    this.ApplyWidgetSettings = function (aElement) {
      var S = "";
      if (aElement.id === "") aElement.id = this.$class.GenerateID();
      S = this.$class.AddClasses(this.FClasses,this.WidgetClasses(),false);
      if (S !== "") this.$class.AddClasses$1(aElement,S,false);
      if (this.FStyles.GetCount() > 0) this.FStyles.ApplyToDOM$1(aElement);
      if (!this.FVisible) this.ApplyVisible(aElement,this.FVisible);
      if ((this.FStyleRefresh === 1) || ((this.FElementID !== "") && (this.FElementID !== ""))) this.FStyles.RefreshFromDOM(aElement,false);
    };
    this.DoRenderHTML = function (aParent, aElement) {
      var Result = null;
      if (aParent === null) window.console.log(this.DisplayElementName() + ": render without parent!");
      Result = aElement;
      return Result;
    };
    this.ApplyData = function () {
      var $Self = this;
      var AID = "";
      function MaybeSet(El, AName) {
        if (El != null) El.dataset[AName] = AID;
      };
      AID = $Self.GetElementID();
      if ($Self.GetElement() != null) $Self.GetElement().dataset[$mod.SElementClass] = $Self.$classname;
      MaybeSet($Self.GetElement(),$mod.SElementData);
      MaybeSet($Self.GetTopElement(),$mod.STopElementData);
      if ($Self.$class.AllowChildren()) MaybeSet($Self.GetContentElement(),$mod.SContentElementData);
    };
    this.RemoveData = function () {
      var $Self = this;
      function MaybeUnSet(El, AName) {
        if (El != null) pas.JS.JSDelete(El.dataset,AName);
      };
      MaybeUnSet($Self.GetElement(),$mod.SElementData);
      MaybeUnSet($Self.GetTopElement(),$mod.STopElementData);
      MaybeUnSet($Self.GetContentElement(),$mod.SContentElementData);
    };
    this.RenderHTML = function (aParent) {
      var Result = null;
      var aTag = "";
      aTag = this.HTMLTag();
      if (aTag === "") {
        Result = null}
       else {
        Result = this.$class.FixedElement();
        if (Result === null) Result = this.$class.CreateElement(this.HTMLTag(),this.$class.GenerateID());
      };
      if ((Result != null) && (aParent != null)) aParent.appendChild(Result);
      if (this.FBeforeRenderHTML != null) this.FBeforeRenderHTML(this);
      Result = this.DoRenderHTML(aParent,Result);
      if (Result != null) {
        this.ApplyWidgetSettings(Result);
        this.HookupEvents(Result);
      };
      if (this.FAfterRenderHTML != null) this.FAfterRenderHTML(this);
      return Result;
    };
    this.DoUnRender = function (aParent) {
      if ((aParent != null) && (this.FElement != null)) {
        if (this.FOwnsElement) aParent.removeChild(this.FElement);
        this.FElement = null;
      };
    };
    this.UnRender = function (aParent) {
      if (this.FBeforeUnRenderHTML != null) this.FBeforeUnRenderHTML(this);
      this.RemoveData();
      this.DoUnRender(aParent);
      if (this.FAfterUnRenderHTML != null) this.FAfterUnRenderHTML(this);
    };
    this.Create$1 = function (aOwner) {
      pas.Classes.TComponent.Create$1.call(this,aOwner);
      this.FChildren = new Array();
      this.FStyles = this.CreateStyles();
      this.FMyHook = rtl.createCallback(this,"EventEntry");
      this.FParent = this.$class.DefaultParent();
      this.FVisible = true;
      return this;
    };
    this.Destroy = function () {
      var I = 0;
      var C = null;
      for (var $l1 = 0, $end2 = this.FChildren.length - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        C = rtl.getObject(this.FChildren[I]);
        this.FChildren[I] = null;
        C = rtl.freeLoc(C);
      };
      this.SetParent(null);
      this.SetParentID("");
      this.FChildren = null;
      pas.SysUtils.FreeAndNil({p: this, get: function () {
          return this.p.FStyles;
        }, set: function (v) {
          this.p.FStyles = v;
        }});
      pas.Classes.TComponent.Destroy.call(this);
    };
    this.AllowChildren = function () {
      var Result = false;
      Result = true;
      return Result;
    };
    this.AddClasses = function (Source, aClasses, Normalize) {
      var Result = "";
      var T = [];
      var S = "";
      Result = Source;
      if (Normalize) Result = Result.replace(new RegExp("\\s\\s+","g")," ");
      if (aClasses === "") return Result;
      T = Result.split(" ");
      for (var $in1 = aClasses.split(" "), $l2 = 0, $end3 = rtl.length($in1) - 1; $l2 <= $end3; $l2++) {
        S = $in1[$l2];
        if (S !== "") {
          if (T.indexOf(S) === -1) T.push(S);
        };
      };
      Result = T.join(" ");
      return Result;
    };
    this.AddClasses$1 = function (el, aClasses, Normalize) {
      var Result = "";
      Result = this.AddClasses(el.className,aClasses,Normalize);
      el.className = pas.SysUtils.Trim(Result);
      return Result;
    };
    this.Refresh = function () {
      var P = null;
      P = null;
      if (this.FElement != null) {
        P = this.GetParentElement();
        if ((P != null) && (this.FElementID === "")) P.removeChild(this.FElement);
      };
      this.InvalidateParentElement();
      this.InvalidateElement();
      this.EnsureElement();
    };
    this.FindWidgetByID = function (aElementID, Recurse) {
      var Result = null;
      var I = 0;
      Result = null;
      if (aElementID === "") return Result;
      if (aElementID === this.GetElementID()) return this;
      I = this.GetChildCount() - 1;
      while ((I >= 0) && (Result === null)) {
        Result = this.GetChild(I);
        if (Result.GetElementID() !== aElementID) Result = null;
        I -= 1;
      };
      if ((Result === null) && Recurse) {
        I = this.GetChildCount() - 1;
        while ((I >= 0) && (Result === null)) {
          Result = this.GetChild(I).FindWidgetByID(aElementID,true);
          I -= 1;
        };
      };
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($mod,"TWebWidget",$mod.TCustomWebWidget,function () {
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("ParentID",3,rtl.string,"GetParentID","SetParentID");
    $r.addProperty("ElementID",3,rtl.string,"GetElementID","SetElementID");
    $r.addProperty("Classes",3,rtl.string,"GetClasses","SetClasses");
    $r.addProperty("Styles",2,$mod.$rtti["TWebWidgetStyles"],"FStyles","SetStyles");
    $r.addProperty("StyleRefresh",0,$mod.$rtti["TStyleRefresh"],"FStyleRefresh","FStyleRefresh");
    $r.addProperty("Visible",3,rtl.boolean,"GetVisible","SetVisible");
    $r.addProperty("BeforeRenderHTML",0,pas.Classes.$rtti["TNotifyEvent"],"FBeforeRenderHTML","FBeforeRenderHTML");
    $r.addProperty("AfterRenderHTML",0,pas.Classes.$rtti["TNotifyEvent"],"FAfterRenderHTML","FAfterRenderHTML");
    $r.addProperty("OnAbort",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 0});
    $r.addProperty("OnAnimationCancel",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 1});
    $r.addProperty("OnAnimationEnd",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 2});
    $r.addProperty("OnAnimationIteration",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 3});
    $r.addProperty("OnAnimationStart",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 4});
    $r.addProperty("OnAuxClick",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 5});
    $r.addProperty("OnBlur",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 6});
    $r.addProperty("OnCancel",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 7});
    $r.addProperty("OnCanPlay",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 8});
    $r.addProperty("OnCanPlayThrough",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 9});
    $r.addProperty("OnChange",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 10});
    $r.addProperty("OnClick",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 11});
    $r.addProperty("OnCompositionEnd",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 12});
    $r.addProperty("OnCompositionStart",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 13});
    $r.addProperty("OnCompositionUpdate",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 14});
    $r.addProperty("OnContextMenu",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 15});
    $r.addProperty("OnCopy",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 16});
    $r.addProperty("OnCut",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 17});
    $r.addProperty("OnCueChange",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 18});
    $r.addProperty("OnDblClick",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 19});
    $r.addProperty("OnDurationChange",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 20});
    $r.addProperty("OnEnded",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 21});
    $r.addProperty("OnError",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 22});
    $r.addProperty("OnFocus",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 23});
    $r.addProperty("OnFocusIn",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 24});
    $r.addProperty("OnFocusOut",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 25});
    $r.addProperty("OnGotPointerCapture",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 26});
    $r.addProperty("OnInput",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 27});
    $r.addProperty("OnInvalid",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 28});
    $r.addProperty("OnKeyDown",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 29});
    $r.addProperty("OnKeyPress",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 30});
    $r.addProperty("OnKeyUp",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 31});
    $r.addProperty("OnLoad",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 32});
    $r.addProperty("OnLoadedData",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 33});
    $r.addProperty("OnLoadedMetaData",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 34});
    $r.addProperty("OnLoadend",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 35});
    $r.addProperty("OnLoadStart",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 36});
    $r.addProperty("OnLostPointerCapture",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 37});
    $r.addProperty("OnMouseDown",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 38});
    $r.addProperty("OnMouseEnter",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 39});
    $r.addProperty("OnMouseLeave",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 40});
    $r.addProperty("OnMouseMove",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 41});
    $r.addProperty("OnMouseOut",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 42});
    $r.addProperty("OnMouseUp",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 43});
    $r.addProperty("OnOverFlow",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 44});
    $r.addProperty("OnPaste",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 45});
    $r.addProperty("OnPause",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 46});
    $r.addProperty("OnPlay",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 47});
    $r.addProperty("OnPointerCancel",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 48});
    $r.addProperty("OnPointerDown",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 49});
    $r.addProperty("OnPointerEnter",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 50});
    $r.addProperty("OnPointerLeave",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 51});
    $r.addProperty("OnPointerMove",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 52});
    $r.addProperty("OnPointerOut",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 53});
    $r.addProperty("OnPointerOver",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 54});
    $r.addProperty("OnPointerUp",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 55});
    $r.addProperty("OnReset",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 56});
    $r.addProperty("OnResize",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 57});
    $r.addProperty("OnScroll",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 58});
    $r.addProperty("OnSelect",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 59});
    $r.addProperty("OnSubmit",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 60});
    $r.addProperty("OnTouchStart",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 61});
    $r.addProperty("OnTransitionCancel",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 62});
    $r.addProperty("OnTransitionEnd",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 63});
    $r.addProperty("OnTransitionRun",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 64});
    $r.addProperty("OnTransitionStart",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 65});
    $r.addProperty("OnWheel",19,$mod.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 66});
  });
  rtl.createClass($mod,"TContainerWidget",$mod.TWebWidget,function () {
    this.KnownStyleCount = 6;
    this.KnownStyles = ["min-width","max-width","min-height","max-height","display","width","height"];
    this.SetKnownStyle = function (AIndex, AValue) {
      this.FStyles.EnsureStyle(this.KnownStyles[AIndex],"").SetValue(AValue);
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "div";
      return Result;
    };
    this.Create$1 = function (aOwner) {
      $mod.TCustomWebWidget.Create$1.call(this,aOwner);
      this.SetKnownStyle(0,"32px");
      this.SetKnownStyle(2,"12px");
      this.SetKnownStyle(5,"50%");
      this.SetKnownStyle(6,"50%");
      return this;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($mod,"TCustomTemplateWidget",$mod.TWebWidget,function () {
    this.$init = function () {
      $mod.TWebWidget.$init.call(this);
      this.FContainerTag = "";
      this.FTemplate = null;
    };
    this.$final = function () {
      this.FTemplate = undefined;
      $mod.TWebWidget.$final.call(this);
    };
    this.DoTemplateChanged = function (Sender) {
      if (this.GetRendered()) this.Refresh();
    };
    this.GetTemplateHTML = function () {
      var Result = "";
      Result = this.FTemplate.GetTextStr();
      return Result;
    };
    this.ApplyTemplate = function (aElement) {
      aElement.innerHTML = this.GetTemplateHTML();
    };
    this.DoRenderHTML = function (aParent, aElement) {
      var Result = null;
      if (this.FContainerTag === "") {
        this.ApplyTemplate(aParent);
        Result = aParent.firstElementChild;
      } else {
        this.ApplyTemplate(aElement);
        Result = aElement;
      };
      return Result;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = this.FContainerTag;
      return Result;
    };
    this.Create$1 = function (aOwner) {
      $mod.TCustomWebWidget.Create$1.call(this,aOwner);
      this.FTemplate = pas.Classes.TStringList.$create("Create$1");
      this.FTemplate.FOnChange = rtl.createCallback(this,"DoTemplateChanged");
      this.FContainerTag = "";
      return this;
    };
    this.Destroy = function () {
      pas.SysUtils.FreeAndNil({p: this, get: function () {
          return this.p.FTemplate;
        }, set: function (v) {
          this.p.FTemplate = v;
        }});
      $mod.TCustomWebWidget.Destroy.call(this);
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $mod.$resourcestrings = {SErrCannotSetParentAndElementID: {org: "ElementID and ParentID cannot be set at the same time."}, SErrCannotRenderWithoutParent: {org: "Cannot render without parent"}, SErrInvalidChildIndex: {org: "Invalid child index: value %d is not in valid range [0..%d]"}, SErrElementIDNotAllowed: {org: "Setting element ID is not allowed"}, SErrParentIDNotAllowed: {org: "Setting parent ID is not allowed"}, SErrParentNotAllowed: {org: "Setting parent is not allowed"}};
});
rtl.module("htmlwidgets",["System","Classes","SysUtils","webwidget","JS","Web"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"TButtonWidget",pas.webwidget.TWebWidget,function () {
    this.$init = function () {
      pas.webwidget.TWebWidget.$init.call(this);
      this.FText = "";
    };
    this.SetText = function (AValue) {
      if (this.FText === AValue) return;
      this.FText = AValue;
      if (this.GetRendered()) this.GetElement().innerText = this.FText;
    };
    this.SetName = function (NewName) {
      var Old = "";
      Old = this.FName;
      pas.Classes.TComponent.SetName.call(this,NewName);
      if ((this.FText === Old) && (4 in this.FComponentState)) this.FText = NewName;
    };
    this.ApplyWidgetSettings = function (aElement) {
      pas.webwidget.TCustomWebWidget.ApplyWidgetSettings.apply(this,arguments);
      aElement.innerText = this.FText;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "button";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($mod,"TViewPort",pas.webwidget.TCustomWebWidget,function () {
    this.FInstance = null;
    this.FixedParent = function () {
      var Result = null;
      Result = document.documentElement;
      return Result;
    };
    this.FixedElement = function () {
      var Result = null;
      Result = document.body;
      return Result;
    };
    this.DoRenderHTML = function (aParent, aElement) {
      var Result = null;
      Result = this.$class.FixedElement();
      return Result;
    };
    this.Create$1 = function (aOwner) {
      pas.webwidget.TCustomWebWidget.Create$1.call(this,aOwner);
      this.EnsureElement();
      return this;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "body";
      return Result;
    };
    this.Instance = function () {
      var Result = null;
      if (this.FInstance === null) $mod.TViewPort.FInstance = $mod.TViewPort.$create("Create$1",[null]);
      Result = this.FInstance;
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($mod,"TWebPage",pas.webwidget.TCustomWebWidget,function () {
    this.DefaultParentElement = function () {
      var Result = null;
      Result = $mod.TViewPort.Instance().GetElement();
      return Result;
    };
    this.DefaultParent = function () {
      var Result = null;
      Result = $mod.TViewPort.Instance();
      return Result;
    };
    this.DoUnRender = function (aParent) {
      pas.webwidget.TCustomWebWidget.DoUnRender.call(this,aParent);
    };
    this.Create$1 = function (AOwner) {
      pas.webwidget.TCustomWebWidget.Create$1.call(this,AOwner);
      this.SetClasses("WebPage");
      return this;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "div";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("ParentID",3,rtl.string,"GetParentID","SetParentID");
    $r.addProperty("ElementID",3,rtl.string,"GetElementID","SetElementID");
    $r.addProperty("Classes",3,rtl.string,"GetClasses","SetClasses");
    $r.addProperty("Styles",2,pas.webwidget.$rtti["TWebWidgetStyles"],"FStyles","SetStyles");
    $r.addProperty("StyleRefresh",0,pas.webwidget.$rtti["TStyleRefresh"],"FStyleRefresh","FStyleRefresh");
    $r.addProperty("Visible",3,rtl.boolean,"GetVisible","SetVisible");
    $r.addProperty("BeforeRenderHTML",0,pas.Classes.$rtti["TNotifyEvent"],"FBeforeRenderHTML","FBeforeRenderHTML");
    $r.addProperty("AfterRenderHTML",0,pas.Classes.$rtti["TNotifyEvent"],"FAfterRenderHTML","FAfterRenderHTML");
    $r.addProperty("OnAbort",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 0});
    $r.addProperty("OnAnimationCancel",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 1});
    $r.addProperty("OnAnimationEnd",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 2});
    $r.addProperty("OnAnimationIteration",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 3});
    $r.addProperty("OnAnimationStart",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 4});
    $r.addProperty("OnAuxClick",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 5});
    $r.addProperty("OnBlur",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 6});
    $r.addProperty("OnCancel",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 7});
    $r.addProperty("OnCanPlay",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 8});
    $r.addProperty("OnCanPlayThrough",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 9});
    $r.addProperty("OnChange",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 10});
    $r.addProperty("OnClick",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 11});
    $r.addProperty("OnCompositionEnd",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 12});
    $r.addProperty("OnCompositionStart",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 13});
    $r.addProperty("OnCompositionUpdate",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 14});
    $r.addProperty("OnContextMenu",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 15});
    $r.addProperty("OnCopy",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 16});
    $r.addProperty("OnCut",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 17});
    $r.addProperty("OnCueChange",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 18});
    $r.addProperty("OnDblClick",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 19});
    $r.addProperty("OnDurationChange",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 20});
    $r.addProperty("OnEnded",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 21});
    $r.addProperty("OnError",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 22});
    $r.addProperty("OnFocus",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 23});
    $r.addProperty("OnFocusIn",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 24});
    $r.addProperty("OnFocusOut",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 25});
    $r.addProperty("OnGotPointerCapture",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 26});
    $r.addProperty("OnInput",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 27});
    $r.addProperty("OnInvalid",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 28});
    $r.addProperty("OnKeyDown",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 29});
    $r.addProperty("OnKeyPress",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 30});
    $r.addProperty("OnKeyUp",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 31});
    $r.addProperty("OnLoad",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 32});
    $r.addProperty("OnLoadedData",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 33});
    $r.addProperty("OnLoadedMetaData",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 34});
    $r.addProperty("OnLoadend",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 35});
    $r.addProperty("OnLoadStart",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 36});
    $r.addProperty("OnLostPointerCapture",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 37});
    $r.addProperty("OnMouseDown",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 38});
    $r.addProperty("OnMouseEnter",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 39});
    $r.addProperty("OnMouseLeave",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 40});
    $r.addProperty("OnMouseMove",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 41});
    $r.addProperty("OnMouseOut",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 42});
    $r.addProperty("OnMouseUp",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 43});
    $r.addProperty("OnOverFlow",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 44});
    $r.addProperty("OnPaste",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 45});
    $r.addProperty("OnPause",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 46});
    $r.addProperty("OnPlay",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 47});
    $r.addProperty("OnPointerCancel",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 48});
    $r.addProperty("OnPointerDown",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 49});
    $r.addProperty("OnPointerEnter",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 50});
    $r.addProperty("OnPointerLeave",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 51});
    $r.addProperty("OnPointerMove",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 52});
    $r.addProperty("OnPointerOut",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 53});
    $r.addProperty("OnPointerOver",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 54});
    $r.addProperty("OnPointerUp",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 55});
    $r.addProperty("OnReset",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 56});
    $r.addProperty("OnResize",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 57});
    $r.addProperty("OnScroll",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 58});
    $r.addProperty("OnSelect",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 59});
    $r.addProperty("OnSubmit",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 60});
    $r.addProperty("OnTouchStart",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 61});
    $r.addProperty("OnTransitionCancel",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 62});
    $r.addProperty("OnTransitionEnd",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 63});
    $r.addProperty("OnTransitionRun",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 64});
    $r.addProperty("OnTransitionStart",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 65});
    $r.addProperty("OnWheel",19,pas.webwidget.$rtti["THTMLNotifyEvent"],"GetHTMLEvent","SetHTMLEvent",{index: 66});
  });
  rtl.createClass($mod,"TCustomInputWidget",pas.webwidget.TWebWidget,function () {
    this.$init = function () {
      pas.webwidget.TWebWidget.$init.call(this);
      this.FValue = "";
      this.FValueName = "";
      this.FText = "";
      this.FReadOnly = false;
      this.FRequired = false;
    };
    this.GetText = function () {
      var Result = "";
      var Inp = null;
      Inp = this.GetElement();
      if (Inp != null) {
        Result = Inp.innerText}
       else Result = this.FText;
      return Result;
    };
    this.GetValue = function () {
      var Result = "";
      var Inp = null;
      Inp = this.GetInputElement();
      if (Inp != null) {
        Result = Inp.value}
       else Result = this.FValue;
      return Result;
    };
    this.GetValueName = function () {
      var Result = "";
      var Inp = null;
      Inp = this.GetInputElement();
      if (Inp != null) {
        Result = Inp.name}
       else {
        Result = this.FValueName;
        if (Result === "") Result = this.FName;
      };
      return Result;
    };
    this.SetText = function (AValue) {
      var Inp = null;
      if (AValue === this.GetText()) return;
      this.FText = AValue;
      Inp = this.GetElement();
      if (Inp != null) Inp.innerText = AValue;
    };
    this.SetValue = function (AValue) {
      var Inp = null;
      if (AValue === this.GetValue()) return;
      this.FValue = AValue;
      Inp = this.GetInputElement();
      if (Inp != null) Inp.value = AValue;
    };
    this.GetInputElement = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.SetValueName = function (AValue) {
      var Inp = null;
      if (AValue === this.GetValueName()) return;
      this.FValueName = AValue;
      Inp = this.GetInputElement();
      if (Inp != null) Inp.name = AValue;
    };
    this.SetName = function (NewName) {
      var Old = "";
      Old = this.FName;
      pas.Classes.TComponent.SetName.call(this,NewName);
      if (this.GetValue() === Old) this.SetValue(NewName);
    };
    this.ApplyWidgetSettings = function (aElement) {
      pas.webwidget.TCustomWebWidget.ApplyWidgetSettings.apply(this,arguments);
      if (this.GetExternalElement() && (this.FValue === "")) {
        this.FValue = aElement.value}
       else {
        aElement.type = this.InputType();
        aElement.name = this.FValueName;
        aElement.value = this.FValue;
        aElement.required = this.FRequired;
        aElement.readOnly = this.FReadOnly;
      };
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "input";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  this.TInputTextType = {"0": "ittText", ittText: 0, "1": "ittPassword", ittPassword: 1, "2": "ittNumber", ittNumber: 2, "3": "ittEmail", ittEmail: 3, "4": "ittSearch", ittSearch: 4, "5": "ittTelephone", ittTelephone: 5, "6": "ittURL", ittURL: 6, "7": "ittColor", ittColor: 7};
  $mod.$rtti.$Enum("TInputTextType",{minvalue: 0, maxvalue: 7, ordtype: 1, enumtype: this.TInputTextType});
  rtl.createClass($mod,"TTextInputWidget",$mod.TCustomInputWidget,function () {
    this.$init = function () {
      $mod.TCustomInputWidget.$init.call(this);
      this.FMaxLength = 0;
      this.FMinLength = 0;
      this.FTextType = 0;
    };
    this.GetAsNumber = function () {
      var Result = 0;
      Result = pas.SysUtils.StrToIntDef(this.GetValue(),0);
      return Result;
    };
    this.GetMaxLength = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FMaxLength = this.GetInputElement().maxLength;
      Result = this.FMaxLength;
      return Result;
    };
    this.GetMinLength = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FMinLength = this.GetInputElement().minLength;
      Result = this.FMinLength;
      return Result;
    };
    this.GetTextType = function () {
      var Result = 0;
      Result = this.FTextType;
      return Result;
    };
    this.SetAsNumber = function (AValue) {
      this.SetValue(pas.SysUtils.IntToStr(AValue));
    };
    this.SetMaxLength = function (AValue) {
      if (AValue === this.FMaxLength) return;
      this.FMaxLength = AValue;
      if (this.GetRendered()) this.GetInputElement().maxLength = this.FMaxLength;
    };
    this.SetMinLength = function (AValue) {
      if (AValue === this.FMinLength) return;
      this.FMinLength = AValue;
      if (this.GetRendered()) this.GetInputElement().minLength = this.FMinLength;
    };
    this.SetTextType = function (AValue) {
      if (AValue === this.FTextType) return;
      this.FTextType = AValue;
      if (this.GetRendered()) this.Refresh();
    };
    this.ApplyWidgetSettings = function (aElement) {
      $mod.TCustomInputWidget.ApplyWidgetSettings.call(this,aElement);
      if (this.FMaxLength !== 0) aElement.maxLength = this.FMaxLength;
      if (this.FMinLength !== 0) aElement.minLength = this.FMinLength;
    };
    this.AllowChildren = function () {
      var Result = false;
      Result = false;
      return Result;
    };
    var Types = ["text","password","number","email","search","tel","url","color"];
    this.InputType = function () {
      var Result = "";
      Result = Types[this.FTextType];
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Value",3,rtl.string,"GetValue","SetValue");
    $r.addProperty("ValueName",3,rtl.string,"GetValueName","SetValueName");
    $r.addProperty("TextType",3,$mod.$rtti["TInputTextType"],"GetTextType","SetTextType");
    $r.addProperty("AsNumber",3,rtl.nativeint,"GetAsNumber","SetAsNumber");
    $r.addProperty("MaxLength",3,rtl.nativeint,"GetMaxLength","SetMaxLength");
    $r.addProperty("MinLength",3,rtl.nativeint,"GetMinLength","SetMinLength");
  });
  rtl.createClass($mod,"TCheckableInputWidget",$mod.TCustomInputWidget,function () {
    this.$init = function () {
      $mod.TCustomInputWidget.$init.call(this);
      this.FChecked = false;
    };
    this.GetChecked = function () {
      var Result = false;
      if (this.GetIsElementDirty()) this.FChecked = this.GetInputElement().checked;
      Result = this.FChecked;
      return Result;
    };
    this.SetChecked = function (AValue) {
      if (this.GetChecked() === AValue) return;
      if (this.GetRendered()) this.GetInputElement().checked = AValue;
      this.FChecked = AValue;
    };
    this.ApplyWidgetSettings = function (aElement) {
      $mod.TCustomInputWidget.ApplyWidgetSettings.call(this,aElement);
      aElement.checked = this.FChecked;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($mod,"TRadioInputWidget",$mod.TCheckableInputWidget,function () {
    this.InputType = function () {
      var Result = "";
      Result = "radio";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Value",3,rtl.string,"GetValue","SetValue");
    $r.addProperty("ValueName",3,rtl.string,"GetValueName","SetValueName");
    $r.addProperty("Checked",3,rtl.boolean,"GetChecked","SetChecked");
    $r.addProperty("Text",3,rtl.string,"GetText","SetText");
  });
  rtl.createClass($mod,"TCheckboxInputWidget",$mod.TCheckableInputWidget,function () {
    this.InputType = function () {
      var Result = "";
      Result = "checkbox";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Value",3,rtl.string,"GetValue","SetValue");
    $r.addProperty("ValueName",3,rtl.string,"GetValueName","SetValueName");
    $r.addProperty("Checked",3,rtl.boolean,"GetChecked","SetChecked");
    $r.addProperty("Text",3,rtl.string,"GetText","SetText");
  });
  this.TTextAreaWrap = {"0": "tawSoft", tawSoft: 0, "1": "tawHard", tawHard: 1, "2": "tawOff", tawOff: 2};
  $mod.$rtti.$Enum("TTextAreaWrap",{minvalue: 0, maxvalue: 2, ordtype: 1, enumtype: this.TTextAreaWrap});
  rtl.createClass($mod,"TTextAreaWidget",pas.webwidget.TWebWidget,function () {
    this.$init = function () {
      pas.webwidget.TWebWidget.$init.call(this);
      this.FLines = null;
      this.FIgnoreChanges = false;
      this.FMaxLength = 0;
      this.FValueName = "";
      this.FRows = 0;
      this.FColumns = 0;
      this.FWrap = 0;
      this.FRequired = false;
      this.FReadOnly = false;
    };
    this.$final = function () {
      this.FLines = undefined;
      pas.webwidget.TWebWidget.$final.call(this);
    };
    var Wraps = ["soft","hard","off"];
    this.ApplyWrap = function (aElement) {
      aElement.wrap = Wraps[this.FWrap];
    };
    this.DoLineChanges = function (Sender) {
      if (this.GetRendered() && !this.FIgnoreChanges) this.ApplyLines(this.GetTextArea());
    };
    this.GetColumns = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FColumns = this.GetTextArea().cols;
      Result = this.FColumns;
      return Result;
    };
    this.GetLines = function () {
      var Result = null;
      if (this.GetIsElementDirty()) {
        this.FIgnoreChanges = true;
        try {
          this.LinesFromHTML(this.GetElement().innerHTML);
        } finally {
          this.FIgnoreChanges = false;
        };
      };
      Result = this.FLines;
      return Result;
    };
    this.GetReadOnly = function () {
      var Result = false;
      if (this.GetIsElementDirty()) this.FReadOnly = this.GetTextArea().readOnly;
      Result = this.FReadOnly;
      return Result;
    };
    this.GetRequired = function () {
      var Result = false;
      if (this.GetIsElementDirty()) this.FRequired = this.GetTextArea().required;
      Result = this.FRequired;
      return Result;
    };
    this.GetRows = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FRows = this.GetTextArea().rows;
      Result = this.FRows;
      return Result;
    };
    this.GetValueName = function () {
      var Result = "";
      if (this.GetIsElementDirty()) this.FValueName = this.GetElement().name;
      Result = this.FValueName;
      return Result;
    };
    this.SetColumns = function (AValue) {
      if (AValue === this.FColumns) return;
      this.FColumns = AValue;
      if (this.GetRendered()) this.GetTextArea().cols = AValue;
    };
    this.SetLines = function (AValue) {
      if (this.FLines === AValue) return;
      this.FLines.Assign(AValue);
    };
    this.SetMaxLength = function (AValue) {
      if (this.FMaxLength === AValue) return;
      this.FMaxLength = AValue;
      if (this.GetRendered()) this.GetTextArea().maxLength = AValue;
    };
    this.SetReadonly = function (AValue) {
      if (AValue === this.GetReadOnly()) return;
      this.FReadOnly = AValue;
      if (this.GetRendered()) this.GetTextArea().readOnly = this.FReadOnly;
    };
    this.SetRequired = function (AValue) {
      if (AValue === this.GetRequired()) return;
      this.FRequired = AValue;
      if (this.GetRendered()) this.GetTextArea().required = this.FRequired;
    };
    this.SetRows = function (AValue) {
      if (AValue === this.FRows) return;
      this.FRows = AValue;
      if (this.GetRendered()) this.GetTextArea().rows = AValue;
    };
    this.SetValueName = function (AValue) {
      if (AValue === this.FValueName) return;
      this.FValueName = AValue;
      if (this.GetRendered()) this.GetTextArea().name = AValue;
    };
    this.GetTextArea = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.SetWrap = function (AValue) {
      if (this.FWrap === AValue) return;
      this.FWrap = AValue;
      if (this.GetRendered()) this.ApplyWrap(this.GetTextArea());
    };
    this.ApplyLines = function (aElement) {
      aElement.innerHTML = this.FLines.GetTextStr();
    };
    this.LinesFromHTML = function (aHTML) {
      this.FLines.SetTextStr(pas.SysUtils.StringReplace(aHTML,"<br>",pas.System.sLineBreak,rtl.createSet(1,0)));
    };
    this.SetName = function (NewName) {
      var Old = "";
      Old = this.FName;
      pas.Classes.TComponent.SetName.call(this,NewName);
      if (4 in this.FComponentState) {
        if (this.FLines.GetCount() === 0) {
          this.FLines.Add(this.FName)}
         else if ((this.FLines.GetCount() === 1) && (this.FLines.Get(0) === Old)) this.FLines.Put(0,this.FName);
      };
    };
    this.ApplyWidgetSettings = function (aElement) {
      pas.webwidget.TCustomWebWidget.ApplyWidgetSettings.call(this,aElement);
      if (this.FMaxLength > 0) aElement.maxLength = this.FMaxLength;
      if (this.FColumns > 0) aElement.cols = this.FColumns;
      if (this.FRows > 0) aElement.rows = this.FRows;
      if (this.FLines.GetCount() > 0) this.ApplyLines(aElement);
      if (this.FValueName !== "") aElement.name = this.FValueName;
      aElement.readOnly = this.FReadOnly;
      aElement.required = this.FRequired;
      this.ApplyWrap(aElement);
    };
    this.Create$1 = function (aOwner) {
      pas.webwidget.TCustomWebWidget.Create$1.call(this,aOwner);
      this.FLines = pas.Classes.TStringList.$create("Create$1");
      this.FLines.FOnChange = rtl.createCallback(this,"DoLineChanges");
      this.FColumns = 50;
      this.FRows = 10;
      return this;
    };
    this.Destroy = function () {
      pas.SysUtils.FreeAndNil({p: this, get: function () {
          return this.p.FLines;
        }, set: function (v) {
          this.p.FLines = v;
        }});
      pas.webwidget.TCustomWebWidget.Destroy.call(this);
    };
    this.AllowChildren = function () {
      var Result = false;
      Result = false;
      return Result;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "textarea";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("ValueName",3,rtl.string,"GetValueName","SetValueName");
    $r.addProperty("Rows",3,rtl.longword,"GetRows","SetRows");
    $r.addProperty("Columns",3,rtl.longword,"GetColumns","SetColumns");
    $r.addProperty("Lines",3,pas.Classes.$rtti["TStrings"],"GetLines","SetLines");
    $r.addProperty("MaxLength",2,rtl.longword,"FMaxLength","SetMaxLength");
    $r.addProperty("Wrap",2,$mod.$rtti["TTextAreaWrap"],"FWrap","SetWrap");
    $r.addProperty("ReadOnly",3,rtl.boolean,"GetReadOnly","SetReadonly");
    $r.addProperty("Required",3,rtl.boolean,"GetRequired","SetRequired");
  });
  rtl.createClass($mod,"TImageWidget",pas.webwidget.TWebWidget,function () {
    this.$init = function () {
      pas.webwidget.TWebWidget.$init.call(this);
      this.FHeight = 0;
      this.FWidth = 0;
      this.FSrc = "";
    };
    this.GetHeight = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FHeight = this.GetImg().height;
      Result = this.FHeight;
      return Result;
    };
    this.GetImg = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.GetSrc = function () {
      var Result = "";
      if (this.GetIsElementDirty()) this.FSrc = this.GetImg().src;
      Result = this.FSrc;
      return Result;
    };
    this.GetWidth = function () {
      var Result = 0;
      if (this.GetIsElementDirty()) this.FWidth = this.GetImg().width;
      Result = this.FWidth;
      return Result;
    };
    this.SetHeight = function (AValue) {
      if (AValue === this.GetHeight()) return;
      this.FHeight = AValue;
      if (this.GetRendered()) this.GetImg().height = AValue;
    };
    this.SetSrc = function (AValue) {
      if (AValue === this.GetSrc()) return;
      this.FSrc = AValue;
      if (this.GetRendered()) this.GetImg().src = this.FSrc;
    };
    this.SetWidth = function (AValue) {
      if (AValue === this.GetWidth()) return;
      this.FWidth = AValue;
      if (this.GetRendered()) this.GetImg().width = AValue;
    };
    this.ApplyWidgetSettings = function (aElement) {
      pas.webwidget.TCustomWebWidget.ApplyWidgetSettings.call(this,aElement);
      aElement.src = this.FSrc;
      aElement.height = this.FHeight;
      aElement.width = this.FWidth;
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "img";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Src",3,rtl.string,"GetSrc","SetSrc");
    $r.addProperty("Width",3,rtl.longint,"GetWidth","SetWidth");
    $r.addProperty("Height",3,rtl.longint,"GetHeight","SetHeight");
  });
  rtl.createClass($mod,"TSelectWidget",pas.webwidget.TWebWidget,function () {
    this.$init = function () {
      pas.webwidget.TWebWidget.$init.call(this);
      this.FItems = null;
      this.FValues = null;
      this.FOptions = [];
      this.FSelectedIndex = 0;
      this.FMultiple = false;
    };
    this.$final = function () {
      this.FItems = undefined;
      this.FValues = undefined;
      this.FOptions = undefined;
      pas.webwidget.TWebWidget.$final.call(this);
    };
    this.GetMultiple = function () {
      var Result = false;
      if (this.GetIsElementDirty()) this.FMultiple = this.GetSelect().multiple;
      Result = this.FMultiple;
      return Result;
    };
    this.GetSelectedIndex = function () {
      var Result = 0;
      if (this.GetRendered()) this.FSelectedIndex = this.GetSelect().selectedIndex;
      Result = this.FSelectedIndex;
      return Result;
    };
    this.GetItems = function () {
      var Result = null;
      Result = this.FItems;
      return Result;
    };
    this.GetSelect = function () {
      var Result = null;
      Result = this.GetElement();
      return Result;
    };
    this.GetValues = function () {
      var Result = null;
      Result = this.FValues;
      return Result;
    };
    this.OptionsChanged = function (Sender) {
      if (this.GetRendered()) this.BuildOptions(this.GetSelect());
    };
    this.SetMultiple = function (AValue) {
      if (AValue === this.GetMultiple()) return;
      this.FMultiple = AValue;
      if (this.GetRendered()) this.GetSelect().multiple = this.FMultiple;
    };
    this.SetSelectedIndex = function (AValue) {
      if (this.GetSelectedIndex() === AValue) return;
      this.FSelectedIndex = AValue;
      if (this.GetRendered()) this.GetSelect().selectedIndex = this.FSelectedIndex;
    };
    this.setItems = function (AValue) {
      if (AValue === this.FItems) return;
      this.FItems.Assign(AValue);
    };
    this.setValues = function (AValue) {
      if (AValue === this.FValues) return;
      this.FValues.Assign(AValue);
    };
    this.BuildOptions = function (aSelect) {
      var O = null;
      var Itms = null;
      var Vals = null;
      var I = 0;
      var Idx = 0;
      this.FOptions = rtl.arraySetLength(this.FOptions,null,0);
      aSelect.innerHTML = "";
      Itms = this.FItems;
      Vals = this.FValues;
      Idx = this.FSelectedIndex;
      this.FOptions = rtl.arraySetLength(this.FOptions,null,Itms.GetCount());
      for (var $l1 = 0, $end2 = Itms.GetCount() - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        O = this.$class.CreateElement("option","");
        this.FOptions[I] = O;
        O.innerText = Itms.Get(I);
        if (I < Vals.GetCount()) O.value = Vals.Get(I);
        if (I === Idx) O.selected = true;
        aSelect.appendChild(O);
      };
    };
    this.ApplyWidgetSettings = function (aElement) {
      pas.webwidget.TCustomWebWidget.ApplyWidgetSettings.call(this,aElement);
      this.BuildOptions(aElement);
    };
    this.Create$1 = function (aOWner) {
      pas.webwidget.TCustomWebWidget.Create$1.call(this,aOWner);
      this.FItems = pas.Classes.TStringList.$create("Create$1");
      this.FItems.FOnChange = rtl.createCallback(this,"OptionsChanged");
      this.FValues = pas.Classes.TStringList.$create("Create$1");
      this.FValues.FOnChange = rtl.createCallback(this,"OptionsChanged");
      return this;
    };
    this.Destroy = function () {
      pas.webwidget.TCustomWebWidget.Destroy.call(this);
    };
    this.HTMLTag = function () {
      var Result = "";
      Result = "select";
      return Result;
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Items",3,pas.Classes.$rtti["TStrings"],"GetItems","setItems");
    $r.addProperty("Values",3,pas.Classes.$rtti["TStrings"],"GetValues","setValues");
    $r.addProperty("SelectedIndex",3,rtl.longint,"GetSelectedIndex","SetSelectedIndex");
    $r.addProperty("Multiple",3,rtl.boolean,"GetMultiple","SetMultiple");
  });
},[]);
rtl.module("contnrs",["System","SysUtils","Classes"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"TObjectList",pas.Classes.TList,function () {
    this.$init = function () {
      pas.Classes.TList.$init.call(this);
      this.FFreeObjects = false;
    };
    this.Notify = function (Ptr, Action) {
      var O = null;
      if (this.FFreeObjects) if (Action === 2) {
        O = rtl.getObject(Ptr);
        O = rtl.freeLoc(O);
      };
      pas.Classes.TList.Notify.call(this,Ptr,Action);
    };
    this.GetItem = function (Index) {
      var Result = null;
      Result = rtl.getObject(pas.Classes.TList.Get.call(this,Index));
      return Result;
    };
    this.Create$2 = function () {
      pas.Classes.TList.Create$1.call(this);
      this.FFreeObjects = true;
      return this;
    };
    this.Add$1 = function (AObject) {
      var Result = 0;
      Result = pas.Classes.TList.Add.call(this,AObject);
      return Result;
    };
  });
},["JS"]);
rtl.module("webideclient",["System","Classes","SysUtils","JS","Web"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"TIDERequest",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FXHR = null;
      this.FOnResponse = null;
    };
    this.$final = function () {
      this.FXHR = undefined;
      this.FOnResponse = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.ProcessResponse = function () {
      var P = null;
      if ((Math.floor(this.FXHR.status / 100) === 2) && (this.FXHR.getResponseHeader("Content-Type") === "application\/json")) {
        P = JSON.parse(this.FXHR.responseText)}
       else P = null;
      if (this.FOnResponse != null) this.FOnResponse(this.FXHR.status,this.FXHR.statusText,P);
    };
    this.DoStateChange = function () {
      var $tmp1 = this.FXHR.readyState;
      if ($tmp1 === 4) {
        if (this.FOnResponse != null) this.ProcessResponse();
        this.Free();
      };
    };
    this.Create$1 = function (aMethod, aURl, aPayLoad, aOnResponse) {
      var S = "";
      this.FOnResponse = aOnResponse;
      this.FXHR = new XMLHttpRequest();
      this.FXHR.open(aMethod,aURl);
      if (aPayLoad != null) {
        S = JSON.stringify(aPayLoad)}
       else S = "";
      this.FXHR.setRequestHeader("Content-Type","application\/json");
      this.FXHR.onreadystatechange = rtl.createCallback(this,"DoStateChange");
      this.FXHR.send(S);
      return this;
    };
  });
  rtl.createClass($mod,"TIDEClient",pas.Classes.TComponent,function () {
    this.$init = function () {
      pas.Classes.TComponent.$init.call(this);
      this.FActionID = 0;
      this.FOnActionResponse = null;
      this.FClientID = 0;
      this.FIDEURL = "";
      this.FLastPoll = null;
    };
    this.$final = function () {
      this.FOnActionResponse = undefined;
      this.FLastPoll = undefined;
      pas.Classes.TComponent.$final.call(this);
    };
    this.OnActionSent = function (aCode, aCodeText, aPayload) {
      var aID = 0;
      var aName = "";
      var aActionPayload = null;
      if (Math.floor(aCode / 100) === 2) {
        aID = Math.floor(aPayload["id"]);
        aName = "" + aPayload["name"];
        aActionPayload = rtl.getObject(aPayload["payload"]);
        if (this.FOnActionResponse != null) this.FOnActionResponse(this,aID,aName,aActionPayload);
      };
    };
    this.Create$1 = function (aOwner) {
      pas.Classes.TComponent.Create$1.apply(this,arguments);
      this.FLastPoll = null;
      this.FIDEURL = "http:\/\/" + window.location.hostname + ":" + window.location.port + "\/IDE\/";
      return this;
    };
    this.GetNextID = function () {
      var Result = 0;
      this.FActionID += 1;
      Result = this.FActionID;
      return Result;
    };
    this.SendAction = function (aName, aPayLoad) {
      var aAction = null;
      var aID = 0;
      var req = null;
      aID = this.GetNextID();
      aAction = pas.JS.New(["id",aID,"name",aName,"payload",aPayLoad]);
      req = $mod.TIDERequest.$create("Create$1",["POST",this.FIDEURL + "Action\/" + pas.SysUtils.IntToStr(this.FClientID) + "\/" + pas.SysUtils.IntToStr(aID),aAction,rtl.createCallback(this,"OnActionSent")]);
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
});
rtl.module("designer",["System","Classes","SysUtils","libjquery","webwidget","htmlwidgets","contnrs","JS","Web","webideclient"],function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  rtl.createClass($mod,"TRegisteredWidget",pas.System.TObject,function () {
    this.$init = function () {
      pas.System.TObject.$init.call(this);
      this.FClass = null;
      this.FImageName = "";
    };
    this.$final = function () {
      this.FClass = undefined;
      pas.System.TObject.$final.call(this);
    };
    this.Create$1 = function (aClass, aImageName) {
      this.FClass = aClass;
      this.FImageName = aImageName;
      return this;
    };
  });
  rtl.createClass($mod,"TWidgetButtonWidget",pas.htmlwidgets.TButtonWidget,function () {
    this.$init = function () {
      pas.htmlwidgets.TButtonWidget.$init.call(this);
      this.MyWidget = null;
    };
    this.$final = function () {
      this.MyWidget = undefined;
      pas.htmlwidgets.TButtonWidget.$final.call(this);
    };
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addField("MyWidget",$mod.$rtti["TRegisteredWidget"]);
  });
  rtl.createHelper($mod,"TSortable",null,function () {
  });
  rtl.createClass($mod,"TDesignDemo",pas.Classes.TComponent,function () {
    this.$init = function () {
      pas.Classes.TComponent.$init.call(this);
      this.FIDEClient = null;
      this.FAddWidget = null;
      this.FPage = null;
      this.FToolBar = null;
      this.FButtons = rtl.arraySetLength(null,null,10);
      this.FWidgetButtons = [];
      this.FRegisteredWidgets = null;
    };
    this.$final = function () {
      this.FIDEClient = undefined;
      this.FAddWidget = undefined;
      this.FPage = undefined;
      this.FToolBar = undefined;
      this.FButtons = undefined;
      this.FWidgetButtons = undefined;
      this.FRegisteredWidgets = undefined;
      pas.Classes.TComponent.$final.call(this);
    };
    this.CreateNewWidget = function (aParent, aClass) {
      var Result = null;
      Result = aClass.$create("Create$1",[this.FPage]);
      Result.SetName(Result.$classname + pas.SysUtils.IntToStr(this.FPage.GetChildCount()));
      Result.SetParent(aParent);
      Result.Refresh();
      if (this.FIDEClient != null) this.FIDEClient.SendAction("create",pas.JS.New(["widget",Result.FName,"class",Result.$classname]));
      return Result;
    };
    var Toolbar = '<div class="designerToolbar">' + '<div class="designerDragHandle ui-icon ui-icon-arrow-4"><\/div>' + '<div class="designerDelete ui-icon ui-icon-trash"><\/div>' + "<\/div>";
    this.DoActive = function (Event) {
      var Result = false;
      var aParent = null;
      var aNew = null;
      var aNewActive = null;
      var aNewWidget = null;
      Result = true;
      jQuery(".designerActive").removeClass("designerActive");
      jQuery(".designerToolbar").remove();
      aNewActive = Event.target;
      aParent = this.FPage.FindWidgetByID("" + aNewActive.dataset[pas.webwidget.STopElementData],true);
      if ((this.FAddWidget !== null) && (aParent !== null)) {
        aNewWidget = this.FAddWidget;
        this.FAddWidget = null;
        if (aParent !== null) {
          aNew = this.CreateNewWidget(aParent,aNewWidget.FClass);
          aNewActive = aNew.GetTopElement();
          if (aNew.GetContentElement() != null) jQuery(aNew.GetContentElement()).sortable(this.SortableOptions());
          jQuery(aNewActive).on("click",rtl.createCallback(this,"DoActive"));
          aParent = aNew;
        };
      };
      jQuery(aNewActive).addClass("designerActive").prepend(Toolbar);
      if ((aParent != null) && (this.FIDEClient != null)) this.FIDEClient.SendAction("select",pas.JS.New(["widget",aParent.FName,"class",aParent.$classname,"state",this.StreamWidget(aParent)]));
      return Result;
    };
    this.DoWidgetAddClick = function (Sender, Event) {
      this.SetAddMode(rtl.as(Sender,$mod.TWidgetButtonWidget).MyWidget);
    };
    this.SortableOptions = function () {
      var Result = null;
      Result = pas.JS.New(["items","> [data-ww-element-top]","connectWith","[data-ww-element-content]","placeholder","designerPlaceholder","tolerance","pointer","cancel",""]);
      return Result;
    };
    this.StreamWidget = function (aWidget) {
      var Result = "";
      var S = null;
      var T = null;
      T = null;
      S = pas.Classes.TBytesStream.$create("Create$1",[[]]);
      try {
        S.WriteComponent(aWidget);
        S.SetPosition(0);
        T = pas.Classes.TStringStream.$create("Create$1",[""]);
        pas.Classes.ObjectBinaryToText(S,T);
        Result = T.GetDataString();
      } finally {
        T = rtl.freeLoc(T);
        S = rtl.freeLoc(S);
      };
      return Result;
    };
    this.Create$1 = function (aOwner) {
      pas.Classes.TComponent.Create$1.call(this,aOwner);
      this.FRegisteredWidgets = pas.contnrs.TObjectList.$create("Create$2");
      this.RegisterWidgets();
      this.FillToolBar();
      this.SetupPage();
      jQuery($impl.SSortableSelect).sortable(this.SortableOptions());
      jQuery("#designpage").on("click","[data-ww-element-top]",rtl.createCallback(this,"DoActive"));
      jQuery("#designpage").on("click",rtl.createCallback(this,"DoActive"));
      return this;
    };
    this.Destroy = function () {
      pas.SysUtils.FreeAndNil({p: this, get: function () {
          return this.p.FRegisteredWidgets;
        }, set: function (v) {
          this.p.FRegisteredWidgets = v;
        }});
      pas.Classes.TComponent.Destroy.call(this);
    };
    this.RegisterWidgets = function () {
      this.RegisterWidget(pas.htmlwidgets.TButtonWidget,"button");
      this.RegisterWidget(pas.htmlwidgets.TCheckboxInputWidget,"checkbox");
      this.RegisterWidget(pas.htmlwidgets.TRadioInputWidget,"radio");
      this.RegisterWidget(pas.htmlwidgets.TTextInputWidget,"edit");
      this.RegisterWidget(pas.htmlwidgets.TImageWidget,"image");
      this.RegisterWidget(pas.htmlwidgets.TTextAreaWidget,"memo");
      this.RegisterWidget(pas.htmlwidgets.TSelectWidget,"select");
      this.RegisterWidget(pas.webwidget.TContainerWidget,"container");
      this.RegisterWidget($impl.TJumboWidget,"jumbo");
    };
    this.RegisterWidget = function (aClass, aImageName) {
      this.FRegisteredWidgets.Add$1($mod.TRegisteredWidget.$create("Create$1",[aClass,aImageName]));
    };
    this.SetAddMode = function (aRegisteredWidget) {
      this.FAddWidget = aRegisteredWidget;
    };
    this.FillToolBar = function () {
      var RW = null;
      var I = 0;
      var Btn = null;
      this.FToolBar = pas.webwidget.TContainerWidget.$create("Create$1",[this]);
      this.FToolBar.FStyles.EnsureStyle("min-height","34px");
      this.FToolBar.FStyles.RemoveStyle("width");
      this.FToolBar.FStyles.RemoveStyle("height");
      this.FToolBar.SetElementID("toolbar");
      this.FToolBar.Refresh();
      this.FWidgetButtons = rtl.arraySetLength(this.FWidgetButtons,null,this.FRegisteredWidgets.GetCount());
      for (var $l1 = 0, $end2 = this.FRegisteredWidgets.GetCount() - 1; $l1 <= $end2; $l1++) {
        I = $l1;
        RW = this.FRegisteredWidgets.GetItem(I);
        Btn = $mod.TWidgetButtonWidget.$create("Create$1",[this]);
        this.FWidgetButtons[I] = Btn;
        Btn.MyWidget = RW;
        Btn.SetClasses("btn btn-light");
        Btn.SetText("");
        Btn.SetParent(this.FToolBar);
        Btn.FStyles.Add$1("background-image",'url("widgets\/' + RW.FImageName + '.png")');
        Btn.Refresh();
        Btn.SetDataset("widgetClass",RW.FClass.$classname);
        Btn.SetHTMLEvent(11,rtl.createCallback(this,"DoWidgetAddClick"));
      };
    };
    var ButtonClasses = ["primary","secondary","success","danger","warning","info","light","dark","link"];
    this.SetupPage = function () {
      var I = 0;
      this.FPage = pas.htmlwidgets.TWebPage.$create("Create$1",[this]);
      this.FPage.SetElementID("designpage");
      this.FPage.Refresh();
      for (I = 0; I <= 9; I++) {
        this.FButtons[I - 1] = $mod.TWidgetButtonWidget.$create("Create$1",[this.FPage]);
        this.FButtons[I - 1].SetClasses("btn btn-" + ButtonClasses[I % 9]);
        this.FButtons[I - 1].SetText("Button #" + pas.SysUtils.IntToStr(I + 1));
        this.FButtons[I - 1].SetParent(this.FPage);
        this.FButtons[I - 1].Refresh();
      };
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
},null,function () {
  "use strict";
  var $mod = this;
  var $impl = $mod.$impl;
  $impl.SSortableSelect = "#designpage, #designpage [data-ww-element-content]";
  rtl.createClass($impl,"TJumboWidget",pas.webwidget.TCustomTemplateWidget,function () {
    this.Create$1 = function (aOwner) {
      pas.webwidget.TCustomTemplateWidget.Create$1.call(this,aOwner);
      this.FTemplate.SetTextStr('<div class="jumbotron">' + pas.System.sLineBreak + '<h1 class="display-4">Hello, world!<\/h1>' + pas.System.sLineBreak + '<p class="lead">This is a simple hero unit, a simple jumbotron-style component for calling extra attention to featured content or information.<\/p>' + pas.System.sLineBreak + '<hr class="my-4">' + pas.System.sLineBreak + "<p>It uses utility classes for typography and spacing to space content out within the larger container.<\/p>" + pas.System.sLineBreak + '<p class="lead">' + pas.System.sLineBreak + '  <a class="btn btn-primary btn-lg" href="#" role="button">Learn more<\/a>' + pas.System.sLineBreak + "<\/p>" + pas.System.sLineBreak + "<\/div>");
      return this;
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  rtl.createClass($impl,"TWidgetHack",pas.webwidget.TCustomWebWidget,function () {
    rtl.addIntf(this,pas.System.IUnknown);
    var $r = this.$rtti;
    $r.addProperty("Element",1,pas.Web.$rtti["TJSHTMLElement"],"GetElement","");
    $r.addProperty("ElementID",3,rtl.string,"GetElementID","SetElementID");
    $r.addProperty("TopElement",1,pas.Web.$rtti["TJSHTMLElement"],"GetTopElement","");
    $r.addProperty("ContentElement",1,pas.Web.$rtti["TJSHTMLElement"],"GetContentElement","");
  });
});
rtl.module("program",["System","browserapp","JS","Classes","SysUtils","Web","designer","webideclient"],function () {
  "use strict";
  var $mod = this;
  rtl.createClass($mod,"TMyApplication",pas.browserapp.TBrowserApplication,function () {
    this.$init = function () {
      pas.browserapp.TBrowserApplication.$init.call(this);
      this.FDemo = null;
    };
    this.$final = function () {
      this.FDemo = undefined;
      pas.browserapp.TBrowserApplication.$final.call(this);
    };
    this.DoRun = function () {
      this.FDemo = pas.designer.TDesignDemo.$create("Create$1",[this]);
    };
    rtl.addIntf(this,pas.System.IUnknown);
  });
  this.Application = null;
  $mod.$main = function () {
    $mod.Application = $mod.TMyApplication.$create("Create$1",[null]);
    $mod.Application.Initialize();
    $mod.Application.Run();
  };
});
//# sourceMappingURL=designdemo.js.map
